/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.web.rs;

import java.util.List;

import org.codehaus.jackson.annotate.JsonCreator;
import org.codehaus.jackson.annotate.JsonProperty;

/**
 * Data object. Holds a list of {@link Fingerprint}s associated to audio entity
 * identifier. Audio entity represents some uniquely identifiable audio clip.
 * Audio entity could be for example MusicBrainz Track ID.
 */
public class AudioEntity {

	private final List<Fingerprint> fingerprints;

	private final String identifier;

	@JsonCreator
	public AudioEntity(@JsonProperty("identifier") String identifier,
			@JsonProperty("fingerprints") List<Fingerprint> fingerprints) {
		this.identifier = identifier;
		this.fingerprints = fingerprints;
	}

	public List<Fingerprint> getFingerprints() {
		return fingerprints;
	}

	public String getIdentifier() {
		return identifier;
	}

	@Override
	public String toString() {
		return AudioEntity.class.getSimpleName() + " [identifier=" + identifier
				+ ", fingerprints=" + fingerprints + "]";
	}
}
