/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.web.rs;

import java.util.List;

import org.codehaus.jackson.annotate.JsonCreator;
import org.codehaus.jackson.annotate.JsonProperty;

/**
 * Data object. Each fingerprint has an unique identifier, an audio entity
 * identifier and a list of segments representing parts of the original audio
 * data.
 */
public class Fingerprint {

	private final long approvals;

	private final String audioEntityId;

	private final String identifier;

	private final List<Repeat> repeats;

	private final List<Segment> segments;

	@JsonCreator
	public Fingerprint(@JsonProperty("identifier") String identifier,
			@JsonProperty("audioEntityId") String audioEntityId,
			@JsonProperty("segments") List<Segment> segments,
			@JsonProperty("repeats") List<Repeat> repeats,
			@JsonProperty("approvals") long approvals) {
		this.identifier = identifier;
		this.audioEntityId = audioEntityId;
		this.segments = segments;
		this.repeats = repeats;
		this.approvals = approvals;
	}

	public long getApprovals() {
		return approvals;
	}

	public String getAudioEntityId() {
		return audioEntityId;
	}

	public String getIdentifier() {
		return identifier;
	}

	public List<Repeat> getRepeats() {
		return repeats;
	}

	public List<Segment> getSegments() {
		return segments;
	}

	public String toShortString() {
		return "[audioEntityId=" + audioEntityId + ", identifier=" + identifier
				+ ", approvals=" + approvals + ", repeats=" + repeats.size()
				+ ", segments=" + segments.size() + "]";
	}

	@Override
	public String toString() {
		return Fingerprint.class.getSimpleName() + " [audioEntityId="
				+ audioEntityId + ", identifier=" + identifier + ", approvals="
				+ approvals + ", repeats=" + repeats + ", segments=" + segments
				+ "]";
	}

}
