/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.web.rs;

import java.io.Serializable;
import java.util.List;

import org.codehaus.jackson.annotate.JsonCreator;
import org.codehaus.jackson.annotate.JsonProperty;

/**
 * Represents cluster of {@link RepeatSearchEntry}s.
 */
public class RepeatCluster implements Serializable {

	private static final long serialVersionUID = -3162211661385411864L;

	private final int relativePosition;

	private final List<Repeat> repeatSearchEntries;

	/**
	 * 
	 * @param repeatSearchEntries
	 * @param relativePosition
	 *            Position relative to a query element position. Calculated as
	 *            <code>queryRepeat.getPosition() - cacheRepeat.getPosition()</code>
	 *            . Average of clustered elements.
	 */
	@JsonCreator
	public RepeatCluster(
			@JsonProperty("repeatSearchEntries") List<Repeat> repeatSearchEntries,
			@JsonProperty("relativePosition") int relativePosition) {
		this.repeatSearchEntries = repeatSearchEntries;
		this.relativePosition = relativePosition;
	}

	/**
	 * Position relative to a query element position. Calculated as
	 * <code>queryRepeat.getPosition() - cacheRepeat.getPosition()</code> .
	 * Average of clustered elements.
	 * 
	 * @return
	 */
	public int getRelativePosition() {
		return relativePosition;
	}

	public List<Repeat> getRepeatSearchEntries() {
		return repeatSearchEntries;
	}

	@Override
	public String toString() {
		return RepeatCluster.class.getSimpleName() + " [relativePosition="
				+ relativePosition + ", repeatSearchEntries="
				+ repeatSearchEntries + "]";
	}

}
