/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.web.rs;

import java.io.Serializable;
import java.util.List;

import org.codehaus.jackson.annotate.JsonCreator;
import org.codehaus.jackson.annotate.JsonProperty;

/**
 * Represents a cluster of {@link Segment}s.
 */
public class SegmentCluster implements Serializable {

	private static final long serialVersionUID = 1759204301365421068L;

	private final List<Segment> querySegments;

	private final int relativePosition;

	/**
	 * 
	 * @param position
	 *            Relative match position. Calculated as
	 *            <code>querySegment.getPosition() - cacheSegment.getPosition()</code>
	 *            . Average of clustered segment relative positions.
	 * @param score
	 * @param querySegments
	 */
	@JsonCreator
	public SegmentCluster(
			@JsonProperty("relativePosition") int relativePosition,
			@JsonProperty("querySegments") List<Segment> querySegments) {
		this.relativePosition = relativePosition;
		this.querySegments = querySegments;
	}

	public List<Segment> getQuerySegments() {
		return querySegments;
	}

	/**
	 * Relative match position. Calculated as
	 * <code>querySegment.getPosition() - cacheSegment.getPosition()</code>.
	 * Average of clustered segment relative positions.
	 * 
	 * @return
	 */
	public int getRelativePosition() {
		return relativePosition;
	}

	@Override
	public String toString() {
		return SegmentCluster.class.getSimpleName() + " [relativePosition="
				+ relativePosition + ", querySegments=" + querySegments + "]";
	}

}
