/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.web.rs;

import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.Assert;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.junit.Test;

public class SearchResultTest {

	protected <T> T fromJson(String jsonData, Class<T> valueType)
			throws JsonParseException, JsonMappingException, IOException {
		return new ObjectMapper().<T> readValue(jsonData, valueType);
	}

	protected <T> T fromJson(String jsonData, TypeReference<T> valueType)
			throws JsonParseException, JsonMappingException, IOException {
		return new ObjectMapper().<T> readValue(jsonData, valueType);
	}

	private Match getMatch() {
		String fpid = "fpid";
		List<Segment> segments = getSegmentList();
		List<SegmentCluster> segmentClusters = getSegmentClusters();
		List<RepeatCluster> repeatClusters = getRepeatClusters();
		List<Repeat> repeats = getRepeats();
		return new Match(fpid, segmentClusters, segments, repeatClusters,
				repeats);
	}

	private List<SegmentCluster> getSegmentClusters() {
		ArrayList<SegmentCluster> list = new ArrayList<SegmentCluster>();
		list.add(getSegmentCluster());
		return list;
	}

	private List<RepeatCluster> getRepeatClusters() {
		ArrayList<RepeatCluster> list = new ArrayList<RepeatCluster>();
		list.add(getRepeatCluster());
		return list;
	}

	private RepeatCluster getRepeatCluster() {
		List<Repeat> repeatSearchEntries = getRepeats();
		int relativePosition = 0;
		return new RepeatCluster(repeatSearchEntries, relativePosition);
	}

	private List<Repeat> getRepeats() {
		List<Repeat> list = new ArrayList<Repeat>();
		list.add(getRepeat());
		return list;
	}

	private Repeat getRepeat() {
		return new Repeat('9', 2, 4);
	}

	private Set<Match> getMatchSet() {
		Set<Match> matchSet = new HashSet<Match>();
		matchSet.add(getMatch());
		return matchSet;
	}

	private SegmentCluster getSegmentCluster() {
		return new SegmentCluster(1, null);
	}

	private SearchResult getSearchResult() {
		return new SearchResult("id", getMatchSet());
	}

	private Segment getSegment() {
		return new Segment("data", 3);
	}

	private List<Segment> getSegmentList() {
		List<Segment> list = new ArrayList<Segment>();
		list.add(getSegment());
		return list;
	}

	@SuppressWarnings("unchecked")
	@Test
	public void jacksonTest() throws JsonParseException, JsonMappingException,
			IOException {
		List<SearchResult> expected = new ArrayList<SearchResult>();
		expected.add(getSearchResult());
		String json = toJson(expected);
		List<SearchResult> actual = fromJson(json, ArrayList.class);
		String json2 = toJson(actual);
		Assert.assertEquals(json, json2);

	}

	protected String toJson(Object o) throws JsonParseException,
			JsonMappingException, IOException {
		StringWriter sw = new StringWriter();
		new ObjectMapper().writeValue(sw, o);
		return sw.toString();
	}
}
