/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class MemRepeatGroupCache implements IRepeatGroupCache {

	private final Map<IRepeatGroupKey, Set<IRepeatGroupValue>> cache;

	public MemRepeatGroupCache() {
		cache = new HashMap<IRepeatGroupKey, Set<IRepeatGroupValue>>();
	}

	public void clear() {
		cache.clear();
	}

	@Override
	public void add(IRepeatGroupKey key, IRepeatGroupValue value) {
		Set<IRepeatGroupValue> set = cache.get(key);
		if (set == null) {
			set = new HashSet<IRepeatGroupValue>();
			cache.put(key, set);
		}
		set.add(value);
	}

	@Override
	public void delete(IRepeatGroupKey key, IRepeatGroupValue value) {
		Set<IRepeatGroupValue> set = cache.get(key);
		if (set != null) {
			set.remove(value);
			if (set.isEmpty()) {
				cache.remove(key);
			}
		}
	}

	@Override
	public Set<IRepeatGroupValue> get(IRepeatGroupKey key) {
		Set<IRepeatGroupValue> set = cache.get(key);
		if (set == null) {
			set = new HashSet<IRepeatGroupValue>();
		}
		return set;
	}

	@Override
	public Map<IRepeatGroupKey, Set<IRepeatGroupValue>> getAll(
			IRepeatGroupKey[] keys) {
		Map<IRepeatGroupKey, Set<IRepeatGroupValue>> retval = new HashMap<IRepeatGroupKey, Set<IRepeatGroupValue>>(
				keys.length);
		for (IRepeatGroupKey key : keys) {
			retval.put(key, get(key));
		}
		return retval;
	}

}
