/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Cache implementation backed by {@link HashMap}.
 * <p>
 * This class is thread safe for read operations (get() methods). Add, delete
 * and clear method are not thread safe.
 */
public class MemSegmentCache implements ISegmentCache {

	private final Map<String, Set<ISegmentCacheValue>> cache;

	public MemSegmentCache() {
		cache = new HashMap<String, Set<ISegmentCacheValue>>();
	}

	@Override
	public void add(String key, ISegmentCacheValue value) {
		Set<ISegmentCacheValue> set = cache.get(key);
		if (set == null) {
			set = new HashSet<ISegmentCacheValue>();
			cache.put(key, set);
		}
		set.add(value);
	}

	@Override
	public void clear() {
		cache.clear();
	}

	@Override
	public void delete(String key, ISegmentCacheValue value) {
		Set<ISegmentCacheValue> set = cache.get(key);
		if (set != null) {
			set.remove(value);
			if (set.isEmpty()) {
				cache.remove(key);
			}
		}
	}

	@Override
	public Set<ISegmentCacheValue> get(String seg) {
		Set<ISegmentCacheValue> set = cache.get(seg);
		if (set == null) {
			set = new HashSet<ISegmentCacheValue>();
		}
		return set;
	}

	@Override
	public Map<String, Set<ISegmentCacheValue>> getAll(String[] keys) {
		Map<String, Set<ISegmentCacheValue>> retval = new HashMap<String, Set<ISegmentCacheValue>>(
				keys.length);
		for (String key : keys) {
			retval.put(key, get(key));
		}
		return retval;
	}

}
