/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.io.Serializable;
import java.util.Arrays;

public class RepeatGroupKey implements Serializable, IRepeatGroupKey {

	private static final long serialVersionUID = -4930412965124444007L;

	private final char[] chars;

	private final int[] values;

	public RepeatGroupKey(char[] chars, int[] values) {
		this.chars = chars;
		this.values = values;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RepeatGroupKey other = (RepeatGroupKey) obj;
		if (!Arrays.equals(chars, other.chars))
			return false;
		if (!Arrays.equals(values, other.values))
			return false;
		return true;
	}

	@Override
	public char[] getChars() {
		return chars;
	}

	@Override
	public int[] getValues() {
		return values;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(chars);
		result = prime * result + Arrays.hashCode(values);
		return result;
	}

	@Override
	public String toString() {
		return RepeatGroupKey.class.getSimpleName() + " [chars="
				+ Arrays.toString(chars) + ", values="
				+ Arrays.toString(values) + "]";
	}

}
