/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.io.Serializable;
import java.util.Arrays;

public class RepeatGroupValue implements IRepeatGroupValue, Serializable {

	private static final long serialVersionUID = 7208182149033765508L;

	private final long fpId;

	private final int[] originalValues;

	private final int position;

	public RepeatGroupValue(long fpId, int position, int[] originalValues) {
		this.fpId = fpId;
		this.position = position;
		this.originalValues = originalValues;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RepeatGroupValue other = (RepeatGroupValue) obj;
		if (fpId != other.fpId)
			return false;
		if (!Arrays.equals(originalValues, other.originalValues))
			return false;
		if (position != other.position)
			return false;
		return true;
	}

	@Override
	public long getFingerprintId() {
		return fpId;
	}

	@Override
	public int[] getOriginalValues() {
		return originalValues;
	}

	@Override
	public int getPosition() {
		return position;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (fpId ^ (fpId >>> 32));
		result = prime * result + Arrays.hashCode(originalValues);
		result = prime * result + position;
		return result;
	}

	@Override
	public String toString() {
		return RepeatGroupValue.class.getSimpleName() + " [fpId=" + fpId
				+ ", originalValues=" + Arrays.toString(originalValues)
				+ ", position=" + position + "]";
	}

}
