/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.util.Map;
import java.util.Set;

/**
 * Cache contains a set of cache entries associated to a key. String segment is
 * a key and {@link SegmentCacheValue} is value.
 */
public interface SegmentCache {

	/**
	 * Append a new cache entry to segment.
	 * 
	 * @param segment
	 * @param ce
	 */
	void addCacheEntry(String segment, ISegmentCacheValue ce);

	/**
	 * Clears the cache.
	 */
	void clear();

	void deleteCacheEntry(String segment, ISegmentCacheValue ce);

	/**
	 * Retrieves cache entries associated to a segment.
	 * 
	 * @param seg
	 * @return cache entries associated to segment or an empty set if segment
	 *         has no cache entries.
	 */
	Set<ISegmentCacheValue> get(String seg);

	/**
	 * Same as invoking {@link #get(String)} for each individual segments.
	 * 
	 * @param seg
	 * @return all cache entries by segment or an empty map if no cache entries
	 *         where found.
	 */
	Map<String, Set<ISegmentCacheValue>> get(String[] seg);
}
