/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.io.Serializable;

/**
 * Encapsulates fingerprint identifier and segment position.
 */
public class SegmentCacheValue implements Serializable, ISegmentCacheValue {

	private static final long serialVersionUID = -3946294400138577691L;

	private final int hashcode;

	private final long id;

	private final int position;

	/**
	 * 
	 * @param id
	 *            fingerprint identifier
	 * @param position
	 *            segment position.
	 */
	public SegmentCacheValue(final long id, final int position) {
		this.id = id;
		this.position = position;
		hashcode = calcHashCode();
	}

	public int calcHashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (id ^ (id >>> 32));
		result = prime * result + position;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SegmentCacheValue other = (SegmentCacheValue) obj;
		if (id != other.id)
			return false;
		if (position != other.position)
			return false;
		return true;
	}

	public long getFingerprintId() {
		return id;
	}

	public int getSegmentPosition() {
		return position;
	}

	@Override
	public int hashCode() {
		return hashcode;
	}

	@Override
	public String toString() {
		return SegmentCacheValue.class.getSimpleName() + " [id=" + id + ", position="
				+ position + "]";
	}

}
