/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.jhears.cache.IRepeatGroupCache;
import org.jhears.cache.IRepeatGroupKey;
import org.jhears.cache.IRepeatGroupValue;
import org.jhears.cache.MemRepeatGroupCache;
import org.jhears.cache.RepeatGroupKey;
import org.jhears.cache.RepeatGroupValue;
import org.junit.Assert;
import org.junit.Test;

public class RepeatGroupCacheTest {

	@Test
	public void testEmpty() {
		IRepeatGroupCache cache = new MemRepeatGroupCache();
		char[] chars = "ab".toCharArray();
		int[] combinations = new int[] { 3, 5, 7 };
		IRepeatGroupKey lookupKey = new RepeatGroupKey(chars, combinations);
		Set<IRepeatGroupValue> cacheEntries = cache.get(lookupKey);
		Assert.assertTrue(cacheEntries.isEmpty());
	}

	@Test
	public void testInsert1() {
		IRepeatGroupCache cache = new MemRepeatGroupCache();

		IRepeatGroupValue cacheEntry = new RepeatGroupValue(123L, 0, new int[] {
				2, 6, 8 });
		RepeatGroupKey insertKey = new RepeatGroupKey("ab".toCharArray(),
				new int[] { 3, 5, 7 });
		cache.add(insertKey, cacheEntry);

		char[] chars = "ab".toCharArray();
		int[] combinations = new int[] { 3, 5, 7 };
		RepeatGroupKey fpLSHKey = new RepeatGroupKey(chars, combinations);
		Set<IRepeatGroupValue> cacheEntries = cache.get(fpLSHKey);
		Assert.assertEquals(1, cacheEntries.size());
		Assert.assertEquals(
				new RepeatGroupValue(123L, 0, new int[] { 2, 6, 8 }),
				cacheEntries.iterator().next());
	}

	@Test
	public void testInsert2() {
		IRepeatGroupCache cache = new MemRepeatGroupCache();

		IRepeatGroupValue cacheEntry = new RepeatGroupValue(123L, 0, new int[] {
				2, 6, 8 });
		RepeatGroupKey insertKey = new RepeatGroupKey("ab".toCharArray(),
				new int[] { 3, 5, 7 });
		cache.add(insertKey, cacheEntry);

		char[] chars = "ab".toCharArray();
		int[][] combinations = new int[2][];
		combinations[0] = new int[] { 2, 4, 6 };
		combinations[1] = new int[] { 3, 5, 7 };
		RepeatGroupKey[] fpLSHKeys = new RepeatGroupKey[2];
		fpLSHKeys[0] = new RepeatGroupKey(chars, combinations[0]);
		fpLSHKeys[1] = new RepeatGroupKey(chars, combinations[1]);
		Map<IRepeatGroupKey, Set<IRepeatGroupValue>> cacheEntries = cache
				.getAll(fpLSHKeys);
		Assert.assertEquals(2, cacheEntries.size());
		Assert.assertTrue(cacheEntries.get(fpLSHKeys[0]).isEmpty());
		Assert.assertEquals(
				new RepeatGroupValue(123L, 0, new int[] { 2, 6, 8 }),
				cacheEntries.get(fpLSHKeys[1]).iterator().next());
	}

	@Test
	public void testInsert3() {
		IRepeatGroupCache cache = new MemRepeatGroupCache();

		RepeatGroupValue cacheEntry = new RepeatGroupValue(123L, 0, new int[] {
				2, 6, 8 });
		RepeatGroupKey insertKey = new RepeatGroupKey("ab".toCharArray(),
				new int[] { 3, 5, 7 });
		cache.add(insertKey, cacheEntry);
		char[] chars = "ab".toCharArray();
		int[][] combinations = new int[2][];
		combinations[0] = new int[] { 2, 4, 6 };
		combinations[1] = new int[] { 3, 5, 8 }; // fails because 8
		RepeatGroupKey[] fpLSHKeys = new RepeatGroupKey[2];
		fpLSHKeys[0] = new RepeatGroupKey(chars, combinations[0]);
		fpLSHKeys[1] = new RepeatGroupKey(chars, combinations[1]);
		Map<IRepeatGroupKey, Set<IRepeatGroupValue>> cacheEntries = cache
				.getAll(fpLSHKeys);
		Assert.assertEquals(2, cacheEntries.size());
		Iterator<Set<IRepeatGroupValue>> i = cacheEntries.values().iterator();
		Assert.assertEquals(0, i.next().size());
		Assert.assertEquals(0, i.next().size());
	}

	@Test
	public void testInsert4() {
		IRepeatGroupCache cache = new MemRepeatGroupCache();

		RepeatGroupValue cacheEntry = new RepeatGroupValue(123L, 0, new int[] {
				2, 6, 8 });
		// fails because aX
		RepeatGroupKey insertKey = new RepeatGroupKey("aX".toCharArray(),
				new int[] { 3, 5, 7 });
		cache.add(insertKey, cacheEntry);

		char[] chars = "ab".toCharArray();
		int[] combinations = new int[] { 3, 5, 7 };
		RepeatGroupKey fpLSHKey = new RepeatGroupKey(chars, combinations);
		Set<IRepeatGroupValue> cacheEntries = cache.get(fpLSHKey);
		Assert.assertEquals(0, cacheEntries.size());
	}
}
