/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.math;

/**
 * The {@link #compareTo(WeightedInt)}, {@link #equals(Object)},
 * {@link #hashCode()} use only the {@link #getWeight()} value. The integer
 * value is just a payload which is carried along.
 */
public class WeightedInt implements Comparable<WeightedInt> {

	private final int value;

	private final double weight;

	public WeightedInt(final double weight, final int value) {
		this.weight = weight;
		this.value = value;
	}

	@Override
	public int compareTo(WeightedInt o) {
		return Double.compare(weight, o.weight);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		WeightedInt other = (WeightedInt) obj;
		if (Double.doubleToLongBits(weight) != Double
				.doubleToLongBits(other.weight))
			return false;
		return true;
	}

	public int getValue() {
		return value;
	}

	public double getWeight() {
		return weight;
	}

	@Override
	public int hashCode() {
		final long temp = Double.doubleToLongBits(weight);
		return (int) (temp ^ (temp >>> 32));
	}
}
