/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.math;

import java.util.Random;

import junit.framework.Assert;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.LoggerContext;

public class ELBGTest {

	public static void main(String[] args) {
		LoggerContext lc = (LoggerContext) LoggerFactory.getILoggerFactory();
		lc.getLogger(Logger.ROOT_LOGGER_NAME).setLevel(Level.INFO);
		ELBGTest t = new ELBGTest();
		t.cantor(2756887000984075579L);
	}

	public void cantor(long seed) {
		Random rnd = new Random(seed);
		double[][] data = new double[2000][];
		for (int i = 0; i < 2000; i++) {
			data[i] = dot2(rnd);
		}
		sleep(0);
		ELBG elbg = new ELBG(data, 16, 0.001, 0.01, rnd, 1 << 16);
		elbg.setParanoidMode(true);
		double[][] y = elbg.calculate().y;
		sleep(0);
		for (int r = 0; r < y.length; r++) {
			for (int c = 0; c < y[0].length; c++) {
				if (!check(y[r][c])) {
					Assert.fail("seed " + seed + " " + check(y[r][c]));
					System.out.println(y[r][c]);
					return;
				}
			}
		}
	}

	@Test
	public void cantorTest() {
		LoggerContext lc = (LoggerContext) LoggerFactory.getILoggerFactory();
		lc.getLogger(Logger.ROOT_LOGGER_NAME).setLevel(Level.INFO);
		Random r = new Random(181332337831200L);
		for (int i = 0; i < 100; i++) {
			cantor(r.nextLong());
		}
	}

	private boolean check(double d) {
		d = 0.5 + d;
		double a = 1.0 / 9.0 / 3.0;

		if (d >= 0 && d < a * 1) {
			return false;
		} else if (d >= a * 2 && d < a * 3) {
			return false;
		} else if (d >= a * 6 && d < a * 7) {
			return false;
		} else if (d >= a * 8 && d < a * 9) {
			return false;
		} else if (d >= a * 18 && d < a * 19) {
			return false;
		} else if (d >= a * 20 && d < a * 21) {
			return false;
		} else if (d >= a * 24 && d < a * 25) {
			return false;
		} else if (d >= a * 26 && d < a * 27) {
			return false;
		}
		return true;
	}

	private double[] dot2(Random rnd) {

		return new double[] { cantorVal(rnd), cantorVal(rnd) };
	}

	private void sleep(long time) {
		try {
			Thread.sleep(time);
		} catch (InterruptedException e) {
		}
	}

	private void centroidAt0_0(long seed) {
		int rows = 117;
		int cols = 2;
		int codewords = 4;
		double x[][] = new double[rows][cols];
		Random rnd = new Random(seed);
		for (int r = 0; r < rows; r++) {
			for (int c = 0; c < cols; c++) {
				double d = 1 + rnd.nextDouble() + Math.abs(rnd.nextGaussian());
				if (rnd.nextBoolean())
					d = -d;
				x[r][c] = d;
			}
			// System.out.println(Arrays.toString(x[r]));
		}

		ELBG elbg = new ELBG(x, codewords, 0.01, 0.1, rnd, 1 << 16);
		double[][] y = elbg.calculate().y;
		boolean ok1 = false;
		boolean ok2 = false;
		boolean ok3 = false;
		boolean ok4 = false;
		for (double[] c : y) {
			if (!ok1) {
				ok1 = c[0] > 0 && c[1] > 0;
			}
			if (!ok2) {
				ok2 = c[0] > 0 && c[1] < 0;
			}
			if (!ok3) {
				ok3 = c[0] < 0 && c[1] > 0;
			}
			if (!ok4) {
				ok4 = c[0] < 0 && c[1] < 0;
			}
		}

		if (!ok1) {
			Assert.fail("seed " + seed);
		}
		if (!ok2) {
			Assert.fail("seed " + seed);
		}
		if (!ok3) {
			Assert.fail("seed " + seed);
		}
		if (!ok4) {
			Assert.fail("seed " + seed);
		}

	}

	private void clustersAt15_25_35_45(long seed) {
		int rows = 19;
		int cols = 2;
		int codewords = 4;
		double x[][] = new double[rows][cols];
		Random rnd = new Random(seed);
		for (int r = 0; r < rows; r++) {
			int mult = (r + 1) / (codewords + 1) + 1;
			mult *= 10;
			for (int c = 0; c < cols; c++) {
				double d = 1 + rnd.nextDouble() + Math.abs(rnd.nextGaussian());
				d += mult;
				x[r][c] = d;
			}
			// System.out.println(mult + " " + Arrays.toString(x[r]));
		}

		ELBG elbg = new ELBG(x, codewords, 0.01, 0.1, rnd, 1 << 16);
		double[][] y = elbg.calculate().y;
		boolean ok1 = false;
		boolean ok2 = false;
		boolean ok3 = false;
		boolean ok4 = false;
		for (double[] c : y) {
			if (!ok1) {
				ok1 = c[0] > 10 && c[0] < 20 && c[1] > 10 && c[1] < 20;
			}
			if (!ok2) {
				ok2 = c[0] > 20 && c[0] < 30 && c[1] > 20 && c[1] < 30;
			}
			if (!ok3) {
				ok3 = c[0] > 30 && c[0] < 40 && c[1] > 30 && c[1] < 40;
			}
			if (!ok4) {
				ok4 = c[0] > 40 && c[0] < 50 && c[1] > 40 && c[1] < 50;
			}
		}
		if (!ok1) {
			Assert.fail("seed " + seed);
		}
		if (!ok2) {
			Assert.fail("seed " + seed);
		}
		if (!ok3) {
			Assert.fail("seed " + seed);
		}
		if (!ok4) {
			Assert.fail("seed " + seed);
		}

	}

	@Test
	public void testPerfect() {
		Random rnd = new Random(0);
		double[][] x = new double[][] { { 100, 200 }, { 300, 400 } };
		double[][] y = new double[][] { { 101, 202 }, { 303, 404 } };
		ELBG elbg = new ELBG(x, y, 0.1, rnd, 1 << 16);
		elbg.calculate();
	}

	@Test
	public void testPerfect2() {
		Random rnd = new Random(0);
		double[][] x = new double[][] { { 0, 0 }, { 0, 0 } };
		double[][] y = new double[][] { { 0, 0 }, { 0, 0 } };
		ELBG elbg = new ELBG(x, y, 0.1, rnd, 1 << 16);
		elbg.calculate();
	}

	@Test
	public void testCentroindAt0_0() {
		Random r = new Random(8674187844256335618L);
		for (int i = 0; i < 10; i++) {
			centroidAt0_0(r.nextLong());
		}
	}

	@Test
	public void testClustersAt15_25_35_45() {
		Random r = new Random(254486841079717039L);
		for (int i = 0; i < 10; i++) {
			clustersAt15_25_35_45(r.nextLong());
		}
	}

	private double cantorVal(Random rnd) {
		double d = 0;
		double a = 1.0 / 9.0 / 3.0;
		while (true) {
			d = rnd.nextDouble();
			if (d >= 0 && d < a * 1) {
				break;
			} else if (d >= a * 2 && d < a * 3) {
				break;
			} else if (d >= a * 6 && d < a * 7) {
				break;
			} else if (d >= a * 8 && d < a * 9) {
				break;
			} else if (d >= a * 18 && d < a * 19) {
				break;
			} else if (d >= a * 20 && d < a * 21) {
				break;
			} else if (d >= a * 24 && d < a * 25) {
				break;
			} else if (d >= a * 26 && d < a * 27) {
				break;
			}
		}
		d = 0.5 - d;
		return d;
	}
}
