/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears;

import java.io.File;
import java.io.IOException;
import java.lang.Thread.UncaughtExceptionHandler;

import org.jhears.cnf.CnfManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.util.ContextInitializer;

public class JHearsClient {

	/**
	 * Parse command line and execute the client
	 * 
	 * @throws IOException
	 */
	public static int executeClient(String[] args) throws IOException {
		return new JHearsClient().perform(args);
	}

	/**
	 * Does following: {@code System.exit(executeClient(args))}
	 * 
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		System.exit(executeClient(args));
	}

	private final Logger logger;

	public JHearsClient() throws IOException {
		CnfManager cnfManager = new CnfManager();
		if (System.getProperty(ContextInitializer.CONFIG_FILE_PROPERTY) == null) {
			File loggerCnf = new File(cnfManager.getConfigDir(), "logback.xml");
			System.setProperty(ContextInitializer.CONFIG_FILE_PROPERTY,
					loggerCnf.getAbsolutePath());
		}
		logger = LoggerFactory.getLogger(JHearsClient.class);
		// JulSlf4jBridge.install();
		jvmSetup();
	}

	private void jvmSetup() {
		UncaughtExceptionHandler eh = new UncaughtExceptionHandler() {

			@Override
			public void uncaughtException(Thread t, Throwable e) {
				logger.error("Uncaught exception. thread=" + t, e);
			}
		};
		Thread.setDefaultUncaughtExceptionHandler(eh);
		// Thread hook = new Thread(new Runnable() {
		//
		// @Override
		// public void run() {
		// logger.info("Shutdown hook executing");
		// logger.info("Shutdown complete");
		// }
		// });
		// Runtime.getRuntime().addShutdownHook(hook);
	}

	public int perform(String[] args) {
		IJHearsClient client = new JHearsClientNG();
		ClientCommandLine cmdline = new ClientCommandLine(client);
		return cmdline.parseCommandLine(args);
	}

}
