/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * {@link ICountingBitArray} implementation which is persisted to a file.
 * 
 * @author Juha Heljoranta
 * 
 */
public class CountingFileBitArray extends AbstractFileBitArray implements
		ICountingBitArray {

	private static final long serialVersionUID = 1237186845439404889L;

	/**
	 * 
	 * @param length
	 *            The length of the bit array.
	 * @param file
	 *            The file where the bit array is persisted.
	 */
	public CountingFileBitArray(final int length, File file) {
		this(length, file, true, Mode.RW);
	}

	/**
	 * 
	 * @param length
	 *            The length of the bit array.
	 * @param file
	 *            The file where the bit array is persisted.
	 * @param autoClose
	 *            True if the file should be closed automatically after each
	 *            operation.
	 * @param mode
	 *            The mode in which the file should be opened.
	 */
	public CountingFileBitArray(final int length, File file, boolean autoClose,
			Mode mode) {
		super(length, file, length, autoClose, mode);
	}

	@Override
	protected void clear(final int index, RandomAccessFile raf)
			throws IOException {
		raf.seek(index);
		int b = raf.read();
		if (b == -1) {
			throw new EOFException();
		}
		if ((b & 0xff) > 0) {
			raf.seek(index);
			raf.write((b & 0xff) - 1);
		}
	}

	@Override
	protected boolean get(int index, RandomAccessFile raf) throws IOException {
		raf.seek(index);
		int b = raf.read();
		if (b == -1) {
			throw new EOFException();
		}
		return (b & 0xff) > 0;
	}

	@Override
	protected void set(int index, RandomAccessFile raf) throws IOException {
		raf.seek(index);
		int b = raf.read();
		if (b == -1) {
			throw new EOFException();
		}
		if ((b & 0xff) < 0xff) {
			raf.seek(index);
			raf.write((b & 0xff) + 1);
		} else {
			throw new CountingBitArrayOverflowException();
		}
	}

}
