/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * {@link IBitArray} implementation which is persisted to a file.
 * 
 * @author Juha Heljoranta
 * 
 */
public class FileBitArray extends AbstractFileBitArray {

	private static final long serialVersionUID = -3378630308563973930L;

	protected static final int sizeOfLong = Long.SIZE / Byte.SIZE;

	/**
	 * 
	 * @param length
	 *            The length of the bit array.
	 * @param file
	 *            The file where the bit array is persisted.
	 */
	public FileBitArray(final int length, File file) {
		this(length, file, true, Mode.RW);
	}

	/**
	 * 
	 * @param length
	 *            The length of the bit array.
	 * @param file
	 *            The file where the bit array is persisted.
	 * @param autoClose
	 *            True if the file should be closed automatically after each
	 *            operation.
	 * @param mode
	 *            The mode in which the file should be opened.
	 */
	public FileBitArray(final int length, File file, boolean autoClose,
			Mode mode) {
		// the file size must be multiple of 8 since long is 8 bytes.
		super(length, file, ((long) length / Long.SIZE + 1) * sizeOfLong,
				autoClose, mode);
	}

	@Override
	protected void clear(final int index, RandomAccessFile raf)
			throws IOException {
		raf.seek(index / Long.SIZE * sizeOfLong);
		long l = raf.readLong();
		raf.seek(index / Long.SIZE * sizeOfLong);
		l &= ~(1L << (index % Long.SIZE));
		raf.writeLong(l);
	}

	@Override
	protected boolean get(int index, RandomAccessFile raf) throws IOException {
		raf.seek(index / Long.SIZE * sizeOfLong);
		long l = raf.readLong();
		return (l & (1L << (index % Long.SIZE))) != 0L;
	}

	@Override
	protected void set(int index, RandomAccessFile raf) throws IOException {
		raf.seek(index / Long.SIZE * sizeOfLong);
		long l = raf.readLong();
		raf.seek(index / Long.SIZE * sizeOfLong);
		l |= 1L << (index % Long.SIZE);
		raf.writeLong(l);
	}

}
