/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

import java.io.Serializable;

/**
 * Forwards calls to a delegate.
 * 
 * @author Juha Heljoranta
 * 
 */
public class ForwardingBitArray implements IBitArray, Serializable {

	private static final long serialVersionUID = 1672133134239699889L;

	private volatile IBitArray delegate;

	public ForwardingBitArray() {
		this(null);
	}

	public ForwardingBitArray(IBitArray delegate) {
		this.delegate = delegate;
	}

	@Override
	public void clear() {
		delegate.clear();
	}

	@Override
	public void clear(int index) throws IndexOutOfBoundsException {
		delegate.clear(index);
	}

	@Override
	public void clearAll(int[] indexes) throws IndexOutOfBoundsException {
		delegate.clearAll(indexes);
	}

	@Override
	public boolean get(int index) throws IndexOutOfBoundsException {
		return delegate.get(index);
	}

	@Override
	public boolean[] getAll(int[] indexes) throws IndexOutOfBoundsException {
		return delegate.getAll(indexes);
	}

	public IBitArray getDelegate() {
		return delegate;
	}

	@Override
	public void set(int index) throws IndexOutOfBoundsException {
		delegate.set(index);
	}

	@Override
	public void setAll(int[] indexes) throws IndexOutOfBoundsException {
		delegate.setAll(indexes);
	}

	public void setDelegate(IBitArray delegate) {
		this.delegate = delegate;
	}

	@Override
	public int size() {
		return delegate.size();
	}

	@Override
	public String toString() {
		return ForwardingBitArray.class.getSimpleName() + " [delegate="
				+ delegate + "]";
	}

}
