/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

/**
 * Array of bit values.
 * 
 * @author Juha Heljoranta
 * 
 */
public interface IBitArray {

	/**
	 * Set all bits to false.
	 */
	public void clear();

	/**
	 * Set bit to false.
	 * 
	 * @param index
	 *            Bit index to set false.
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range (
	 *             <code>index &lt; 0 || index &gt;= size()</code>)
	 */
	public void clear(int index) throws IndexOutOfBoundsException;

	public void clearAll(int[] indexes) throws IndexOutOfBoundsException;

	/**
	 * Get bit value.
	 * 
	 * @param index
	 *            Bit index which value is returned.
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range (
	 *             <code>index &lt; 0 || index &gt;= size()</code>)
	 * @return value of bit.
	 */
	public boolean get(int index) throws IndexOutOfBoundsException;

	public boolean[] getAll(int[] indexes) throws IndexOutOfBoundsException;

	/**
	 * Set bit to true.
	 * 
	 * @param index
	 *            Bit index to set true.
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range (
	 *             <code>index &lt; 0 || index &gt;= size()</code>)
	 */
	public void set(int index) throws IndexOutOfBoundsException;

	public void setAll(int[] indexes) throws IndexOutOfBoundsException;

	/**
	 * Returns the number of bits in this array.
	 * 
	 * @return the number of bits in this array.
	 */
	public int size();

}
