/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

/**
 * Bit array where each bit is backed by a small counter (say, a {@code byte})
 * which is incremented on {@link #set(int)} and decremented on
 * {@link #clear(int)}.
 * 
 * @author Juha Heljoranta
 * 
 */
public interface ICountingBitArray extends IBitArray {

	/**
	 * Decrements counter by one if counter value is greater than zero.
	 * 
	 * @throws CountingBitArrayOverflowException
	 *             if counter value would be come negative.
	 */
	@Override
	public void clear(int index) throws IndexOutOfBoundsException,
			CountingBitArrayOverflowException;

	/**
	 * Returns true if counter value is greater than zero.
	 */
	@Override
	public boolean get(int index) throws IndexOutOfBoundsException;

	/**
	 * Increments counter value by one.
	 * 
	 * @throws CountingBitArrayOverflowException
	 *             if counter value would overflow.
	 */
	@Override
	public void set(int index) throws IndexOutOfBoundsException,
			CountingBitArrayOverflowException;

}
