/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import java.io.Serializable;

/**
 * Abstract data object for storing bloom filter state.
 * 
 * @author Juha Heljoranta
 * 
 */
public abstract class AbstractBFState implements IBloomFilterState,
		Serializable {

	private static final long serialVersionUID = -4219662477792111183L;

	private final int k;

	private final int m;

	private long size;

	public AbstractBFState(int m, int k) {
		this.m = m;
		this.k = k;
		size = 0L;
	}

	@Override
	public void clear() {
		size = 0L;
	}

	@Override
	public void decSize() {
		size--;
	}

	@Override
	public int getK() {
		return k;
	}

	@Override
	public int getM() {
		return m;
	}

	@Override
	public void incSize() {
		size++;
	}

	@Override
	public boolean isEmpty() {
		return size == 0L;
	}

	@Override
	public long size() {
		return size;
	}

	@Override
	public String toString() {
		return AbstractBFState.class.getSimpleName() + " [k=" + k + ", m=" + m
				+ ", size=" + size + "]";
	}

}
