/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import org.jhears.bitarray.BitArray;
import org.jhears.bitarray.IBitArray;

/**
 * Bloom filter state backed by {@link BitArray}.
 * 
 * @author Juha Heljoranta
 * 
 */
public class BitArrayBFState extends AbstractBFState {

	private static final long serialVersionUID = -8908238620328850153L;

	private final BitArray arr;

	public BitArrayBFState(int m, int k) {
		super(m, k);
		arr = new BitArray(m * k);
	}

	@Override
	public void clear() {
		super.clear();
		arr.clear();
	}

	@Override
	public long filterSize() {
		return arr.size();
	}

	@Override
	public IBitArray getBitArray() {
		return arr;
	}

	/**
	 * @return false.
	 */
	@Override
	public boolean supportsRemove() {
		return false;
	}

	@Override
	public String toString() {
		return BitArrayBFState.class.getSimpleName() + " [arr=" + arr
				+ ", super=" + super.toString() + "]";
	}

}
