/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import java.io.Serializable;

import org.jhears.filter.bloom.util.BloomMath;
import org.jhears.util.HashFunction;
import org.jhears.util.LCG;

/**
 * Default {@link IHasher} implementation.
 * <p>
 * This class is thread safe.
 * 
 * @author Juha Heljoranta
 * 
 */
public class Hasher implements IHasher, Serializable {

	private static final long serialVersionUID = 121922790080872037L;

	/** Hash function. */
	private final HashFunction<Object> genericHash;

	/** Pseudo random number generator. */
	private final LCG lcg;

	/**
	 * Creates a new instance.
	 * 
	 * @param genericHash
	 *            Hash function.
	 * @param lcg
	 *            Random number generator.
	 */
	public Hasher(HashFunction<Object> genericHash, LCG lcg) {
		this.genericHash = genericHash;
		this.lcg = lcg;
	}

	@Override
	public int[] hash(final long value, final int count) {
		return BloomMath.positiveAbs(lcg.nextInts(value, count));
	}

	@Override
	public int[] hash(final Object value, final int count) {
		return hash(genericHash.hash(value), count);
	}

}
