/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

/**
 * Common methods for all bloom filters.
 * 
 * @author Juha Heljoranta
 */
public interface IBaseFilter {

	/**
	 * Set all bits to false and size to zero.
	 */
	public void clear();

	/**
	 * Returns the number of bits allocated to this filter.
	 * 
	 * @return The number of bits allocated to this filter.
	 */
	public long filterSize();

	/**
	 * Returns true if this filter is empty.
	 * 
	 * @return
	 */
	public boolean isEmpty();

	/**
	 * The number of elements added to this filter.
	 * 
	 * @return The number of elements added to this filter.
	 */
	public long size();

	/**
	 * Returns true if this bloom filter supports element removal.
	 * 
	 * @return true if this bloom filter supports element removal.
	 */
	public boolean supportsRemove();

}
