/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

/**
 * Bloom filter.
 * <p>
 * In bloom filters false positives are possible, but false negatives are not.
 * That is, you can trust in the <code>false</code> values returned by
 * {@link #contains(Object)} but not the <code>true</code> values.
 * 
 * @author Juha Heljoranta
 * 
 * @param <T>
 *            Type of the elements this filter holds.
 */
public interface IBloomFilter<T> extends IBaseFilter {

	/**
	 * Adds specified element to this filter.
	 * 
	 * @param t
	 *            Element to be added to this filter.
	 */
	public void add(T t);

	/**
	 * Return <code>false</code> if this filter does <b>not</b> contains the
	 * specified element. False positives are possible but false negatives are
	 * not. In other words, <code>false</code> return value can be trusted but
	 * <code>true</code> return values not.
	 * 
	 * @param t
	 *            Element whose presence in this filter is tested.
	 * @return <code>false</code> if this filter doesn't contain the specified
	 *         element.
	 */
	public boolean contains(T t);

	/**
	 * Removes specified element from this filter.
	 * 
	 * @param t
	 *            Element to be removed from this filter.
	 * @return true if element was removed otherwise false.
	 * @throws UnsupportedOperationException
	 *             If implementation doesn't support removal.
	 */
	public boolean remove(T t) throws UnsupportedOperationException;

}
