/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import org.jhears.bitarray.IBitArray;

/**
 * Data object storing the state of a bloom filter.
 * 
 * @author Juha Heljoranta
 * 
 */
public interface IBloomFilterState {

	/**
	 * Set size to zero and clear the backing bit array.
	 */
	void clear();

	/**
	 * Decrement state by one.
	 */
	void decSize();

	/**
	 * The value of {@code getBitArray().size()} (the number of bits in bit
	 * array).
	 * 
	 * @return
	 */
	long filterSize();

	/**
	 * The bit array backing this state.
	 * 
	 * @return
	 */
	IBitArray getBitArray();

	/**
	 * The number of slices this filter state consist.
	 * 
	 * @return
	 */
	int getK();

	/**
	 * The number of bits per slice.
	 * 
	 * @return
	 */
	int getM();

	/**
	 * Increment state by one.
	 */
	void incSize();

	boolean isEmpty();

	/**
	 * The number of elements this state holds.
	 * 
	 * @return
	 */
	long size();

	/**
	 * True if this state supports element removal.
	 * 
	 * @return
	 */
	boolean supportsRemove();

}