/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;


/**
 * Special variant of Bloom Filters. {@link #add(int[])} and
 * {@link #contains(int[])} methods take array of hash function values
 * calculated from an element.
 * 
 * @author Juha Heljoranta
 * 
 */
public interface IHashValueBloomFilter extends IBaseFilter {

	/**
	 * Adds an element to this filter using hash values. Each array element
	 * represents output of hash function applied to element.
	 * 
	 * 
	 * @param hashValues
	 *            Hash values of an element which is added to this filter. All
	 *            values must be positive.
	 */
	public void add(int[] hashValues);

	/**
	 * Return <code>false</code> if this filter does <b>not</b> contains the
	 * specified element. False positives are possible but false negatives are
	 * not. In other words, <code>false</code> return value can be trusted but
	 * <code>true</code> return values not.
	 * 
	 * @param hashValues
	 *            Hash values of an element whose presence in this filter is
	 *            tested. All values must be positive. Each array element
	 *            represents output of hash function applied to element.
	 * @return <code>false</code> if this filter doesn't contain the specified
	 *         element.
	 */
	public boolean contains(int[] hashValues);

	/**
	 * Removes an element from this filter using hash values. Each array element
	 * represents output of hash function applied to element.
	 * 
	 * 
	 * @param hashValues
	 *            Hash values of an element which is remove to this filter. All
	 *            values must be positive.
	 * @return true if filter contained the element and the element was removed.
	 * @throws UnsupportedOperationException
	 *             If implementation doesn't support removing.
	 */
	public boolean remove(int[] hashValues)
			throws UnsupportedOperationException;

}
