/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

/**
 * Produces different hash values from given element.
 * <p>
 * Implementations must be thread-safe.
 * 
 * @author Juha Heljoranta
 * 
 */
public interface IHasher {

	/**
	 * Creates <code>count</code> number of different hashes from
	 * <code>value</code>. Returned values must be positive.
	 * 
	 * @param value
	 *            Seed value.
	 * @param count
	 *            The number of hashes to calculate.
	 * @throws IllegalArgumentException
	 *             If count is less than one.
	 * @return <code>count</code> number of different positive hashes from
	 *         value.
	 */
	public int[] hash(long value, int count) throws IllegalArgumentException;

	/**
	 * Creates <code>count</code> number of different hashes from
	 * <code>value</code>. Returned values must be positive.
	 * 
	 * @param value
	 *            Hashable element.
	 * @param count
	 *            The number of hashes to calculate.
	 * @throws IllegalArgumentException
	 *             If count is less than one.
	 * @return <code>count</code> number of different positive hashes from
	 *         value.
	 */
	public int[] hash(Object value, int count) throws IllegalArgumentException;

}
