/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom.scalable;

import java.util.Arrays;

import org.jhears.filter.bloom.CountingBitArrayBFState;
import org.jhears.filter.bloom.IBloomFilterState;

public class CountingBitArraySBFState extends AbstractSBFState {

	private static final long serialVersionUID = -2825195808941544110L;

	private CountingBitArrayBFState[] states;

	public CountingBitArraySBFState(int s, double r, int m0, double p0) {
		super(s, r, m0, p0);
		states = new CountingBitArrayBFState[0];
	}

	@Override
	protected IBloomFilterState appendNewState(int mi, int ki) {
		int i = states.length;
		states = Arrays.copyOf(states, i + 1);
		states[i] = new CountingBitArrayBFState(mi, ki);
		return states[i];
	}

	@Override
	protected void clearBFStates() {
		states = Arrays.copyOf(states, 1);
		states[0].clear();
	}

	@Override
	public int filterCount() {
		return states.length;
	}

	@Override
	protected IBloomFilterState[] getBFStates() {
		return states;
	}

	@Override
	protected void removeLastState() {
		states = Arrays.copyOf(states, states.length - 1);
	}

	@Override
	public String toString() {
		return BitArraySBFState.class.getSimpleName() + " [states="
				+ Arrays.toString(states) + ", super=" + super.toString() + "]";
	}

}
