/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.tuple;

import java.util.List;
import java.util.NoSuchElementException;

/**
 * Ordered list of elements.
 * 
 * @author Juha Heljoranta
 * 
 */
public interface Tuple {

	/**
	 * Returns the element at the specified position in this tuple.
	 * 
	 * @param index
	 *            index of the element to return
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range (
	 *             <code>index &lt; 0 || index &gt;= size()</code>)
	 * @return the element at the specified position in this tuple
	 */
	public Object get(int index) throws IndexOutOfBoundsException;

	/**
	 * Returns the first element of the tuple. Same as calling {@link #get(0)}
	 * 
	 * @return The first element of the tuple.
	 * @throws NoSuchElementException
	 *             if tuple size is zero.
	 */
	public Object head() throws NoSuchElementException;

	/**
	 * Returns the number of elements in this tuple.
	 * 
	 * @return the number of elements in this tuple.
	 */
	public int size();

	/**
	 * Get all elements except the head.
	 * 
	 * @return Remaining elements or an empty if no elements are available.
	 */
	public List<Object> tail();

	/**
	 * Returns a List containing all of the elements in this tuple in proper
	 * order (from first to last element).
	 * 
	 * @return Returns a List containing all of the elements in this tuple in
	 *         proper order.
	 */
	public List<Object> values();

}
