/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.tuple;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * Tuple with zero objects.
 * 
 * @author Juha Heljoranta
 * 
 */
public class Tuple0 implements Serializable, Tuple {

	private static final long serialVersionUID = 2765452935748206834L;

	public Tuple0() {
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Tuple0 other = (Tuple0) obj;
		if (values() == null) {
			if (other.values() != null)
				return false;
		} else if (!values().equals(other.values()))
			return false;
		return true;
	}

	@Override
	public Object get(int index) throws IndexOutOfBoundsException {
		throw new ArrayIndexOutOfBoundsException(index);
	}

	/**
	 * Returns the hash code value for this tuple. Uses following code to
	 * calculate the hash: <code>values().hashCode()</code>
	 * 
	 * @return the hash code value for this tuple.
	 */
	@Override
	public int hashCode() {
		return values().hashCode();
	}

	@Override
	public Object head() {
		throw new NoSuchElementException();
	}

	@Override
	public int size() {
		return 0;
	}

	@Override
	public List<Object> tail() {
		return new ArrayList<Object>();
	}

	@Override
	public String toString() {
		return getClass().getSimpleName() + " [values()=" + values() + "]";
	}

	@Override
	public List<Object> values() {
		return new ArrayList<Object>();
	}

}
