/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.tuple;

import java.util.List;

/**
 * Immutable tuple. Allows type safety via getX() methods if user takes
 * advantage of generics.
 * 
 * @author Juha Heljoranta
 * 
 */
public class Tuple11<A, B, C, D, E, F, G, H, I, J, K> extends
		Tuple10<A, B, C, D, E, F, G, H, I, J> {

	private static final long serialVersionUID = 6006408357692524795L;

	private final K o;

	public Tuple11(A a, B b, C c, D d, E e, F f, G g, H h, I i, J j, K k) {
		super(a, b, c, d, e, f, g, h, i, j);
		this.o = k;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		@SuppressWarnings("rawtypes")
		Tuple11 other = (Tuple11) obj;
		if (values() == null) {
			if (other.values() != null)
				return false;
		} else if (!values().equals(other.values()))
			return false;
		return true;
	}

	@Override
	public Object get(int index) throws IndexOutOfBoundsException {
		if (index == 10) {
			return get11();
		} else {
			return super.get(index);
		}
	}

	/**
	 * Returns the 11th element in this tuple.
	 * 
	 * @return the 11th element in this tuple.
	 */
	public K get11() {
		return o;
	}

	@Override
	public int size() {
		return 11;
	}

	@Override
	public List<Object> tail() {
		List<Object> list = super.tail();
		list.add(o);
		return list;
	}

	@Override
	public List<Object> values() {
		List<Object> list = super.values();
		list.add(o);
		return list;
	}

}
