/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.tuple;

import java.util.List;

/**
 * Immutable tuple. Allows type safety via getX() methods if user takes
 * advantage of generics.
 * 
 * @author Juha Heljoranta
 * 
 */
public class Tuple4<A, B, C, D> extends Tuple3<A, B, C> {

	private static final long serialVersionUID = -2813545251134322727L;

	private final D o;

	public Tuple4(A a, B b, C c, D d) {
		super(a, b, c);
		this.o = d;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		@SuppressWarnings("rawtypes")
		Tuple4 other = (Tuple4) obj;
		if (values() == null) {
			if (other.values() != null)
				return false;
		} else if (!values().equals(other.values()))
			return false;
		return true;
	}

	@Override
	public Object get(int index) throws IndexOutOfBoundsException {
		if (index == 3) {
			return get4();
		} else {
			return super.get(index);
		}
	}

	/**
	 * Returns the 4th element in this tuple.
	 * 
	 * @return the 4th element in this tuple.
	 */
	public D get4() {
		return o;
	}

	@Override
	public int size() {
		return 4;
	}

	@Override
	public List<Object> tail() {
		List<Object> list = super.tail();
		list.add(o);
		return list;
	}

	@Override
	public List<Object> values() {
		List<Object> list = super.values();
		list.add(o);
		return list;
	}

}
