/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.util;

public final class MathUtil {

	public static int[] seq(int length, int offset) {
		int[] seq = new int[length];
		for (int i = 0; i < length; i++) {
			seq[i] = i + offset;
		}
		return seq;
	}

	public static int[] seq(int length) {
		return seq(length, 0);
	}

	/**
	 * Returns absolute value. Method guarantees that the returned value is
	 * {@code >= 0}.
	 * <p>
	 * In case a value is {@code Integer.MIN_VALUE} the absolute is calculated
	 * from {@code Integer.MIN_VALUE + 1}.
	 * 
	 * @param value
	 * @return
	 */
	public static final int positiveAbs(final int value) {
		// prevent Math.abs() from returning Integer.MIN_VALUE
		return value == Integer.MIN_VALUE ? Math.abs(value + 1) : Math
				.abs(value);
	}

	/**
	 * Returns absolute values of value. Method guarantees that the returned
	 * value is {@code >= 0}.
	 * <p>
	 * In case a value is {@code Long.MIN_VALUE} the absolute is calculated from
	 * {@code Long.MIN_VALUE + 1}.
	 * 
	 * @param value
	 * @return
	 */
	public static final long trueAbs(final long value) {
		// prevent Math.abs() from returning Long.MIN_VALUE
		return value == Long.MIN_VALUE ? Math.abs(value + 1L) : Math.abs(value);
	}

	private MathUtil() {
	}

}
