/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

import java.util.Random;

import org.junit.Assert;
import org.junit.Test;

public class BitArrayUtilTest {

	private void setRandom(IBitArray arr, Random r, int length) {
		for (int i = 0; i < length / 2; i++) {
			arr.set(r.nextInt(length));
		}
	}

	@Test
	public void test() {
		int[] lenArr = new int[] { 0, 1, 2, 3, (1 << 13) - 3, (1 << 13) - 2,
				(1 << 13) - 1, (1 << 13) - 0, (1 << 13) + 1, (1 << 13) + 2,
				(1 << 13) + 3 };
		Random r = new Random(0);
		for (int length : lenArr) {
			IBitArray from = new BitArray(length);
			setRandom(from, r, length);
			IBitArray to = new BitArray(length);
			BitArrayUtil.copy(from, to);
			Assert.assertTrue(BitArrayUtil.equals(from, to));
		}
		int length = 1 << 10;
		IBitArray from = new BitArray(length);
		setRandom(from, r, length);
		IBitArray to = new BitArray(length);
		BitArrayUtil.copy(from, to);
		int i = r.nextInt(length);
		if (to.get(i)) {
			to.clear(i);
		} else {
			to.set(i);
		}
		Assert.assertFalse(BitArrayUtil.equals(from, to));
	}

}
