/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

@RunWith(value = MockitoJUnitRunner.class)
public class CachedBitArrayTest {

	@Test
	public void testClear() {
		IBitArray slowBitArray = Mockito.mock(IBitArray.class);
		IBitArray fastBitArray = Mockito.mock(IBitArray.class);
		CachedBitArray cached = new CachedBitArray(slowBitArray, fastBitArray);
		cached.clear();
		Mockito.verify(slowBitArray).clear();
		Mockito.verify(fastBitArray).clear();
	}

	@Test
	public void testClear1() {
		IBitArray slowBitArray = Mockito.mock(IBitArray.class);
		IBitArray fastBitArray = Mockito.mock(IBitArray.class);
		CachedBitArray cached = new CachedBitArray(slowBitArray, fastBitArray);
		cached.clear(1);
		Mockito.verify(slowBitArray).clear(1);
		Mockito.verify(fastBitArray).clear(1);
	}

	@Test
	public void testGet() {
		IBitArray slowBitArray = Mockito.mock(IBitArray.class);
		IBitArray fastBitArray = Mockito.mock(IBitArray.class);
		CachedBitArray cached = new CachedBitArray(slowBitArray, fastBitArray);
		cached.get(1);
		Mockito.verify(fastBitArray).get(1);
		Mockito.verifyZeroInteractions(slowBitArray);
	}

	@Test
	public void testSet() {
		IBitArray slowBitArray = Mockito.mock(IBitArray.class);
		IBitArray fastBitArray = Mockito.mock(IBitArray.class);
		CachedBitArray cached = new CachedBitArray(slowBitArray, fastBitArray);
		cached.set(2);
		Mockito.verify(slowBitArray).set(2);
		Mockito.verify(fastBitArray).set(2);
	}

	@Test
	public void testSize() {
		IBitArray slowBitArray = Mockito.mock(IBitArray.class);
		IBitArray fastBitArray = Mockito.mock(IBitArray.class);
		CachedBitArray cached = new CachedBitArray(slowBitArray, fastBitArray);
		cached.size();
		Mockito.verify(fastBitArray).size();
		Mockito.verifyZeroInteractions(slowBitArray);
	}

	// protected IBitArray getBitArray(int capacity) {
	// DatabaseConfig databaseConfig = BerkeleyMapManager
	// .getDefaultDatabaseConfig();
	// databaseConfig.setTemporary(true);
	// DatabaseMap<Long, Long> map = new BerkeleyMap<Long, Long>(manager,
	// databaseName, databaseConfig, serialization, hashFunction);
	// DatabaseBitArray persistent = new DatabaseBitArray(map, capacity);
	// persistent.clear();
	// BitArray cache = new BitArray(capacity);
	// // no need to init since we're clear
	// return new CachedBitArray(persistent, cache);
	// }
	//
	// public IBitArray cacheLoader(DatabaseMap<Long, Long> persistent,
	// int capacity) {
	// final long[] larr = new long[capacity / Long.SIZE + 1];
	// IterationObserver<Entry<Long, Long>> observer = new
	// IterationObserver<Entry<Long, Long>>() {
	//
	// @Override
	// public boolean observeNext() {
	// return true;
	// }
	//
	// @Override
	// public void observe(Entry<Long, Long> elt) {
	// int arrIdx = elt.getKey().intValue();
	// long data = elt.getValue().longValue();
	// larr[arrIdx] = data;
	// }
	// };
	// persistent.iterateMapEntries(observer);
	// return new BitArray(capacity, larr) {
	// private static final long serialVersionUID = 6642168045745382369L;
	// };
	// }
}
