/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.bitarray;

import org.junit.Assert;
import org.junit.Test;

public abstract class ICountingBitArrayTest extends IBitArrayTest {

	@Override
	protected abstract ICountingBitArray getBitArray(int capacity);

	@Test
	public void testCounting() {
		ICountingBitArray cba = getBitArray(1);
		Assert.assertFalse(cba.get(0));
		for (int i = 0; i < 0xff; i++) {
			cba.set(0);
			Assert.assertTrue(cba.get(0));
		}
		for (int i = 0; i < 0xff - 1; i++) {
			cba.clear(0);
			Assert.assertTrue(cba.get(0));
		}
		cba.clear(0);
		Assert.assertFalse(cba.get(0));
		cba.clear(0);
		Assert.assertFalse(cba.get(0));
		cba.set(0);
		Assert.assertTrue(cba.get(0));
		cba.clear(0);
		Assert.assertFalse(cba.get(0));
		cba.clear(0);
		Assert.assertFalse(cba.get(0));
	}

	@Test
	public void testNotOverflow() {
		ICountingBitArray cba = getBitArray(1);
		for (int i = 0; i < 0xff; i++) {
			cba.set(0);
		}
	}

	@Test(expected = CountingBitArrayOverflowException.class)
	public void testOverflow() {
		ICountingBitArray cba = getBitArray(1);
		for (int i = 0; i < 0xff + 1; i++) {
			cba.set(0);
		}
	}

	@Override
	protected int[] getCapacities() {
		return new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 30, 31, 32, 33, 34, 60,
				61, 62, 63, 64, 65, 66, 67, 68, 69, 125, 126, 127, 128, 129,
				130, 131, 132, 253, 254, 255 };
	}

}
