/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import org.jhears.filter.bloom.IBloomFilter;
import org.junit.Assert;
import org.junit.Test;

public abstract class AbstractBloomFilterTest extends AbstractFilterTest {

	protected abstract IBloomFilter<Long> createFilter(int size, double p0);

	protected void test(int size, double p0) {

		IBloomFilter<Long> filter = createFilter(size, p0);

		double errorProbability = testAndGetFalsePositives(size, filter, p0);

		double maxErr = 1.05; // +5%
		double minErr = 0.01; // -100%
		double expected = p0 * maxErr;
		Assert.assertTrue("errorProbability <= expected. errorProbability="
				+ errorProbability + " expected=" + expected,
				errorProbability <= expected);
		expected = p0 * minErr;
		Assert.assertTrue("errorProbability >= expected. errorProbability="
				+ errorProbability + " expected=" + expected,
				errorProbability >= expected);
	}

	@Test
	public void testP0OneBasispoint() {
		int size = 1;
		double p0 = 0.0001;
		test(size, p0);
	}

	@Test
	public void testP0OnePercet() {
		int size = 10240;
		double p0 = 0.01;
		test(size, p0);
	}

	@Test
	public void testP0OnePermil() {
		int size = 10240;
		double p0 = 0.001;
		test(size, p0);
	}

	@Test
	public void testP0TenPercet() {
		int size = 10240;
		double p0 = 0.1;
		test(size, p0);
	}

}
