/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;

import org.jhears.filter.bloom.IBloomFilter;
import org.junit.Assert;
import org.junit.Test;

public abstract class AbstractCountingBloomFilterTest extends
		AbstractBloomFilterTest {

	protected void addAndRemove(int size, double p0) {
		IBloomFilter<Long> filter = createFilter(size, p0);

		HashSet<Long> reference = new HashSet<Long>();
		Random r = new Random(0);
		for (int i = 0; i < size; i++) {
			Long value = Long.valueOf(r.nextLong());
			reference.add(value);
			filter.add(value);
		}
		for (Long value : reference) {
			Assert.assertTrue(filter.contains(value));
		}

		for (Long value : new ArrayList<Long>(reference)) {
			Assert.assertTrue(filter.remove(value));
			reference.remove(value);
		}

		Assert.assertEquals(reference.size(), filter.size());
	}

	private void doRandom(IBloomFilter<Long> filter) {
		Random r = new Random(0);
		List<Long> re = new ArrayList<Long>();
		filter.clear();
		for (int i = 0; i < getRandomTestSize(); i++) {
			switch (r.nextInt(4)) {
			case 0:
				Long v = r.nextLong();
				re.add(v);
				filter.add(v);
				break;
			case 1:
				if (!re.isEmpty()) {
					Long x = re.get(r.nextInt(re.size()));
					Assert.assertTrue(filter.contains(x));
				}
				break;
			case 2:
				if (!re.isEmpty()) {
					Long x = re.remove(r.nextInt(re.size()));
					Assert.assertTrue(filter.remove(x));
				}
				break;
			case 3:
				Assert.assertEquals(re.size(), filter.size());
				break;
			default:
				break;
			}
		}
	}

	protected abstract int getRandomTestSize();

	@Test
	public void testAddAndRemove() {
		int size = 1024;
		double p0 = 0.01;
		addAndRemove(size, p0);
	}

	@Test
	public void testRandom() {
		int size = 1024;
		double p0 = 0.01;
		IBloomFilter<Long> filter = createFilter(size, p0);
		doRandom(filter);
	}

}
