/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import java.util.HashSet;
import java.util.Random;

import org.jhears.filter.bloom.IBloomFilter;
import org.junit.Assert;

public abstract class AbstractFilterTest {

	protected double testAndGetFalsePositives(int size,
			IBloomFilter<Long> filter, double p) {
		HashSet<Long> reference = new HashSet<Long>();
		Random r = new Random(0);
		for (int i = 0; i < size; i++) {
			Long value = Long.valueOf(r.nextLong());
			reference.add(value);
			filter.add(value);
		}
		for (Long value : reference) {
			Assert.assertTrue(filter.contains(value));
		}
		// check false positives.
		int checks = (int) ((1.0 / p) * 1000.0); // try to get 1000 falsePos
		int falsePositive = 0;
		for (int i = 0; i < checks; i++) {
			Long value = Long.valueOf(r.nextLong());
			if (filter.contains(value) && !reference.contains(value)) {
				falsePositive++;
			}
		}
		Assert.assertEquals(size, filter.size());
		return falsePositive / (double) checks;
	}

}
