/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom;

import org.jhears.filter.bloom.BloomFilter;
import org.jhears.filter.bloom.HVBloomFilter;
import org.jhears.filter.bloom.CountingBitArrayBFState;
import org.jhears.filter.bloom.IBloomFilter;
import org.jhears.filter.bloom.IBloomFilterState;
import org.jhears.filter.bloom.util.BloomMath;
import org.jhears.util.DefaultHashFunction;
import org.jhears.util.HashFunction;
import org.jhears.util.LCG;

public class CountingBloomFilterTest extends AbstractCountingBloomFilterTest {

	@Override
	protected IBloomFilter<Long> createFilter(int size, double p0) {
		int m0 = BloomMath.optimalSliceSize(size);
		int k0 = BloomMath.optimalSliceCount(p0);
		IBloomFilterState state = new CountingBitArrayBFState(m0, k0);
		HVBloomFilter alg = new HVBloomFilter(state);
		HashFunction<Object> genericHash = new DefaultHashFunction<Object>();
		IHasher hasher = new Hasher(genericHash, new LCG());
		return new BloomFilter<Long>(alg, hasher);
	}

	@Override
	protected int getRandomTestSize() {
		return 10000;
	}

}
