/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom.scalable;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;

import org.jhears.filter.bloom.IBloomFilter;
import org.jhears.filter.bloom.scalable.IScalableBloomFilter;
import org.jhears.filter.bloom.util.BloomMath;
import org.junit.Assert;
import org.junit.Test;

public abstract class AbstractCountingScalableBloomFilterTest extends
		AbstractScalableBloomFilterTest {

	protected void addAndRemove(int size, double p0, double r, int s, int m0) {
		IBloomFilter<Long> filter = createFilter(size, p0, r, s, m0);

		HashSet<Long> reference = new HashSet<Long>();
		Random rnd = new Random(0);
		int failedDeletes = 0;
		int tests = getRandomTestSize();
		for (int i = 0; i < tests; i++) {
			Long value = Long.valueOf(rnd.nextLong());
			reference.add(value);
			filter.add(value);
		}
		for (Long value : reference) {
			Assert.assertTrue(filter.contains(value));
		}

		for (Long value : new ArrayList<Long>(reference)) {
			if (!filter.remove(value)) {
				failedDeletes++;
			}
			reference.remove(value);
		}

		Assert.assertEquals(reference.size(), filter.size() - failedDeletes);

		Assert.assertTrue("expected less than " + tests * 0.001
				+ " failures but got " + failedDeletes,
				failedDeletes < tests * 0.001);

	}

	private void doRandom(IScalableBloomFilter<Long> filter, int testCnt) {
		Random r = new Random(0);
		List<Long> re = new ArrayList<Long>();
		filter.clear();
		int failedRemoves = 0;
		for (int i = 0; i < testCnt; i++) {
			switch (r.nextInt(5)) {
			case 0:
				Long v = r.nextLong();
				re.add(v);
				filter.add(v);
				break;
			case 1:
				if (!re.isEmpty()) {
					Long x = re.get(r.nextInt(re.size()));
					Assert.assertTrue(filter.contains(x));
				}
				break;
			case 2:
				if (!re.isEmpty()) {
					Long x = re.remove(r.nextInt(re.size()));
					if (!filter.remove(x)) {
						failedRemoves++;
					}
				}
				break;
			case 3:
				Assert.assertEquals(re.size(), filter.size() - failedRemoves);
				break;
			case 4:
				filter.compact();
				break;
			default:
				break;
			}
		}
		Assert.assertTrue("Expected less than " + testCnt * 0.001
				+ " failed removals but got " + failedRemoves,
				failedRemoves < testCnt * 0.001);
	}

	protected abstract int getRandomTestSize();

	@Test
	public void testAddAndRemove() {
		int isize = 10;
		double r = 0.85;
		double P = 0.01;
		int s = 2;
		int mag = 2;
		double p0 = BloomMath.optimalSBFP0(P, r);
		int size = (int) Math.round(Math.pow(isize, mag));
		int m0 = BloomMath.optimalSliceSize(isize);
		addAndRemove(size, p0, r, s, m0);
	}

	@Test
	public void testRandom() {
		int isize = 10;
		double r = 0.85;
		double P = 0.01;
		int s = 2;
		int mag = 2;
		double p0 = BloomMath.optimalSBFP0(P, r);
		int size = (int) Math.round(Math.pow(isize, mag));
		int m0 = BloomMath.optimalSliceSize(isize);
		IScalableBloomFilter<Long> filter = createFilter(size, p0, r, s, m0);
		doRandom(filter, getRandomTestSize());
	}

}
