/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom.scalable;

import java.util.ArrayList;
import java.util.List;

import org.jhears.filter.bloom.AbstractFilterTest;
import org.jhears.filter.bloom.scalable.IScalableBloomFilter;
import org.jhears.filter.bloom.util.BloomMath;
import org.junit.Assert;
import org.junit.Test;

public abstract class AbstractScalableBloomFilterTest extends
		AbstractFilterTest {

	protected abstract IScalableBloomFilter<Long> createFilter(int size,
			double p0, double r, int s, int m0);

	@Test
	public void test() {
		int isize = 10; // initial set size
		List<Double> PList = new ArrayList<Double>();
		PList.add(0.1);
		PList.add(0.01);
		// PList.add(0.001);
		// PList.add(0.0001);
		List<Double> rList = new ArrayList<Double>();
		// rList.add(0.50);
		// rList.add(0.60);
		// rList.add(0.70);
		// rList.add(0.75);
		rList.add(0.80);
		rList.add(0.85);
		rList.add(0.90);
		// rList.add(0.95);
		// rList.add(0.975);
		List<Integer> sList = new ArrayList<Integer>();
		sList.add(2);
		sList.add(4);
		// sList.add(8);
		// sList.add(16);
		List<Integer> magList = new ArrayList<Integer>();
		magList.add(1);
		magList.add(2);
		// magList.add(3);
		// magList.add(4);
		// magList.add(5);
		// magList.add(6);
		// magList.add(7);
		for (double P : PList) {
			for (double r : rList) {
				for (int s : sList) {
					for (int mag : magList) {
						int size = (int) Math.round(Math.pow(isize, mag));
						int m0 = BloomMath.optimalSliceSize(isize);
						test(size, P, r, s, m0, mag);
					}
				}
			}
		}
	}

	protected void test(int size, double P, double r, int s, int m0, int mag) {
		double p0 = BloomMath.optimalSBFP0(P, r);
		IScalableBloomFilter<Long> filter = createFilter(size, p0, r, s, m0);
		double falsePosProb = testAndGetFalsePositives(size, filter, P);

		double maxErr = 1.05; // +5%
		double minErr = 2; // -200%
		try {
			double expected = P * maxErr;
			Assert.assertTrue("False positive probability is " + falsePosProb
					+ " but expected less than " + expected,
					falsePosProb <= expected);
			expected = p0 / minErr;
			Assert.assertTrue("False positive probability is " + falsePosProb
					+ " but expected more than " + expected,
					falsePosProb >= expected);
		} catch (Error t) {
			System.out.println("size=" + size + " P0=" + P + " r=" + r + " s="
					+ s + " m0=" + m0 + " mag=" + mag + " falsePos="
					+ falsePosProb + " filter=" + filter + " " + t.toString());
			throw t;
		}

		// try {
		// System.out.println("size=" + size + " P0=" + P + " r=" + r + " s="
		// + s + " m0=" + m0 + " mag=" + mag + " falsePos="
		// + falsePosProb + " stages=" + sbf.getStages()
		// + " allocateBits=" + sbf.totalBits);
		// CSVWriter writer = new CSVWriter(new FileWriter("yourfile.csv",
		// true));
		// String[] entries = new String[] { "" + size, "" + P, "" + r,
		// "" + s, "" + m0, "" + mag, "" + falsePosProb,
		// "" + sbf.getStages(), "" + sbf.getTotalAllocatedBits() };
		// writer.writeNext(entries);
		// writer.close();
		// } catch (Exception e) {
		// throw new RuntimeException(e);
		// }
	}

}
