/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.filter.bloom.scalable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;

import java.util.Random;

import org.jhears.filter.bloom.Hasher;
import org.jhears.util.DefaultHashFunction;
import org.jhears.util.HashFunction;
import org.jhears.util.LCG;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;

@RunWith(MockitoJUnitRunner.class)
public class HashCreatorTest {

	@Test
	public void testHashValues() {
		HashFunction<Object> genericHash = new DefaultHashFunction<Object>();
		Hasher creator = new Hasher(genericHash, new LCG());

		Random r = new Random(0);

		for (int i = 0; i < 100; i++) {
			int[] hashValues = creator.hash(r.nextLong(), i);
			for (int hash : hashValues) {
				assertTrue(hash >= 0);
			}
		}

	}

	@SuppressWarnings("unchecked")
	@Test
	public void testLongValues() {
		HashFunction<Object> genericHash = mock(HashFunction.class);
		Hasher creator = new Hasher(genericHash, new LCG());
		long value = 7;
		int[] retval = creator.hash(value, 3);
		assertEquals(3, retval.length);
		verify(genericHash, never()).hash(new Object());
	}

	@SuppressWarnings("unchecked")
	@Test
	public void testObject() {
		HashFunction<Object> genericHash = mock(HashFunction.class);
		Hasher creator = new Hasher(genericHash, new LCG());
		Long value = Long.valueOf(7);
		int[] retval = creator.hash(value, 3);
		assertEquals(3, retval.length);
		verify(genericHash).hash(value);
	}
}
