/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.util;

import java.util.NoSuchElementException;

import org.junit.Assert;
import org.junit.Test;

public class ArrayIteratorTest {

	@Test(expected = NoSuchElementException.class)
	public void testEmpty() {
		ArrayIterator<Integer> ii = new ArrayIterator<Integer>(new Integer[0]);
		Assert.assertFalse(ii.hasNext());
		ii.next();
	}

	@Test
	public void testEmptyHasNext() {
		ArrayIterator<Integer> ii = new ArrayIterator<Integer>(new Integer[0]);
		Assert.assertFalse(ii.hasNext());
		Assert.assertFalse(ii.hasNext());
		Assert.assertFalse(ii.hasNext());
		Assert.assertFalse(ii.hasNext());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNull() {
		new ArrayIterator<Integer>(null);
	}

	@Test
	public void testSingle() {
		ArrayIterator<Integer> ii = new ArrayIterator<Integer>(
				new Integer[] { Integer.valueOf(0) });
		Assert.assertTrue(ii.hasNext());
		Assert.assertEquals(Integer.valueOf(0), ii.next());
		Assert.assertFalse(ii.hasNext());
	}

	@Test
	public void testSingleHasNext() {
		ArrayIterator<Integer> ii = new ArrayIterator<Integer>(
				new Integer[] { Integer.valueOf(0) });
		Assert.assertTrue(ii.hasNext());
		Assert.assertTrue(ii.hasNext());
		Assert.assertTrue(ii.hasNext());
		Assert.assertTrue(ii.hasNext());
	}

	@Test
	public void testThree() {
		ArrayIterator<Integer> ii = new ArrayIterator<Integer>(new Integer[] {
				Integer.valueOf(0), Integer.valueOf(1), Integer.valueOf(2) });
		Assert.assertTrue(ii.hasNext());
		Assert.assertEquals(Integer.valueOf(0), ii.next());
		Assert.assertTrue(ii.hasNext());
		Assert.assertEquals(Integer.valueOf(1), ii.next());
		Assert.assertTrue(ii.hasNext());
		Assert.assertEquals(Integer.valueOf(2), ii.next());
		Assert.assertFalse(ii.hasNext());
	}

	@Test
	public void testTwo() {
		ArrayIterator<Integer> ii = new ArrayIterator<Integer>(new Integer[] {
				Integer.valueOf(0), Integer.valueOf(1) });
		Assert.assertTrue(ii.hasNext());
		Assert.assertEquals(Integer.valueOf(0), ii.next());
		Assert.assertTrue(ii.hasNext());
		Assert.assertEquals(Integer.valueOf(1), ii.next());
		Assert.assertFalse(ii.hasNext());
	}
}
