/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.junit.Assert;
import org.junit.Test;

public class IntListTest {

	@Test
	public void testAddAll() {
		IntList ail1 = new IntList();
		for (int i = 0; i < 128; i++) {
			ail1.add(-i);
		}
		IntList ail2 = new IntList();
		for (int i = 128; i < 256; i++) {
			ail2.add(i);
		}
		ail1.addAll(ail2);
		assertEquals(256, ail1.size());
		for (int i = 0; i < 128; i++) {
			assertEquals(-i, ail1.get(i));
		}
		for (int i = 128; i < 256; i++) {
			assertEquals(i, ail1.get(i));
		}
		IntList ail3 = new IntList();
		ail3.addAll(ail1);
		assertEquals(ail1.size(), ail3.size());
		for (int i = 0; i < 128; i++) {
			assertEquals(-i, ail3.get(i));
		}
		for (int i = 128; i < 256; i++) {
			assertEquals(i, ail3.get(i));
		}
	}

	@Test
	public void testAddGet() {
		IntList ail = new IntList();
		for (int i = 0; i < 128; i++) {
			ail.add(-i);
			assertEquals(-i, ail.get(i));
			assertFalse(ail.isEmpty());
			assertEquals(i + 1, ail.size());
		}
	}
	@Test
	public void testAddGetWithZeroIndex() {
		IntList ail = new IntList();
		for (int i = 0; i < 128; i++) {
			ail.add(0,-i);
			assertEquals(-i, ail.get(0));
			assertFalse(ail.isEmpty());
			assertEquals(i + 1, ail.size());
		}
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testAddGet2() {
		IntList ail = new IntList();
		ail.add(12);
		ail.get(1);
	}

	@Test
	public void testConstructor1() {
		IntList ail = new IntList(0);
		assertTrue(ail.isEmpty());
		assertTrue(ail.size() == 0);
	}

	@Test
	public void testConstructor2() {
		IntList ail = new IntList(1);
		assertTrue(ail.isEmpty());
		assertTrue(ail.size() == 0);
	}

	@Test
	public void testDefaultConstructor() {
		IntList ail = new IntList();
		assertTrue(ail.isEmpty());
		assertTrue(ail.size() == 0);
	}

	@Test
	public void testRandom() {
		IntList ail = new IntList();
		List<Integer> ilist = new ArrayList<Integer>();
		Random r = new Random();
		for (int i = 0; i < 10000; i++) {
			switch (r.nextInt(5)) {
			case 0: {
				int value = r.nextInt();
				ail.add(value);
				ilist.add(Integer.valueOf(value));
				break;
			}
			case 1: {
				if (!ilist.isEmpty()) {
					int index = r.nextInt(ilist.size());
					int expected = ilist.remove(index).intValue();
					int actual = ail.remove(index);
					Assert.assertEquals(expected, actual);
				}
				break;
			}
			case 2: {
				Assert.assertEquals(ilist.size(), ail.size());
				break;
			}
			case 3: {
				if (!ilist.isEmpty()) {
					int index = r.nextInt(ilist.size());
					int expected = ilist.get(index).intValue();
					int actual = ail.get(index);
					Assert.assertEquals(expected, actual);
				}
				break;
			}
			case 4: {
				if (!ilist.isEmpty()) {
					int value = r.nextInt();
					int index = r.nextInt(ilist.size());
					int expected = ilist.set(index, Integer.valueOf(value))
							.intValue();
					int actual = ail.set(index, value);
					Assert.assertEquals(expected, actual);
				}
				break;
			}
			default:
				break;
			}
		}
	}

	@Test
	public void testRemove() {
		IntList ail = new IntList();
		List<Integer> ilist = new ArrayList<Integer>();
		for (int i = 0; i < 32; i++) {
			ail.add(i);
			ilist.add(Integer.valueOf(i));
		}
		Random r = new Random();
		while (!ilist.isEmpty()) {
			int index = r.nextInt(ilist.size());
			int expected = ilist.remove(index).intValue();
			int actual = ail.remove(index);
			Assert.assertEquals(expected, actual);
		}
	}
}
