/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cnf;

import java.security.SecureRandom;
import java.util.Arrays;
import java.util.UUID;

import org.apache.commons.configuration.BaseConfiguration;
import org.apache.commons.configuration.Configuration;

public enum Cnf {

	audioPrePros(CnfManager.audioConfPropName, CnfManager.audioConfFname,
			AudioPreProsKey.class),

	calibrator(CnfManager.calibConfPropName, CnfManager.calibConfFname,
			CalibratorKey.class),

	client(CnfManager.clientConfPropName, CnfManager.clientConfFname,
			ClientKey.class),

	fingerprint(CnfManager.fingerprintConfPropName,
			CnfManager.fingerprintConfFname, FingerprintKey.class),

	framework(CnfManager.fwConfPropName, CnfManager.fwConfFname, Framewrk.class),

	search(CnfManager.searchConfPropName, CnfManager.searchConfFname,
			SearchKey.class),

	server(CnfManager.serverConfPropName, CnfManager.serverConfFname,
			ServerKey.class);

	public static final String DEFAULT_BURN_FILTER_VALUE = "1";

	/** Configuration property key names for audio preprocessing. */
	public static class AudioPreProsKey {

		static final String _PREFIX = "prepros";

		public static final String CMD = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "cmd";

		@InternalProperty
		public static final String CNST_PRE_PROC_DIR_NAME = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "preprocessed-audio";

		public static final String FMT = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "formats";

		public static final String INPUT_PIPE_CHAIN_CMD_LIST = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR + "input-pipe-cmd-list";

		public static final String PIPE_TO_RAW = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "pipeToRaw";
		public static final String PIPE_BUFFER_SIZE = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "pipe-buffer-size";

		// public static final String SAMPLE_FILTER_LOADER_CLASSs = PREFIX
		// + SEPARATOR + _PREFIX + SEPARATOR
		// + "sample-filter-loader-class";

		@InternalProperty
		public static final String STR_SUBST_INPUT_FMT = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "inputFormat";

	}

	/** Configuration property key names for calibrator. */
	public static class CalibratorKey {

		static final String _PREFIX = "calibrator";

		public static final String ANALYSIS_TIMEOUT = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "analysis-timeout-in-seconds";

		public static final String CNST_ANALYSIS_DIR_NAME = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "processed-samples";

		public static final String CNST_CODEBOOK_DIR_NAME = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "codebooks";

		// public static final String CNST_PRE_PROC_LOG_PROP = PREFIX +
		// SEPARATOR
		// + _PREFIX + SEPARATOR + "audio-preprocessor-log.properties";

		public static final String ELBG_EPSILON1 = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "elbg-epsilon1";

		public static final String ELBG_EPSILON2 = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "elbg-epsilon2";

		public static final String MAX_ZERO_VALUE = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "max-zero-value";

		public static final String SEED = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "seed";

		public static final String SKIP_SAMPLES_CONST = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "skip-samples-const";

		public static final String SKIP_SAMPLES_MAX_RND = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "skip-samples-max-rnd";

		public static final String LOUDNESS_CMD = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "loudness.cmd";

	}

	/** Configuration property key names for client. */
	public static class ClientKey {

		static final String _PREFIX = "client";

		public static final String SERVER_HOST = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "server-host";

		public static final String SERVER_PASSWORD = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "server-password";

		public static final String SERVER_PATH = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "server-path";

		public static final String SERVER_PORT = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "server-port";

		public static final String SERVER_SCHEME = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "server-scheme";

		public static final String SERVER_USERNAME = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "server-username";

	}

	/** Configuration property key names for fingerprint. */
	public static class FingerprintKey {

		static final String _PREFIX = "fingerprint";

		public static final String BARK_BAND_STOP_COUNT = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "bark-band-stop-count";

		public static final String BARK_BAND_STOPS = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "bark-band-stops";

		public static final String BYTES_PER_SAMPLE = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "bytes-per-sample";

		public static final String CODEBOOK = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "codebook";

		public static final String CODEBOOK_SYMBOL_COUNT = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "codebook-symbols";

		public static final String CONFIGURATION_UUID = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "configuration-uuid";

		public static final String CREATION_DATE = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "configuration-creation-date";

		public static final String SAMPLE_INTERVAL = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "sample-interval";

		public static final String SAMPLE_RATE = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "sample-rate";

		public static final String SAMPLE_WINDOW = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "sample-window";

		public static final String SEGMENT_LENGTH = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "segment-length";

		public static final String SEGMENT_MAX_MUTATIONS = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "segment-max-mutations";

		public static final String SYMBOLS = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "symbols";

		public static final String VERSION = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "version";

		public static final String ZERO_SPECTRUM_TRESHOLD = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "zero-spectrum-treshold";

		public static final String ZERO_SYMBOL_IDX = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "zero-symbol-idx";

		public static final String BURN_FILTER_POWER = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "burn-filter-power";

		public static final String BURN_FILTER_VALUES = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "burn-filter-values";

		public static final String LOUDNESS_FILTER_VALUES = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "loudness-filter-values";

		public static final String SPECTRUM_SMOOTH_ROUNDS = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "spectrum-smooth-rounds";

		public static final String SPECTRUM_SMOOTH_WINDOW = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "spectrum-smooth-window";
		
	}

	/** Configuration property key names for ?. */
	public static class Framewrk {

		static final String _PREFIX = "framework";

		public static final String CACHE_DIGEST_ALGORITHM = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "cache-digest-algorithm";

		public static final String CACHE_DIGEST_BUFFER_SIZE = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR + "cache-digest-buffer-size";

		public static final String CACHE_OVERWRITE_EXISTING = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR + "cache-overwrite-existing";

	}

	/** Configuration property key names for search. */
	public static class SearchKey {

		static final String _PREFIX = "search";

		public static final String COMPLEXITY_MARGIN = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "complexity-margin";

		public static final String COMPLEXITY_TRESHOLD = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "complexity-treshold";

		public static final String INSERT_MAX_SEQ_LEN = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "insert-max-seq-length";

		public static final String INSERT_MIN_SEQ_LEN = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "insert-min-seq-length";

		public static final String MAX_CLUSTER_DISTANCE = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "max-cluster-distance";

		public static final String MAX_DUPLICATE_SEGMENTS = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "max-duplicate-segments";

		public static final String MAX_SEQ_LEN = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "max-seq-length";

		public static final String MIN_SEARCH_SCORE = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "min-search-score";

		public static final String MIN_SEQ_LEN = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "min-seq-length";

		public static final String MIN_UNIQ_SEGMENTS = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "min-unique-segments";

		public static final String MINIMUM_HITS = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "minimum-hits";

		public static final String NONCOMPLEX_MARGIN = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "noncomplex-margin";

		public static final String REPEAT_IGNORE_CHAR = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "repeat-ignore-char";

		public static final String REPEAT_MAX_CLUSTER_DISTANCE = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "repeats-max-cluster-distance";

		public static final String REPEAT_MIN_CLUSTER_SIZE = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "repeats-min-cluster-size";

		public static final String REPEAT_MIN_LENGHT = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "repeat-min-length";

		public static final String REPEAT_QUANTIZER_CODEBOOK = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR + "repeat-quantizer-codebook";

		public static final String REPEAT_QUANTIZER_CONSTANT_MARGIN = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "repeat-quantizer-constant-margin";

		public static final String REPEAT_QUANTIZER_MAX_MULTIPLIER = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "repeat-quantizer-max-multiplier";

		public static final String REPEAT_QUANTIZER_MIN_MULTIPLIER = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "repeat-quantizer-min-multiplier";

		public static final String REPEATS_FOR_INSERT = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "minimum-repeats-for-insert";

		public static final String REPEATS_PER_GROUP = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "repeats-per-group";

		public static final String SEQ_MATCH_THRESHOLD = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "seq-match-threshold";

		public static final String SEQUENCE_ERROR_MARGIN = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "sequence-error-margin";

		public static final String SEQUENCE_MATCH_THRESHOLD = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR + "sequence-match-threshold";

		public static final String SEQUENCE_SMOOTHERS = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "sequence-smoothers";

	}

	public static class ServerKey {

		static final String _PREFIX = "server";

		public static final String SECURITY = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "security";

		public static final String DATADIR = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "data-dir";

		public static final String SEGMENT_CACHE_SBF_ERROR_PROB = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "segment-cache-sbf-error-probability";

		public static final String REPEAT_GROUP_CACHE_SBF_ERROR_PROB = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "repeat-group-cache-sbf-error-probability";

		public static final String SEGMENT_CACHE_SBF_INITIAL_SIZE = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "segment-cache-sbf-initial-size";

		public static final String REPEAT_GROUP_CACHE_SBF_INITIAL_SIZE = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "repeat-group-cache-sbf-initial-size";

		public static final String IDENTIFIER = PREFIX + SEPARATOR + _PREFIX
				+ SEPARATOR + "identifier";

		public static final String RECAPTCHA_SECURE = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "reCpatchaSecure";

		public static final String RECAPTCHA_PUBLIC_KEY = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "reCaptchaPublicKey";

		public static final String RECAPTCHA_PRIVATE_KEY = PREFIX + SEPARATOR
				+ _PREFIX + SEPARATOR + "reCaptchaPrivateKey";

		public static final String RECAPTCHA_CONNECTION_TIMEOUT = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR
				+ "reCaptchaConnectionTimeout";

		public static final String RECAPTCHA_SOCKET_TIMEOUT = PREFIX
				+ SEPARATOR + _PREFIX + SEPARATOR + "reCaptchaSocketTimeout";

	}

	/** Prefix for all jHears configuration properties. */
	public static final String PREFIX = "jhears";

	/** Separator for configuration properties. */
	public static final String SEPARATOR = ".";

	private final String fileName;

	private final Class<?> keyClass;

	private final String propertyName;

	private Cnf(String propertyName, String fileName, Class<?> keyClass) {
		this.propertyName = propertyName;
		this.fileName = fileName;
		this.keyClass = keyClass;
	}

	public Configuration getDefaultConfig() {
		Configuration cnf = new BaseConfiguration();

		switch (this) {
		case calibrator:

			cnf.setProperty(CalibratorKey.SEED, new SecureRandom().nextLong());

			cnf.setProperty(CalibratorKey.SKIP_SAMPLES_CONST,
					Integer.valueOf(1 << 8));

			cnf.setProperty(CalibratorKey.SKIP_SAMPLES_MAX_RND,
					Integer.valueOf(1 << 8));

			cnf.setProperty(CalibratorKey.ANALYSIS_TIMEOUT, 30);

			cnf.setProperty(CalibratorKey.MAX_ZERO_VALUE, 0.0001);

			cnf.setProperty(CalibratorKey.ELBG_EPSILON1, 0.0001);

			cnf.setProperty(CalibratorKey.ELBG_EPSILON2, 0.01);

			cnf.setProperty(CalibratorKey.LOUDNESS_CMD, new String[] { "sox",
					"-n", "-t", ".s2", "-r",
					"${" + FingerprintKey.SAMPLE_RATE + "}", "-c", "1", "-",
					"synth", "3600", "noise", "gain", "-18", "loudness", "+15",
					"gain", "-15", "loudness", "+15", "gain", "-15", "loudness", "+15" });

//			cnf.setProperty(CalibratorKey.LOUDNESS_CMD, new String[] { "sox",
//					"-n", "-t", ".s2", "-r",
//					"${" + FingerprintKey.SAMPLE_RATE + "}", "-c", "1", "-",
//					"synth", "3600", "noise", "gain", "-28", "loudness", "+15",
//					"gain", "-15", "loudness", "+15", "gain", "-15",
//					"loudness", "+15", "gain", "-15", "loudness", "+15" });
			break;
		case fingerprint:
			cnf.setProperty(FingerprintKey.SAMPLE_RATE, 8192);
			// this value might need some tuning
			cnf.setProperty(FingerprintKey.SAMPLE_INTERVAL, 1024);
			cnf.setProperty(FingerprintKey.ZERO_SPECTRUM_TRESHOLD, 0.015);
			int sampleWindow = 8192;
			cnf.setProperty(FingerprintKey.SAMPLE_WINDOW, sampleWindow);
			cnf.setProperty(FingerprintKey.BARK_BAND_STOPS, new String[] {
					"100", "200", "300", "400", "510", "630", "770", "920",
					"1080", "1270", "1480", "1720", "2000", "2320", "2700",
					"3150", "3700" });
			cnf.setProperty(
					FingerprintKey.BARK_BAND_STOP_COUNT,
					cnf.getStringArray(FingerprintKey.BARK_BAND_STOPS).length - 1);
			cnf.setProperty(FingerprintKey.CODEBOOK_SYMBOL_COUNT, 64);

			cnf.setProperty(FingerprintKey.SEGMENT_LENGTH, 8);
			cnf.setProperty(FingerprintKey.SEGMENT_MAX_MUTATIONS, 8);
			cnf.setProperty(FingerprintKey.ZERO_SYMBOL_IDX, 63);

			cnf.setProperty(FingerprintKey.BURN_FILTER_POWER, 4);
			String[] bvalues = new String[sampleWindow / 2];
			Arrays.fill(bvalues, DEFAULT_BURN_FILTER_VALUE);
			cnf.setProperty(FingerprintKey.BURN_FILTER_VALUES, bvalues);

			cnf.setProperty(FingerprintKey.LOUDNESS_FILTER_VALUES,
					Arrays.copyOf(bvalues, bvalues.length));

			cnf.setProperty(FingerprintKey.SYMBOLS,
					"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/");
			cnf.setProperty(FingerprintKey.BYTES_PER_SAMPLE, 2);
			
			cnf.setProperty(FingerprintKey.SPECTRUM_SMOOTH_ROUNDS, 6);
			cnf.setProperty(FingerprintKey.SPECTRUM_SMOOTH_WINDOW, 6);

			break;
		case audioPrePros:
			cnf.setProperty(AudioPreProsKey.FMT, "AUDIO FILE FORMATS: ");
			cnf.setProperty(AudioPreProsKey.CMD, "sox");
			cnf.setProperty(AudioPreProsKey.INPUT_PIPE_CHAIN_CMD_LIST,
					new String[] { AudioPreProsKey.PIPE_TO_RAW });

			cnf.setProperty(AudioPreProsKey.PIPE_TO_RAW, new String[] { "sox",
					"-t", "${" + AudioPreProsKey.STR_SUBST_INPUT_FMT + "}",
					"-", "-t", ".s2", "-", "remix", "-", "rate",
					"${" + FingerprintKey.SAMPLE_RATE + "}" });
			cnf.setProperty(AudioPreProsKey.PIPE_BUFFER_SIZE,
					Integer.valueOf(1 << 20));
			// cnf.setProperty(AudioPreProsKey.SAMPLE_FILTER_LOADER_CLASS,
			// "org.jhears.sample.filter.BurnFilterLoader");

			break;
		case search:
			/*
			 * 16 bit sample has maximum amplitude of 32768. White noise has
			 * average amplitude of ~6144. Typical audio data has roughly
			 * average amplitude of 3072. SoX loudness filter decreases audio
			 * roughly to halve from the normal or more. By setting limit to
			 * roughly 10 % from normal might be reasonable. That that would be
			 * 3072 / 2 * 0.1 = 153.6.
			 */
			cnf.setProperty(SearchKey.MIN_UNIQ_SEGMENTS, 3);
			cnf.setProperty(SearchKey.MAX_DUPLICATE_SEGMENTS, 3);
			cnf.setProperty(SearchKey.COMPLEXITY_TRESHOLD, 6);
			cnf.setProperty(SearchKey.COMPLEXITY_MARGIN, 2);
			cnf.setProperty(SearchKey.MINIMUM_HITS, 2);

			cnf.setProperty(SearchKey.NONCOMPLEX_MARGIN, 2);

			cnf.setProperty(SearchKey.SEQUENCE_ERROR_MARGIN, 0.381966);

			cnf.setProperty(SearchKey.INSERT_MAX_SEQ_LEN, 4096);
			cnf.setProperty(SearchKey.INSERT_MIN_SEQ_LEN, 16);

			cnf.setProperty(SearchKey.MAX_SEQ_LEN, 4096);
			cnf.setProperty(SearchKey.MIN_SEQ_LEN, 16);

			cnf.setProperty(SearchKey.MAX_CLUSTER_DISTANCE, 2);
			cnf.setProperty(SearchKey.MIN_SEARCH_SCORE, 3);

			cnf.setProperty(SearchKey.SEQ_MATCH_THRESHOLD, 0.8f);

			int[] repeatSearchCodebook = new int[] { 7, 10, 13, 18, 25, 35, 49,
					67, 93, 128, 177, 245, 338, 467, 646, 892, 1233, 1704, 2355 };
			cnf.setProperty(SearchKey.REPEAT_QUANTIZER_CODEBOOK,
					repeatSearchCodebook);
			cnf.setProperty(SearchKey.REPEAT_QUANTIZER_CONSTANT_MARGIN, 1);
			cnf.setProperty(SearchKey.REPEAT_QUANTIZER_MIN_MULTIPLIER, 0.9);
			cnf.setProperty(SearchKey.REPEAT_QUANTIZER_MAX_MULTIPLIER, 1.1);

			// index in groups of four
			cnf.setProperty(SearchKey.REPEATS_PER_GROUP, 4);

			// if symbol rate is 8 then 8 gives us 1 second margin. This is
			// relative distance.
			cnf.setProperty(SearchKey.REPEAT_MAX_CLUSTER_DISTANCE, 8);

			// require 2 repeat groups (each containing 4 repeats)
			cnf.setProperty(SearchKey.REPEAT_MIN_CLUSTER_SIZE, 8);

			// ignore less than 7 chars long repeats
			cnf.setProperty(SearchKey.REPEAT_MIN_LENGHT, 7);

			// the ZERO_SYMBOL_IDX from SYMBOLS
			cnf.setProperty(SearchKey.REPEAT_IGNORE_CHAR, "/");

			cnf.setProperty(SearchKey.SEQUENCE_MATCH_THRESHOLD, 0.8f);

			// require 4 repeat groups (each containing 4 repeats)
			cnf.setProperty(SearchKey.REPEATS_FOR_INSERT, 16);

			// values are read in pairs. the first value is the minimum
			// same count. and the second is maximum allowed gap.
			int[] sequenceSmoothers = new int[] { 3, 1, 4, 2, 6, 3, 8, 4 };
			cnf.setProperty(SearchKey.SEQUENCE_SMOOTHERS, sequenceSmoothers);

			break;
		case framework:

			cnf.setProperty(Framewrk.CACHE_DIGEST_ALGORITHM, "MD5");

			cnf.setProperty(Framewrk.CACHE_DIGEST_BUFFER_SIZE, 1 << 18);

			cnf.setProperty(Framewrk.CACHE_OVERWRITE_EXISTING, false);

			break;
		case client:

			cnf.setProperty(ClientKey.SERVER_HOST, "localhost");

			cnf.setProperty(ClientKey.SERVER_PASSWORD, "password");

			cnf.setProperty(ClientKey.SERVER_PATH, "");

			cnf.setProperty(ClientKey.SERVER_PORT, "8080");

			cnf.setProperty(ClientKey.SERVER_SCHEME, "http");

			cnf.setProperty(ClientKey.SERVER_USERNAME, "username");

			break;
		case server:

			String fs = System.getProperty("file.separator");

			cnf.setProperty(ServerKey.DATADIR, "var" + fs + "data" + fs);

			cnf.setProperty(ServerKey.SEGMENT_CACHE_SBF_ERROR_PROB, 0.001);

			cnf.setProperty(ServerKey.REPEAT_GROUP_CACHE_SBF_ERROR_PROB, 0.001);

			cnf.setProperty(ServerKey.SEGMENT_CACHE_SBF_INITIAL_SIZE, 1 << 18);

			cnf.setProperty(ServerKey.REPEAT_GROUP_CACHE_SBF_INITIAL_SIZE,
					1 << 18);

			cnf.setProperty(ServerKey.SECURITY, Boolean.FALSE);

			cnf.setProperty(ServerKey.IDENTIFIER, UUID.randomUUID());

			cnf.setProperty(ServerKey.RECAPTCHA_SECURE, false);

			cnf.setProperty(ServerKey.RECAPTCHA_PRIVATE_KEY, null);

			cnf.setProperty(ServerKey.RECAPTCHA_PUBLIC_KEY, null);

			cnf.setProperty(ServerKey.RECAPTCHA_CONNECTION_TIMEOUT, 8000);

			cnf.setProperty(ServerKey.RECAPTCHA_SOCKET_TIMEOUT, 8000);

			break;
		default:
			throw new IllegalStateException(
					"Cannot determine default configuration for " + this);
		}
		return cnf;
	}

	public String getFileName() {
		return fileName;
	}

	public Class<?> getKeyClass() {
		return keyClass;
	}

	public String getPropertyName() {
		return propertyName;
	}

}
