/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.concurrent;

import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang.NullArgumentException;

/**
 * Class interrupts a given thread if {@link #cancel()} is not called before
 * timeout expires.
 * 
 */
public class ThreadInterrupter extends Thread {

	private final Semaphore semaphore;

	private final Thread thread;

	private final long timeout;

	private final TimeUnit unit;

	public ThreadInterrupter(Thread thread, long timeout, TimeUnit unit) {
		this.thread = thread;
		this.timeout = timeout;
		this.unit = unit;
		if (thread == null) {
			throw new NullArgumentException("thread");
		}
		if (unit == null) {
			throw new NullArgumentException("uni");
		}
		if (timeout < 1L) {
			throw new IllegalArgumentException("timeout");
		}
		semaphore = new Semaphore(1);
	}

	public void cancel() {
		semaphore.release();
	}

	@Override
	public void run() {
		try {
			if (!semaphore.tryAcquire(timeout, unit)) {
				thread.interrupt();
			}
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
		}
	}

}
