/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.exec;

import java.util.Arrays;
import java.util.concurrent.Semaphore;
import java.util.concurrent.atomic.AtomicBoolean;

import org.apache.commons.exec.ExecuteException;
import org.apache.commons.exec.ExecuteResultHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DefaultExecuteResultHandler implements ExecuteResultHandler {

	private final AtomicBoolean retval;
	private final Semaphore semaphore;
	private final String[] cmd;
	private final Logger logger;

	public DefaultExecuteResultHandler(final AtomicBoolean retval,
			final Semaphore semaphore, final String[] cmd) {
		this.logger = LoggerFactory
				.getLogger(DefaultExecuteResultHandler.class);
		this.retval = retval;
		this.semaphore = semaphore;
		this.cmd = cmd;
	}

	@Override
	public void onProcessComplete(int exitValue) {
		try {
			if (logger.isDebugEnabled()) {
				logger.debug("Execution completed with exit value: "
						+ exitValue + ". command: " + Arrays.toString(cmd));
			}
			retval.compareAndSet(true, exitValue == 0);
		} finally {
			semaphore.release();
		}
	}

	@Override
	public void onProcessFailed(ExecuteException e) {
		try {
			logger.error("Execution failed: " + Arrays.toString(cmd), e);
			retval.compareAndSet(true, false);
		} finally {
			semaphore.release();
		}
	}

}
