/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.exec;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;

import org.apache.commons.exec.CommandLine;
import org.apache.commons.exec.DefaultExecutor;
import org.apache.commons.exec.ExecuteWatchdog;
import org.apache.commons.exec.PumpStreamHandler;
import org.jhears.pipe.Pipe;

public final class ExecUtil {

	private ExecUtil() {
	}

	public static CommandLine createCommandLine(final String[] cmd) {
		CommandLine cmdline;
		if (cmd.length > 1) {
			cmdline = new CommandLine(cmd[0]);
			cmdline.addArguments(Arrays.copyOfRange(cmd, 1, cmd.length));
		} else {
			cmdline = new CommandLine(cmd[0]);
		}
		return cmdline;
	};

	public static DefaultExecutor createExecutor(long timoutInMillis, Pipe pipe) {
		DefaultExecutor executor = new DefaultExecutor();
		ExecuteWatchdog watchdog = new ExecuteWatchdog(timoutInMillis);

		OutputStream out = pipe.getOut();
		OutputStream err = pipe.getErr();
		InputStream in = pipe.getIn();
		PumpStreamHandler psh = new ClosePumpStreamHandler(out, err, in);
		executor.setStreamHandler(psh);
		executor.setWatchdog(watchdog);
		executor.setExitValue(0);
		return executor;
	}

}
