/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.file;

import java.io.File;
import java.io.IOException;

import org.apache.commons.codec.binary.Hex;
import org.jhears.security.Digester;

/**
 * Class maps a file to another file based on the digest of the first file
 * contents. The name of the mapped file is hex encoded digest of the first
 * file. Mapped files are located in a directory give in at the constructor.
 */
public class FileContentMapper {

	private final Digester digester;

	private final File dir;

	public FileContentMapper(Digester digester, File dir) {
		this.digester = digester;
		this.dir = dir;
	}

	private String digest(File value) throws IOException {
		return new String(Hex.encodeHex(digester.digest(value)));
	}

	public String getAlgorithm() {
		return digester.getAlgorithm();
	}

	public File getDir() {
		return dir;
	}

	/**
	 * Maps file to another by using file contents.
	 * 
	 * @param file
	 *            File to be mapped.
	 * @return a file which name is a hex encoded digest of the contents of the
	 *         file given in argument.
	 * @throws IOException
	 */
	public File map(File file) throws IOException {
		return new File(dir, digest(file));
	}

}
