/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.jar;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import org.jhears.util.RuntimeIOException;

public class ManifestUtil {

	public static final String MANIFEST_RESOURCE_NAME = "META-INF/MANIFEST.MF";

	/**
	 * Convenience method. Returns a
	 * {@link Attributes.Name#IMPLEMENTATION_VERSION} from a first manifest
	 * which has a matching attribute key <code>key</code> with value
	 * <code>value</code>.
	 * 
	 * @param key
	 * @param value
	 * @return value of a manifest
	 *         {@link Attributes.Name#IMPLEMENTATION_VERSION} attribute or a
	 *         null on failure.
	 * @throws IOException
	 */
	public static String getImplementationVersion(Object key, Object value) {
		List<Attributes> list = ManifestUtil.getManifest(key, value);
		Attributes a;
		if (list.isEmpty()) {
			a = new Attributes();
		} else {
			a = list.get(0);
		}
		return a.getValue(Attributes.Name.IMPLEMENTATION_VERSION);
	}

	/**
	 * Returns {@link Attributes} of all {@link Manifest} which contains a
	 * attribute key <code>key</code> with value <code>value</code>.
	 * 
	 * @param key
	 * @param value
	 * @return
	 * @throws IOException
	 */
	public static List<Attributes> getManifest(Object key, Object value) {
		List<Attributes> list = new ArrayList<Attributes>();
		try {
			Enumeration<URL> resources = Thread.currentThread()
					.getContextClassLoader().getResources(
							MANIFEST_RESOURCE_NAME);
			while (resources.hasMoreElements()) {
				URL resource = resources.nextElement();
				InputStream stream = resource.openStream();
				Manifest manifest = new Manifest(stream);
				stream.close();
				Attributes attributes = manifest.getMainAttributes();
				if (attributes.containsKey(key)
						&& attributes.get(key).equals(value)) {
					list.add(attributes);
				}
			}
			return list;
		} catch (IOException e) {
			throw new RuntimeIOException("Error while accessing manifest", e);
		}
	}

}
