/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.math;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.jhears.io.DelayedWriter;

public class MatrixUtil {

	private MatrixUtil() {
	}

	public static void transposeMatrix(final File srcMatrix,
			final File dstMatrix, final int rows, final int cols,
			final int cellSize, final int readBufSize, final int writeBufSize)
			throws IOException {
		final DataInputStream is = new DataInputStream(new BufferedInputStream(
				new FileInputStream(srcMatrix), readBufSize));
		try {
			final DelayedWriter raf = new DelayedWriter(new RandomAccessFile(
					dstMatrix, "rw"), writeBufSize);
			try {
				final byte[] row = new byte[cellSize * cols];
				for (int r = 0; r < rows; r++) {
					is.readFully(row);
					for (int c = 0; c < cols; c++) {
						final long pos = (((long) c) * rows + r) * cellSize;
						final int from = c * cellSize;
						raf.write(pos, row, from, cellSize);
					}
				}
			} finally {
				raf.close();
			}
		} finally {
			is.close();
		}
	}
}
