/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.math;

import org.apache.commons.math.stat.StatUtils;

/**
 * Holds statistical data about double array such as mean, median and variance.
 */
public class Stat {

	private final double max;

	private final double mean;

	private final double median;

	private final double min;

	private final double q1;

	private final double q3;

	private final int size;

	private final double variance;

	public Stat(final double[] v) {
		min = StatUtils.min(v);
		max = StatUtils.max(v);
		mean = StatUtils.mean(v);
		median = StatUtils.percentile(v, 50.0);
		q1 = StatUtils.percentile(v, 25.0);
		q3 = StatUtils.percentile(v, 75.0);
		variance = StatUtils.variance(v);
		size = v.length;
	}

	public double getMax() {
		return max;
	}

	public double getMean() {
		return mean;
	}

	public double getMedian() {
		return median;
	}

	public double getMin() {
		return min;
	}

	public double getQ1() {
		return q1;
	}

	public double getQ3() {
		return q3;
	}

	public int getSize() {
		return size;
	}

	public double getVariance() {
		return variance;
	}

	@Override
	public String toString() {
		return "size=" + size + " min=" + min + " max=" + max + " mean=" + mean
				+ " median=" + median + " variance=" + variance + " q1=" + q1
				+ " q3=" + q3;
	}

}
