/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.sample;

import org.jhears.util.Mth;

/**
 * Divides values with mean absolute of values thus resulting mean absolute of
 * values to be 1.0. Works the same in frequency and time domain.
 * 
 * @author Juha Heljoranta
 */
public class InplaceGainControlFunction implements
		SampleFunction<double[], Void> {

	/**
	 * Calculate abs and then take the mean value of it
	 * 
	 * @param arr
	 * @return
	 */
	private static double meanAbs(final double[] arr) {
		double val = 0.0;
		for (int i = 0; i < arr.length; i++) {
			val += Math.abs(arr[i]);
		}
		return val / arr.length;
	}

	@Override
	public Void apply(final double[] from) {
		final double mabs = meanAbs(from);
		if (mabs > 0.0) {
			Mth.div(from, mabs);
		}
		return null;
	}

}
