/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.sample;

import org.jhears.util.Mth;

import com.google.common.base.Preconditions;

public class InplaceHannWindowFunction implements
		SampleFunction<double[], Void> {

	private final double[] hannWindow;
	private final int sampleWindowInSamples;

	public InplaceHannWindowFunction(int sampleWindowInSamples) {
		Preconditions.checkArgument(sampleWindowInSamples > 0,
				"Sample window must be positive but was %s",
				Integer.toString(sampleWindowInSamples));
		this.sampleWindowInSamples = sampleWindowInSamples;
		hannWindow = getHannWindow(sampleWindowInSamples);
	}

	@Override
	public Void apply(double[] from) {
		Preconditions.checkArgument(from.length == sampleWindowInSamples,
				"Array length must be %s but was %s",
				Integer.toString(sampleWindowInSamples),
				Integer.toString(from.length));
		Mth.mult(from, hannWindow);
		return null;
	}

	private double[] getHannWindow(final int length) {
		final double[] v = new double[length];
		for (int i = 0; i < length; i++) {
			v[i] = 0.5 * (1.0 - Math.cos((2.0 * Math.PI * i) / (length - 1)));
		}
		return v;
	}

}