/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.sample;

import com.google.common.base.Preconditions;

public class InplaceLoudnessFilterFunction implements
		SampleFunction<double[], Void> {

	private final double[] loudnessFilter;

	public InplaceLoudnessFilterFunction(double[] loudnessFilter) {
		Preconditions.checkNotNull(loudnessFilter,
				"Loundness filter cannot be null");
		this.loudnessFilter = loudnessFilter;
	}

	@Override
	public Void apply(final double[] from) {
		Preconditions.checkArgument(loudnessFilter.length == from.length,
				"Expected input data length=%s but was %s",
				Integer.toString(loudnessFilter.length),
				Integer.toString(from.length));
		for (int i = 0; i < loudnessFilter.length; i++) {
			from[i] *= loudnessFilter[i];
		}
		return null;
	}

}