/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample;

public class RawToPowSpectFunction implements SampleFunction<byte[], double[]> {

	private final InplaceHannWindowFunction hannWindowFun;
	private final RawToSampleFunction rawToSampleFun;
	private final InplaceRemoveDcFunction removeDcFun;
	private final SampleToPowerSpectrumFunction sampleToPowerSpectrumFun;

	public RawToPowSpectFunction(RawToSampleFunction rawToSampleFun,
			InplaceRemoveDcFunction removeDcFun,
			InplaceHannWindowFunction hannWindowFun,
			SampleToPowerSpectrumFunction sampleToPowerSpectrumFun) {
		this.rawToSampleFun = rawToSampleFun;
		this.removeDcFun = removeDcFun;
		this.hannWindowFun = hannWindowFun;
		this.sampleToPowerSpectrumFun = sampleToPowerSpectrumFun;
	}

	@Override
	public double[] apply(final byte[] data) {
		final double[] sample = rawToSampleFun.apply(data);
		removeDcFun.apply(sample);
		hannWindowFun.apply(sample);
		// note: do not try to put in log scale.
		// it makes search unreliable.
		return sampleToPowerSpectrumFun.apply(sample);
	}

}