/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.sample;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class RawToSampleFunction implements SampleFunction<byte[], double[]> {

	/** 2^15 */
	private static final double MAX_SHORT_VALUE = (double) (1 << 15);

	private static double[] castToDoubles(final short[] arr) {
		final double[] dArr = new double[arr.length];
		for (int i = 0; i < arr.length; i++) {
			dArr[i] = arr[i] / MAX_SHORT_VALUE;
		}
		return dArr;
	}

	private final ByteOrder byteOrder;

	private final int sampleWindowInSamples;

	public RawToSampleFunction(int sampleWindowInSamples, ByteOrder byteOrder) {
		this.sampleWindowInSamples = sampleWindowInSamples;
		this.byteOrder = byteOrder;
	}

	@Override
	public double[] apply(byte[] from) {
		short[] shorts = new short[sampleWindowInSamples];
		ByteBuffer.wrap(from).order(byteOrder).asShortBuffer().get(shorts);
		return castToDoubles(shorts);
	}

}