/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample;


public class RawToSmoothedFunction implements SampleFunction<byte[], double[]> {

	private final InplaceGainControlFunction gainCtrlFun;
	private final InplaceLoudnessFilterFunction loudnessFun;
	private final RawToPowSpectFunction rawToPowSpectFun;
	private final SilenceFilterFunction silenceFun;
	private final SmoothFunction smoothFun;

	public RawToSmoothedFunction(RawToPowSpectFunction rawToPowSpectFun,
			InplaceLoudnessFilterFunction loudnessFun,
			SilenceFilterFunction silenceFun,
			InplaceGainControlFunction gainCtrlFun, SmoothFunction smoothFun) {
		this.rawToPowSpectFun = rawToPowSpectFun;
		this.loudnessFun = loudnessFun;
		this.silenceFun = silenceFun;
		this.gainCtrlFun = gainCtrlFun;
		this.smoothFun = smoothFun;
	}

	@Override
	public double[] apply(byte[] data) {
		double[] powerspect = rawToPowSpectFun.apply(data);
		loudnessFun.apply(powerspect);
		powerspect = silenceFun.apply(powerspect);
		if (!silenceFun.isSilence(powerspect)) {
			gainCtrlFun.apply(powerspect);
			return smoothFun.apply(powerspect);
		}
		return powerspect; // silence
	}

	public boolean isSilence(double[] data) {
		return silenceFun.isSilence(data);
	}

}