/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample;

import java.nio.ByteOrder;

import org.apache.commons.configuration.Configuration;
import org.jhears.cnf.Cnf.FingerprintKey;
import org.jhears.cnf.CnfUtil;
import org.picocontainer.MutablePicoContainer;

public class SampleFunctionInjector {

	public static void addFunctionsToContainer(MutablePicoContainer container,
			Configuration configuration) {
		int[] barkBandStops = CnfUtil.toIntArray(configuration
				.getStringArray(FingerprintKey.BARK_BAND_STOPS));
		int sampleRate = configuration.getInt(FingerprintKey.SAMPLE_RATE);
		int sampleWindowInSamples = configuration
				.getInt(FingerprintKey.SAMPLE_WINDOW);
		int freqToPowSpect = sampleRate / sampleWindowInSamples;
		int[][] freqIdxs = barkFreqIndexes(barkBandStops, freqToPowSpect);
		container.addComponent(new SpectrumToBarksFunction(freqIdxs));
		double power = configuration.getInt(FingerprintKey.BURN_FILTER_POWER);
		double[] values = CnfUtil.toDoubleArray(configuration
				.getStringArray(FingerprintKey.BURN_FILTER_VALUES));
		InplaceBurnFunction burnFilterFun = new InplaceBurnFunction(freqIdxs,
				values, power);
		container.addComponent(burnFilterFun);
		container.addComponent(new RawToSampleFunction(sampleWindowInSamples,
				ByteOrder.nativeOrder()));
		container.addComponent(InplaceRemoveDcFunction.class);

		container.addComponent(new InplaceHannWindowFunction(
				sampleWindowInSamples));
		container.addComponent(SampleToPowerSpectrumFunction.class);
		container.addComponent(RawToPowSpectFunction.class);
		double[] loudnessFilter = CnfUtil.toDoubleArray(configuration
				.getStringArray(FingerprintKey.LOUDNESS_FILTER_VALUES));
		container
				.addComponent(new InplaceLoudnessFilterFunction(loudnessFilter));
		double threshold = configuration
				.getDouble(FingerprintKey.ZERO_SPECTRUM_TRESHOLD);
		double[] silence = new double[sampleWindowInSamples / 2];
		container.addComponent(new SilenceFilterFunction(threshold, silence));
		container.addComponent(InplaceGainControlFunction.class);
		container.addComponent(RawToSmoothedFunction.class);
		container.addComponent(RawToBarksFunction.class);
		container.addComponent(InplaceLogFunction.class);
		int smoothRounds = configuration
				.getInt(FingerprintKey.SPECTRUM_SMOOTH_ROUNDS);
		int smoothWindow = configuration
				.getInt(FingerprintKey.SPECTRUM_SMOOTH_WINDOW);
		container.addComponent(new SmoothFunction(smoothRounds, smoothWindow));

		container.start();
	}

	private static int[][] barkFreqIndexes(int[] barkBandStops,
			int freqToPowSpect) {
		int barks = barkBandStops.length - 1;
		int[][] frequenciesByBark = new int[barks][];
		for (int b = 0; b < barks; b++) {
			int bstart = barkBandStops[b] / freqToPowSpect;
			int bstop = barkBandStops[b + 1] / freqToPowSpect;
			frequenciesByBark[b] = new int[bstop - bstart];
			int i = 0;
			for (int f = bstart; f < bstop; f++) {
				frequenciesByBark[b][i++] = f;
			}
		}
		return frequenciesByBark;
	}

}
