/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.sample;

import org.apache.commons.math.MathException;
import org.apache.commons.math.complex.Complex;
import org.apache.commons.math.transform.FastFourierTransformer;

import com.google.common.base.Throwables;

public class SampleToPowerSpectrumFunction implements
		SampleFunction<double[], double[]> {

	/**
	 * Calculate power spectrum of given sample.
	 * 
	 * @return array length {@code (from.length / 2)} where all values are
	 *         {@code >= 0}.
	 */
	@Override
	public double[] apply(final double[] from) {
		final FastFourierTransformer fft = new FastFourierTransformer();
		final Complex[] ca;
		try {
			ca = fft.transform(from);
		} catch (IllegalArgumentException e) {
			throw Throwables.propagate(e);
		} catch (MathException e) {
			throw Throwables.propagate(e);
		}
		// throw away upper half since its mirrored copy of the lower half
		final double[] spect = new double[from.length / 2];
		// we can skip the zero hz band so the i = 1
		for (int i = 1; i < spect.length; i++) {
			spect[i] = ca[i].abs();
		}
		return spect;
	}

}