/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample;

public class SampleUtil {

	/**
	 * Root mean square of values. Pseudo code: {@code sqrt(sum(x^2)/n)}.
	 * 
	 * @param values
	 * @return
	 */
	public static double rms(final double[] values) {
		double rms = 0.0;
		for (int i = 0; i < values.length; i++) {
			rms += values[i] * values[i];
		}
		return Math.sqrt(rms / values.length);
	}

	/**
	 * Warning: not a generic version. Root mean square of frequency domain
	 * first half of {@code X=abs(FFT(x))}. Pseudo code:
	 * {@code sqrt(sum(X^2)*2)/(2*n)}.
	 * 
	 * @param powerSpectrum
	 * @return
	 */
	public static double rmsFreqDomain(final double[] powerSpectrum) {
		double rms = 0.0;
		for (int i = 0; i < powerSpectrum.length; i++) {
			rms += powerSpectrum[i] * powerSpectrum[i];
		}
		// multiply with 2 since the second (mirrored) half is missing
		return Math.sqrt(rms * 2.0) / (powerSpectrum.length * 2);
	}

	private SampleUtil() {

	}

}
