/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.sample;

import org.jhears.util.Mth;

/**
 * Removes peaks and noise from spectrum.
 * 
 * @author Juha Heljoranta
 * 
 */
public class SmoothFunction implements SampleFunction<double[], double[]> {

	private final int rounds;

	private final int window;

	/**
	 * 
	 * @param rounds
	 *            The number of times to apply simple moving average.
	 * @param window
	 *            The number of elements for average window.
	 */
	public SmoothFunction(int rounds, int window) {
		this.rounds = rounds;
		this.window = window;
	}

	@Override
	public double[] apply(double[] from) {
		for (int i = 0; i < rounds; i++) {
			from = Mth.simpleMovingAverage(from, window);
		}
		return from;
	}

}