/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.sample;

import java.util.Arrays;

import org.jhears.util.Mth;

public class SpectrumToBarksFunction implements
		SampleFunction<double[], double[]> {

	private final int[][] freqIdxs;

	// private BufferedOutputStream bos;

	public SpectrumToBarksFunction(int[][] freqIdxs) {
		this.freqIdxs = freqIdxs;
		// try {
		// String name = "/tmp/spect." + System.currentTimeMillis() + ".dat";
		// System.err.println(name);
		// bos = new BufferedOutputStream(new FileOutputStream(new File(name)));
		// } catch (Exception e) {
		// throw Throwables.propagate(e);
		// }
	}

	// private void write(final double[] from) {
	// byte[] data = new byte[from.length * 8];
	// double[] a = new double[from.length];
	// for (int i = 0; i < from.length; i++) {
	// a[i] = from[i] * 0.001;
	// }
	// ByteBuffer.wrap(data).order(ByteOrder.nativeOrder()).asDoubleBuffer()
	// .put(a);
	// try {
	// bos.write(data);
	// } catch (IOException e) {
	// throw Throwables.propagate(e);
	// }
	// }

	@Override
	public double[] apply(final double[] from) {
		// write(from);
		final double[] barks = new double[freqIdxs.length];
		for (int b = 0; b < freqIdxs.length; b++) {
			int start = freqIdxs[b][0];
			int end = freqIdxs[b][freqIdxs[b].length - 1] + 1;
			double[] sortedBarkFreqs = Arrays.copyOfRange(from, start, end);
			Arrays.sort(sortedBarkFreqs);
			double[] cs = Mth.cumulativeSum(sortedBarkFreqs);
			// find a balance point point in terms of weight. think as a
			// mechanical equilibrium point for an array.
			int meanCsIdx = Arrays.binarySearch(cs, cs[cs.length - 1] * 0.5);
			if (meanCsIdx < 0) {
				meanCsIdx = -meanCsIdx - 1;
			}
			// take mean of upper part of the array
			barks[b] = Mth.mean(sortedBarkFreqs, meanCsIdx,
					sortedBarkFreqs.length);
		}
		return barks;
	}

	/**
	 * Returns an array of zeroes.
	 * 
	 * @return
	 */
	public double[] getSilence() {
		return new double[freqIdxs.length];
	}

	// private double meanRange(final int[] indexes, final double[] values) {
	// double retval = 0.0;
	// for (final int i : indexes) {
	// retval += values[i];
	// }
	// return retval / indexes.length;
	// }

}