/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample.filter;

import java.util.Iterator;

import org.apache.commons.configuration.Configuration;
import org.jhears.cnf.Cnf;
import org.jhears.util.Mth;

public final class LoudnessFilterCalibrator {

	private final Configuration configuration;

	public LoudnessFilterCalibrator(Configuration configuration) {
		this.configuration = configuration;
	}

	/**
	 * 
	 * @param powSpectIterator
	 *            power spectrums of white noise which are loudness corrected
	 *            (that is, frequencies around 1-2kHz are most amplified. See
	 *            loudness in wikipedia for details).
	 * @return Multipliers to make spectrum loudness corrected.
	 */
	public double[] calibrate(Iterator<double[]> powSpectIterator) {
		double[] spect = new double[configuration
				.getInt(Cnf.FingerprintKey.SAMPLE_WINDOW) / 2];
		int cnt = 0;
		while (powSpectIterator.hasNext()) {
			double[] powSpect = powSpectIterator.next();
			double[] smooth = powSpect;
			smooth = Mth.simpleMovingAverage(smooth, 3);
			Mth.add(spect, smooth);
			cnt++;
		}
		// smooth it
		for (int i = 0; i < 2048; i++) {
			spect = Mth.simpleMovingAverage(spect, 2);
		}
		// average
		Mth.div(spect, cnt);
		// to scale 0..1
		Mth.div(spect, Mth.max(spect));
		// reset 0hz
		spect[0] = 0.0;
		return spect;
	}
	
}
