/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.Collection;

/**
 * Allows to combine multiple {@link ISmoother}s to single smooth operation.
 * <p>
 * This class is thread safe.
 */
public class CombinedSmoother implements ISmoother {

	private final Collection<ISmoother> smoothers;

	/**
	 * 
	 * @param smoothers
	 *            Smoothers to perform smoothing. Smoothers are applied in order
	 *            they are returned by the collection iterator.
	 * @throws NullPointerException
	 *             If argument is null.
	 * @throws IllegalArgumentException
	 *             If collection is empty.
	 * @see SequenceSmoother
	 */
	public CombinedSmoother(Collection<ISmoother> smoothers)
			throws NullPointerException, IllegalArgumentException {
		if (smoothers.isEmpty()) {
			throw new IllegalArgumentException("Must define at least one "
					+ SequenceSmoother.class.getSimpleName());
		}
		this.smoothers = smoothers;
	}

	/**
	 * Applies {@link SequenceSmoother#smooth(char[])} in same order as iterator
	 * returns the smoothers.
	 * 
	 * @param ca
	 * @see SequenceSmoother#smooth(char[])
	 */
	@Override
	public void smooth(char[] ca) {
		for (ISmoother smoother : smoothers) {
			smoother.smooth(ca);
		}
	}

}
