/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.Arrays;

import org.apache.commons.configuration.Configuration;
import org.jhears.cnf.Cnf.FingerprintKey;

public class FSeqFactory {

	private final char[] chars;
	private final int maxMutations;
	private final char min;
	private final int[] rev;
	private final int segmentLength;

	public FSeqFactory(Configuration fpConf) {
		segmentLength = fpConf.getInt(FingerprintKey.SEGMENT_LENGTH);
		maxMutations = fpConf.getInt(FingerprintKey.SEGMENT_MAX_MUTATIONS);
		if (maxMutations < 0 || maxMutations > MutableFSeqImpl.MAX_MUT_LEN) {
			throw new IllegalArgumentException("maxMutations is out of range");
		}
		if (segmentLength < 1) {
			throw new IllegalArgumentException(
					"segmentLength must be greater than zero");
		}
		if (segmentLength < maxMutations) {
			throw new IllegalArgumentException("segmentLength < maxMutations");
		}
		chars = fpConf.getString(FingerprintKey.SYMBOLS).toCharArray();
		/*
		 * create "reverse" mapping from char array. This allows to validate
		 * user input quickly.
		 */
		char[] cpy = Arrays.copyOf(chars, chars.length);
		Arrays.sort(cpy);
		min = cpy[0];
		char max = cpy[cpy.length - 1];
		rev = new int[max - min + 1];
		Arrays.fill(rev, -1);
		for (int i = 0; i < chars.length; i++) {
			rev[chars[i] - min] = i;
		}
	}

	public MutableFSeq getNewInstance() {
		return new MutableFSeqImpl(segmentLength, maxMutations, min, chars, rev);
	}
}
