/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.Iterator;

import org.jhears.web.rs.Segment;

public class ImmutableFSeq implements FSeq {

	private final int length;
	private final int maxMutations;
	private final int mutations;
	private final int segmentLength;
	private final String seq;

	ImmutableFSeq(String seq, int length, int mutations, int segmentLength,
			int maxMutations) {
		this.seq = seq;
		this.length = length;
		this.mutations = mutations;
		this.segmentLength = segmentLength;
		this.maxMutations = maxMutations;
	}

	@Override
	public String asString() {
		return seq;
	}

	@Override
	public int getFullSegmentCount() {
		return length / segmentLength;
	}

	@Override
	public int getLength() {
		return length;
	}

	@Override
	public int getMaxMutations() {
		return maxMutations;
	}

	@Override
	public int getMutations() {
		return mutations;
	}

	@Override
	public int getSegmentLength() {
		return segmentLength;
	}

	@Override
	public Iterator<Segment> iterator() {
		return new SegmentIterator(segmentLength, seq.toCharArray());
	}

}
