/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.ArrayList;
import java.util.List;

import org.jhears.web.rs.Repeat;

/**
 * Run length encoder for character sequences.
 * <p>
 * This class is thread safe.
 */
public class RLEnc {

	private final char ignore;

	private final int minLength;

	/**
	 * 
	 * @param minLength
	 *            The number of occurrences the character must have in row
	 *            before encoding it.
	 * @param ignore
	 *            Character to ignore.
	 */
	public RLEnc(final int minLength, final char ignore) {
		this.minLength = minLength;
		this.ignore = ignore;
	}

	/**
	 * Encodes parts of the array which length is greater than or equal to
	 * minLengt.
	 * 
	 * @param ca
	 * @param minLength
	 * @return
	 */
	public List<Repeat> encode(final char[] ca) {
		ArrayList<Repeat> retval = new ArrayList<Repeat>();
		final int len = ca.length;
		if (len == 0) {
			return retval; // nothing to do
		}
		char previous = ca[0];
		int sameCnt = 1;
		int i = 1;
		for (; i < len; i++) {
			if (ca[i] == previous) {
				sameCnt++;
			} else {
				if (sameCnt >= minLength && previous != ignore) {
					retval.add(new Repeat(previous, i - sameCnt, sameCnt));
				}
				previous = ca[i];
				sameCnt = 1;
			}
		}
		if (sameCnt >= minLength && previous != ignore) {
			retval.add(new Repeat(previous, i - sameCnt, sameCnt));
		}
		return retval;
	}

}
