/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.Arrays;
import java.util.Iterator;
import java.util.NoSuchElementException;

import org.jhears.web.rs.Segment;

/**
 * Iterates over mutating string.
 * <p>
 * This class doesn't do any input validation. Users of this class are
 * responsible of valid input.
 */
class SegmentIterator implements Iterator<Segment> {

	private final char[] ca;
	private boolean esc;
	private int i;
	private final MutatingStringIterator mi;
	private int pos;
	private final int segLength;
	private final int size;

	/**
	 * 
	 * @param segLength
	 *            segment length
	 * @param d
	 *            string data to be iterated
	 */
	public SegmentIterator(final int segLength, final char[] d) {
		this.segLength = segLength;
		this.ca = d;
		size = d.length;
		esc = false;
		pos = 0;
		i = 0;
		mi = new MutatingStringIterator(segLength);
	}

	@Override
	public boolean hasNext() {
		if (!mi.hasNext()) {
			return parseNextEntry();
		}
		return true;
	}

	@Override
	public Segment next() {
		if (hasNext()) {
			return new Segment(mi.next(), pos - segLength);
		} else {
			throw new NoSuchElementException();
		}
	}

	private boolean parseNextEntry() {
		for (; i < size; i++) {
			if (esc) {
				esc = false;
				int len = Character.digit(ca[i], 16);
				char[] row = Arrays.copyOfRange(ca, i + 1, i + len + 1);
				mi.insertRow(row);
				i += len;
			} else {
				pos++;
				esc = ca[i] == '\\';
				if (!esc) {
					mi.insertRow(new char[] { ca[i] });
				}
			}
			if (mi.hasNext()) {
				i++;
				return true;
			}
		}
		return false;
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}

}
