/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.serialization;

import java.nio.ByteOrder;

public final class CharSerialization {

	private CharSerialization() {
	}

	/**
	 * Encodes char array to byte array. Byte order is
	 * {@link ByteOrder#BIG_ENDIAN}.
	 * 
	 * @param input
	 * @return
	 */
	public static byte[] toByteArray(final char[] input) {
		// same as; ByteBuffer.wrap(output).asCharBuffer().put(input);
		final byte[] output = new byte[input.length * 2];
		for (int i = 0; i < input.length; i++) {
			int j = i * 2;
			output[j] = (byte) ((input[i] >> 8) & 0xff);
			output[j + 1] = (byte) (input[i] & 0xff);
		}
		return output;
	}

	/**
	 * Decodes byte array to char array. Byte order is
	 * {@link ByteOrder#BIG_ENDIAN}.
	 * 
	 * @param input
	 * @return
	 */
	public static char[] toCharArray(final byte[] input) {
		// same as: ByteBuffer.wrap(input).asCharBuffer().get(output);
		final char[] output = new char[input.length / 2];
		for (int i = 0; i < input.length;) {
			output[i / 2] = (char) (((input[i++] & 0xff) << 8) | (input[i++] & 0xff));
		}
		return output;
	}

}
