/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cluster;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

public class ClusterDetectorTest {

	@Test
	public void testNoClusterScattered() {
		int[] input = new int[] { 0, 5, 10, 15 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testEmpty() {
		int[] input = new int[] {};
		List<Cluster> expecteds = new ArrayList<Cluster>();
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testNoClusterTwoOrphan() {
		int[] input = new int[] { 0, 0, 5, 5 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testNoClusterTooShort() {
		int[] input = new int[] { 0, 0 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testSingleStacked() {
		int[] input = new int[] { 0, 0, 0 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 0, 1, 2 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testTwoStacked() {
		int[] input = new int[] { 0, 0, 0, 5, 5, 5 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 0, 1, 2 }));
		expecteds.add(new Cluster(new int[] { 3, 4, 5 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testTwoStackedWithAGap() {
		int[] input = new int[] { 0, 0, 0, 4, 8, 8, 8 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 0, 1, 2 }));
		expecteds.add(new Cluster(new int[] { 4, 5, 6 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testTwoStackedWithGapFixedLow() {
		int[] input = new int[] { 0, 0, 0, 3, 8, 8, 8 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 0, 1, 2, 3 }));
		expecteds.add(new Cluster(new int[] { 4, 5, 6 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testTwoStackedWithGapFixedHigh() {
		int[] input = new int[] { 0, 0, 0, 5, 8, 8, 8 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 0, 1, 2, }));
		expecteds.add(new Cluster(new int[] { 3, 4, 5, 6 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testStackInMiddle() {
		int[] input = new int[] { 0, 10, 10, 10, 20 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 1, 2, 3 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testSparseStack() {
		int[] input = new int[] { 0, 3, 6, 9, 12 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 0, 1, 2, 3, 4 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testSparseStackInMiddle() {
		int[] input = new int[] { -4, 0, 3, 6, 9, 12, 16 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 1, 2, 3, 4, 5 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testStackWithParseInMiddle() {
		int[] input = new int[] { -4, 0, 3, 6, 7, 7, 8, 8, 9, 12, 16 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9 }));
		List<Cluster> actuals = new ClusterDetector(3, 3).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

	@Test
	public void testMinSize() {
		int[] input = new int[] { 0, 0, 5, 5 };
		List<Cluster> expecteds = new ArrayList<Cluster>();
		expecteds.add(new Cluster(new int[] { 0, 1 }));
		expecteds.add(new Cluster(new int[] { 2, 3 }));
		List<Cluster> actuals = new ClusterDetector(3, 2).detectClusters(input);
		Assert.assertEquals(expecteds, actuals);
	}

}
