/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.io;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.junit.Assert;
import org.junit.Test;

public class DelayedWriterTest {

	// private static final class EntryImpl implements Entry<Long, byte[]> {
	//
	// private final Long key;
	// private byte[] value;
	//
	// public EntryImpl(Long key, byte[] value) {
	// this.key = key;
	// this.value = value;
	// }
	//
	// @Override
	// public boolean equals(Object obj) {
	// if (this == obj)
	// return true;
	// if (obj == null)
	// return false;
	// if (getClass() != obj.getClass())
	// return false;
	// EntryImpl other = (EntryImpl) obj;
	// if (key == null) {
	// if (other.key != null)
	// return false;
	// } else if (!key.equals(other.key))
	// return false;
	// if (!Arrays.equals(value, other.value))
	// return false;
	// return true;
	// }
	//
	// @Override
	// public Long getKey() {
	// return key;
	// }
	//
	// @Override
	// public byte[] getValue() {
	// return value;
	// }
	//
	// @Override
	// public int hashCode() {
	// final int prime = 31;
	// int result = 1;
	// result = prime * result + ((key == null) ? 0 : key.hashCode());
	// result = prime * result + Arrays.hashCode(value);
	// return result;
	// }
	//
	// @Override
	// public byte[] setValue(byte[] value) {
	// byte[] retval = this.value;
	// this.value = value;
	// return retval;
	// }
	//
	// @Override
	// public String toString() {
	// return EntryImpl.class.getSimpleName() + " [key=" + key
	// + ", value=" + Arrays.toString(value) + "]";
	// }
	// }

	private static byte[] ba(int... bs) {
		byte[] rval = new byte[bs.length];
		for (int i = 0; i < bs.length; i++) {
			rval[i] = (byte) bs[i];
		}
		return rval;
	}

	private static final class SimpleEntry implements Map.Entry<Long, byte[]> {

		private final Long key;
		private byte[] value;

		SimpleEntry(Long key, byte[] value) {
			this.key = key;
			this.value = value;
		}

		@Override
		public Long getKey() {
			return key;
		}

		@Override
		public byte[] getValue() {
			return value;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((key == null) ? 0 : key.hashCode());
			result = prime * result + Arrays.hashCode(value);
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			SimpleEntry other = (SimpleEntry) obj;
			if (key == null) {
				if (other.key != null)
					return false;
			} else if (!key.equals(other.key))
				return false;
			if (!Arrays.equals(value, other.value))
				return false;
			return true;
		}

		@Override
		public byte[] setValue(byte[] value) {
			byte[] retval = this.value;
			this.value = value;
			return retval;
		}

	}

	private Entry<Long, byte[]> makeEntry(long l, byte[] bs) {
		return new SimpleEntry(Long.valueOf(l), bs);
	}

	@Test
	public void testMergeHead() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(1L, ba(2)));
		elist.add(makeEntry(3L, ba(3)));
		elist.add(makeEntry(5L, ba(4)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(3, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1, 2)), elist.get(0));
		Assert.assertEquals(makeEntry(3L, ba(3)), elist.get(1));
		Assert.assertEquals(makeEntry(5L, ba(4)), elist.get(2));
	}

	@Test
	public void testMergeTail() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(2L, ba(2)));
		elist.add(makeEntry(4L, ba(3)));
		elist.add(makeEntry(5L, ba(4)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(3, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1)), elist.get(0));
		Assert.assertEquals(makeEntry(2L, ba(2)), elist.get(1));
		Assert.assertEquals(makeEntry(4L, ba(3, 4)), elist.get(2));
	}

	@Test
	public void testMergeThree() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(1L, ba(2)));
		elist.add(makeEntry(2L, ba(3)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(1, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1, 2, 3)), elist.get(0));
	}

	@Test
	public void testMergeTwo() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(1L, ba(2)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(1, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1, 2)), elist.get(0));
	}

	@Test
	public void testMergeTwoInMiddle() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(2L, ba(2)));
		elist.add(makeEntry(3L, ba(3)));
		elist.add(makeEntry(5L, ba(4)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(3, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1)), elist.get(0));
		Assert.assertEquals(makeEntry(2L, ba(2, 3)), elist.get(1));
		Assert.assertEquals(makeEntry(5L, ba(4)), elist.get(2));
	}

	@Test
	public void testMergeThreeInMiddle() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(2L, ba(2)));
		elist.add(makeEntry(3L, ba(3)));
		elist.add(makeEntry(4L, ba(4)));
		elist.add(makeEntry(6L, ba(5)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(3, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1)), elist.get(0));
		Assert.assertEquals(makeEntry(2L, ba(2, 3, 4)), elist.get(1));
		Assert.assertEquals(makeEntry(6L, ba(5)), elist.get(2));
	}

	@Test
	public void testMergeFourInMiddle() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(2L, ba(2)));
		elist.add(makeEntry(3L, ba(3)));
		elist.add(makeEntry(4L, ba(4)));
		elist.add(makeEntry(5L, ba(5)));
		elist.add(makeEntry(7L, ba(6)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(3, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1)), elist.get(0));
		Assert.assertEquals(makeEntry(2L, ba(2, 3, 4, 5)), elist.get(1));
		Assert.assertEquals(makeEntry(7L, ba(6)), elist.get(2));
	}

	@Test
	public void testEmpty() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(0, elist.size());
	}

	@Test
	public void testMergeFourInHead() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(2L, ba(2)));
		elist.add(makeEntry(3L, ba(3)));
		elist.add(makeEntry(4L, ba(4)));
		elist.add(makeEntry(5L, ba(5)));
		elist.add(makeEntry(7L, ba(6)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(2, elist.size());
		Assert.assertEquals(makeEntry(2L, ba(2, 3, 4, 5)), elist.get(0));
		Assert.assertEquals(makeEntry(7L, ba(6)), elist.get(1));
	}

	@Test
	public void testMergeFourInTail() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(2L, ba(2)));
		elist.add(makeEntry(3L, ba(3)));
		elist.add(makeEntry(4L, ba(4)));
		elist.add(makeEntry(5L, ba(5)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(2, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1)), elist.get(0));
		Assert.assertEquals(makeEntry(2L, ba(2, 3, 4, 5)), elist.get(1));
	}

	@Test
	public void testOne() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(1, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1)), elist.get(0));
	}

	@Test
	public void testTwoGapping() {
		List<Entry<Long, byte[]>> elist = new ArrayList<Entry<Long, byte[]>>();
		elist.add(makeEntry(0L, ba(1)));
		elist.add(makeEntry(2L, ba(2)));
		elist = DelayedWriter.optimize(elist);
		Assert.assertEquals(2, elist.size());
		Assert.assertEquals(makeEntry(0L, ba(1)), elist.get(0));
		Assert.assertEquals(makeEntry(2L, ba(2)), elist.get(1));
	}
}
