/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.io;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.NoSuchElementException;

import org.junit.Test;

public class StreamIteratorTest {

	private byte[] getTestData(int size) {
		byte[] testData = new byte[size];
		for (int i = 0; i < size; i++) {
			testData[i] = (byte) i;
		}
		return testData;
	}

	@Test(expected = NoSuchElementException.class)
	public void read_1_1_1_4() {
		int chunkSize = 1;
		int stepSize = 1;
		int bufSize = 1;
		int testSize = 4;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 1 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 3 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_2_2_2_6() {
		int chunkSize = 2;
		int stepSize = 2;
		int bufSize = 2;
		int testSize = 6;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 2, 3 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 4, 5 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_2_7_8() {
		int chunkSize = 3;
		int stepSize = 2;
		int bufSize = 7;
		int testSize = 8;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 2, 3, 4 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 4, 5, 6 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_2_7_9() {
		int chunkSize = 3;
		int stepSize = 2;
		int bufSize = 7;
		int testSize = 9;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 2, 3, 4 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 4, 5, 6 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 6, 7, 8 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_5_7_10() {
		int chunkSize = 3;
		int stepSize = 5;
		int bufSize = 7;
		int testSize = 10;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 5, 6, 7 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_5_7_11() {
		int chunkSize = 3;
		int stepSize = 5;
		int bufSize = 7;
		int testSize = 11;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 5, 6, 7 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_5_7_12() {
		int chunkSize = 3;
		int stepSize = 5;
		int bufSize = 7;
		int testSize = 12;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 5, 6, 7 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_5_7_13() {
		int chunkSize = 3;
		int stepSize = 5;
		int bufSize = 7;
		int testSize = 13;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 5, 6, 7 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 10, 11, 12 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_5_7_8() {
		int chunkSize = 3;
		int stepSize = 5;
		int bufSize = 7;
		int testSize = 8;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 5, 6, 7 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test(expected = NoSuchElementException.class)
	public void read_3_5_7_9() {
		int chunkSize = 3;
		int stepSize = 5;
		int bufSize = 7;
		int testSize = 9;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 5, 6, 7 }, itr.next());
		assertFalse(itr.hasNext());
		itr.next(); // bang
	}

	@Test
	public void read_3_3_6_9() {
		int chunkSize = 3;
		int stepSize = 3;
		int bufSize = 6;
		int testSize = 9;
		byte[] testData = getTestData(testSize);
		InputStream is = new ByteArrayInputStream(testData);
		StreamIterator obj = new StreamIterator(is, chunkSize, stepSize,
				bufSize);
		Iterator<byte[]> itr = obj.iterator();
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 0, 1, 2 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 3, 4, 5 }, itr.next());
		assertTrue(itr.hasNext());
		assertArrayEquals(new byte[] { 6, 7, 8 }, itr.next());
		assertFalse(itr.hasNext());
	}

}
