/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.pipe;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

public class PipelineTest {

	private static ExecutorService executor;

	@BeforeClass
	public static void setupClass() {
		executor = Executors.newFixedThreadPool(64);
	}

	@AfterClass
	public static void cleaupClass() {
		executor.shutdownNow();
	}

	private void testr(int pipes, int bufSize) throws IOException,
			InterruptedException {
		byte[] buf = new byte[512];
		for (int i = 0; i < buf.length; i++) {
			buf[i] = (byte) i;
		}
		ByteArrayInputStream is = new ByteArrayInputStream(buf);
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		OutputStream error = new ByteArrayOutputStream();
		List<Pipe> chain = Pipeline.construct(pipes, is, os, error, bufSize);
		ExecutorCompletionService<Void> service = new ExecutorCompletionService<Void>(
				executor);
		for (Pipe pipe : chain) {
			PipePumpper pumpper = new PipePumpper(pipe);
			service.submit(pumpper, null);
		}
		for (int i = 0; i < pipes; i++) {
			service.poll(pipes * 2, TimeUnit.SECONDS);
		}
		Assert.assertArrayEquals(buf, os.toByteArray());
	}

	@Test
	public void test_1_101() throws IOException, InterruptedException {
		testr(1, 101);
	}

	@Test
	public void test_2_101() throws IOException, InterruptedException {
		testr(2, 101);
	}

	@Test
	public void test_3_101() throws IOException, InterruptedException {
		testr(3, 101);
	}

	@Test
	public void test_4_101() throws IOException, InterruptedException {
		testr(4, 101);
	}

	@Test
	public void test_5_101() throws IOException, InterruptedException {
		testr(4, 101);
	}

	@Test
	public void test_1_256() throws IOException, InterruptedException {
		testr(1, 256);
	}

	@Test
	public void test_2_256() throws IOException, InterruptedException {
		testr(2, 256);
	}

	@Test
	public void test_3_256() throws IOException, InterruptedException {
		testr(3, 256);
	}

	@Test
	public void test_4_256() throws IOException, InterruptedException {
		testr(4, 256);
	}

	@Test
	public void test_5_256() throws IOException, InterruptedException {
		testr(4, 256);
	}
}
