/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample;

import junit.framework.Assert;

import org.junit.Test;

public class InplaceBurnFunctionTest {

	private static final double eps = 0.00000001;

	@Test
	public void test() {
		int C = 4;
		// y = 1
		InplaceBurnFunction fun = new InplaceBurnFunction(
				new int[][] { new int[] { 0, 1, 2, 3 } }, new double[] { 1, 1,
						1, 1 }, C);

		// (1-y^C/(x^C+y^C))*x
		// y^C = 1^4 = 1
		// -> (1-1/(x^C+1))*x
		double[] value = new double[] { 0.5, 1, 2, 3 };
		double[] expected = new double[] { calc(value[0], C),
				calc(value[1], C), calc(value[2], C), calc(value[3], C) };
		fun.apply(value);

		Assert.assertEquals(expected[0], value[0], expected[0] * eps);
		Assert.assertEquals(expected[1], value[1], expected[1] * eps);
		// value[1] = 1 => -50% = 0.5
		Assert.assertEquals(0.5, value[1], 0.5 * eps);
		Assert.assertEquals(expected[2], value[2], expected[2] * eps);
		Assert.assertEquals(expected[3], value[3], expected[3] * eps);
	}

	// (1-1/(x^C+1))*x
	private double calc(double x, double C) {
		return (1.0 - 1.0 / (Math.pow(x, C) + 1.0)) * x;
	}
}
