/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample;

import org.junit.Assert;
import org.junit.Test;

public class InplaceGainControlFunctionTest {

	private static double meanAbs(double[] arr) {
		double val = 0.0;
		for (int i = 0; i < arr.length; i++) {
			val += Math.abs(arr[i]);
		}
		return val / arr.length;
	}

	@Test
	public void test() {
		int samplelen = 8192;
		double freq = 64.0;
		double amplitude = 10.0;
		double[] sample = new double[samplelen];
		for (int i = 0; i < samplelen; i++) {
			double value = Math.sin(2.0 * Math.PI * freq * i / samplelen);
			value = value * amplitude;
			sample[i] = value;
		}
		SampleToPowerSpectrumFunction f = new SampleToPowerSpectrumFunction();
		double[] powspect = f.apply(sample);
		InplaceGainControlFunction ff = new InplaceGainControlFunction();
		Assert.assertEquals(amplitude, meanAbs(powspect), 0.001);
		ff.apply(powspect);
		Assert.assertEquals(1.0, meanAbs(powspect), 0.001);

		// test silence
		powspect = new double[4];
		ff.apply(powspect);
		Assert.assertArrayEquals(new double[4], powspect, 0.0001);
	}

	@Test(expected = NullPointerException.class)
	public void testNull() {
		new InplaceGainControlFunction().apply(null);
	}
	
}
