/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.sample;

import org.junit.Assert;
import org.junit.Test;

public class SpectrumToBarksFunctionTest {

	@Test
	public void testRising() {
		int[][] freqIds = new int[][] { new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8,
				9 } };
		SpectrumToBarksFunction f = new SpectrumToBarksFunction(freqIds);
		double[] b = f.apply(new double[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 });
		Assert.assertArrayEquals(new double[] { (7 + 8 + 9) / 3 }, b, 0.0001);
	}

	@Test
	public void testConstant() {
		int[][] freqIds = new int[][] { new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8,
				9 } };
		SpectrumToBarksFunction f = new SpectrumToBarksFunction(freqIds);
		double[] b = f.apply(new double[] { 10, 10, 10, 10, 10, 10, 10, 10, 10,
				10 });
		Assert.assertArrayEquals(new double[] { (10 + 10 + 10 + 10 + 10) / 5 },
				b, 0.0001);
	}

	@Test
	public void testSorting() {
		int[][] freqIds = new int[][] { new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8,
				9 } };
		SpectrumToBarksFunction f = new SpectrumToBarksFunction(freqIds);
		// second and last are swapped
		double[] b = f.apply(new double[] { 11, 20, 13, 14, 15, 16, 17, 18, 19,
				12 });
		Assert.assertArrayEquals(new double[] { (17 + 18 + 19 + 20) / 4 }, b,
				0.0001);
	}

	@Test
	public void testZeroes() {
		int[][] freqIds = new int[][] { new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8,
				9 } };
		SpectrumToBarksFunction f = new SpectrumToBarksFunction(freqIds);
		// second and last are swapped
		double[] b = f.apply(new double[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 });
		Assert.assertArrayEquals(new double[] { 0 }, b, 0.0001);
	}

	@Test
	public void testOnlyOneNonZero() {
		int[][] freqIds = new int[][] { new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8,
				9 } };
		SpectrumToBarksFunction f = new SpectrumToBarksFunction(freqIds);
		// second and last are swapped
		double[] b = f.apply(new double[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 });
		Assert.assertArrayEquals(new double[] { 1 }, b, 0.0001);
	}
}
