/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import org.junit.Assert;
import org.junit.Test;

public class IntQuantizerTest {

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidCodebook1() {
		int[] codebook = new int[] {};
		int offset = 0;
		double minMultiplier = 1;
		double maxMultiplier = 1;
		new IntQuantizer(codebook, offset, minMultiplier, maxMultiplier);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidCodebook2() {
		int[] codebook = new int[] { 1, 1 };
		int offset = 0;
		double minMultiplier = 1;
		double maxMultiplier = 1;
		new IntQuantizer(codebook, offset, minMultiplier, maxMultiplier);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidCodebook3() {
		int[] codebook = new int[] { 2, 2, 3 };
		int offset = 0;
		double minMultiplier = 1;
		double maxMultiplier = 1;
		new IntQuantizer(codebook, offset, minMultiplier, maxMultiplier);
	}

	@Test
	public void testValidCodebook1() {
		int[] codebook = new int[] { 1 };
		int offset = 0;
		double minMultiplier = 1;
		double maxMultiplier = 1;
		new IntQuantizer(codebook, offset, minMultiplier, maxMultiplier);
	}

	@Test
	public void testValidCodebook2() {
		int[] codebook = new int[] { 1, 2 };
		int offset = 0;
		double minMultiplier = 1;
		double maxMultiplier = 1;
		new IntQuantizer(codebook, offset, minMultiplier, maxMultiplier);
	}

	@Test
	public void test1() {
		int[] codebook = new int[] { 4, 8, 16, 32 };
		int offset = 1;
		double minMultiplier = 0.9;
		double maxMultiplier = 1.1;
		IntQuantizer q = new IntQuantizer(codebook, offset, minMultiplier,
				maxMultiplier);

		int[] actual = q.nearestIndexes(new int[] { 4, 8, 16, 32 });
		Assert.assertArrayEquals(new int[] { 0, 1, 2, 3 }, actual);

		actual = q.nearestIndexes(new int[] { 32, 16, 8, 4 });
		Assert.assertArrayEquals(new int[] { 3, 2, 1, 0 }, actual);

		actual = q.nearestIndexes(new int[] { 3, 7, 15, 31 });
		Assert.assertArrayEquals(new int[] { 0, 1, 2, 3 }, actual);

		actual = q.nearestIndexes(new int[] { 5, 9, 17, 33 });
		Assert.assertArrayEquals(new int[] { 0, 1, 2, 3 }, actual);

		actual = q.nearestIndexes(new int[] { 6, 12, 23, 24 });
		Assert.assertArrayEquals(new int[] { 1, 2, 2, 3 }, actual);
	}

	@Test
	public void test2() {
		int[] codebook = new int[] { 4, 8, 16 };
		int offset = 1;
		double minMultiplier = 0.9;
		double maxMultiplier = 1.1;
		IntQuantizer q = new IntQuantizer(codebook, offset, minMultiplier,
				maxMultiplier);

		int[][] actual = q.nearbyIndexes(new int[] { 4, 8, 16 });
		Assert.assertEquals(2, actual.length);
		Assert.assertArrayEquals(new int[] { 0, 1, 2 }, actual[0]);
		Assert.assertArrayEquals(new int[] { 1, 1, 2 }, actual[1]);

		actual = q.nearbyIndexes(new int[] { 6, 12, 24 });
		Assert.assertEquals(4, actual.length);
		Assert.assertArrayEquals(new int[] { 0, 1, 2 }, actual[0]);
		Assert.assertArrayEquals(new int[] { 1, 1, 2 }, actual[1]);
		Assert.assertArrayEquals(new int[] { 0, 2, 2 }, actual[2]);
		Assert.assertArrayEquals(new int[] { 1, 2, 2 }, actual[3]);

		actual = q.nearbyIndexes(new int[] { 2, 6, 12 });
		Assert.assertEquals(4, actual.length);
		Assert.assertArrayEquals(new int[] { 0, 0, 1 }, actual[0]);
		Assert.assertArrayEquals(new int[] { 0, 1, 1 }, actual[1]);
		Assert.assertArrayEquals(new int[] { 0, 0, 2 }, actual[2]);
		Assert.assertArrayEquals(new int[] { 0, 1, 2 }, actual[3]);
	}

}
