/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.configuration.PropertiesConfiguration;
import org.jhears.cnf.Cnf.FingerprintKey;
import org.jhears.web.rs.Segment;
import org.junit.Assert;
import org.junit.Test;

public class MutableFSeqTest {

	private static final String b64str = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

	private MutableFSeq create(int segmentLength, int maxMutation) {
		PropertiesConfiguration c = new PropertiesConfiguration();
		c.setProperty(FingerprintKey.SEGMENT_LENGTH, segmentLength);
		c.setProperty(FingerprintKey.SEGMENT_MAX_MUTATIONS, maxMutation);
		c.setProperty(FingerprintKey.SYMBOLS, b64str);
		FSeqFactory factory = new FSeqFactory(c);
		return factory.getNewInstance();
	}

	private MutableFSeq create(MutableFSeq f1) {
		return new MutableFSeqImpl((MutableFSeqImpl) f1);
	}

	@Test
	public void testComplexity1() {
		MutableFSeq fa = create(4, 4);
		String s = "aaaa";
		fa.append(s);
		Assert.assertEquals("0000", fa.replaceLowComplexity(4, 3, 0, '0')
				.asString());
	}

	@Test
	public void testComplexity10() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678aaaaaaaa12345678";
		fa.append(s);
		Assert.assertEquals("12345678aa0000aa12345678", fa
				.replaceLowComplexity(4, 3, 2, '0').asString());
	}

	@Test
	public void testComplexity10a() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678aaaaaaaaa12345678";
		fa.append(s);
		Assert.assertEquals("1234567800000000012345678", fa
				.replaceLowComplexity(5, 4, 0, '0').asString());
		Assert.assertEquals("1234567800000000012345678", fa
				.replaceLowComplexity(5, 3, 1, '0').asString());
		Assert.assertEquals("1234567800000000012345678", fa
				.replaceLowComplexity(5, 2, 2, '0').asString());
		Assert.assertEquals("1234567800000000012345678", fa
				.replaceLowComplexity(5, 1, 3, '0').asString());
	}

	@Test
	public void testComplexity10b() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678axaxaxaxa12345678";
		fa.append(s);
		Assert.assertEquals("12345678axaxaxaxa12345678", fa
				.replaceLowComplexity(5, 4, 0, '0').asString());
		Assert.assertEquals("12345678a0000000a12345678", fa
				.replaceLowComplexity(5, 3, 1, '0').asString());
		Assert.assertEquals("12345678a0000000a12345678", fa
				.replaceLowComplexity(5, 2, 2, '0').asString());
		Assert.assertEquals("12345678a0000000a12345678", fa
				.replaceLowComplexity(5, 1, 3, '0').asString());
	}

	@Test
	public void testComplexity10c() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678abcabcabc12345678";
		fa.append(s);
		Assert.assertEquals("12345678abcabcabc12345678", fa
				.replaceLowComplexity(5, 4, 0, '0').asString());
		Assert.assertEquals("12345678abcabcabc12345678", fa
				.replaceLowComplexity(5, 3, 1, '0').asString());
		Assert.assertEquals("12345678ab00000bc12345678", fa
				.replaceLowComplexity(5, 2, 2, '0').asString());
		Assert.assertEquals("12345678ab00000bc12345678", fa
				.replaceLowComplexity(5, 1, 3, '0').asString());
	}

	@Test
	public void testComplexity11() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678aaaaaaaa12345678";
		fa.append(s);
		Assert.assertEquals("12345678aa0000aa12345678", fa
				.replaceLowComplexity(5, 4, 2, '0').asString());
	}

	@Test
	public void testComplexity12() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678aaaaaaaa12345678aaaaaaaa12345678";
		fa.append(s);
		Assert.assertEquals("12345678aa0000aa12345678aa0000aa12345678", fa
				.replaceLowComplexity(5, 4, 2, '0').asString());
	}

	@Test
	public void testComplexity13() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678abababab12345678abababab12345678";
		fa.append(s);
		Assert.assertEquals("1234567800000000123456780000000012345678", fa
				.replaceLowComplexity(4, 2, 0, '0').asString());
	}

	@Test
	public void testComplexity14() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678abababab12345678abababab12345678";
		fa.append(s);
		Assert.assertEquals("12345678ab0000ab12345678ab0000ab12345678", fa
				.replaceLowComplexity(4, 2, 2, '0').asString());
	}

	@Test
	public void testComplexity15() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678abababab12345678abababab12345678";
		fa.append(s);
		Assert.assertEquals("12345678abababab12345678abababab12345678", fa
				.replaceLowComplexity(4, 3, 0, '0').asString());
	}

	@Test
	public void testComplexity16() {
		MutableFSeq fa = create(4, 4);
		String s = "abab";
		fa.append(s);
		Assert.assertEquals("abab", fa.replaceLowComplexity(4, 3, 0, '0')
				.asString());
	}

	@Test
	public void testComplexity17() {
		MutableFSeq fa = create(4, 4);
		String s = "abab";
		fa.append(s);
		Assert.assertEquals("0000", fa.replaceLowComplexity(4, 2, 0, '0')
				.asString());
	}

	@Test
	public void testComplexity2() {
		MutableFSeq fa = create(4, 4);
		String s = "abcd";
		fa.append(s);
		Assert.assertEquals("abcd", fa.replaceLowComplexity(4, 3, 0, '0')
				.asString());
	}

	@Test
	public void testComplexity3() {
		MutableFSeq fa = create(4, 4);
		String s = "1234aaaa1234";
		fa.append(s);
		Assert.assertEquals("123400001234", fa.replaceLowComplexity(4, 3, 0,
				'0').asString());
	}

	@Test
	public void testComplexity4() {
		MutableFSeq fa = create(4, 4);
		String s = "aaaa1234aaaa";
		fa.append(s);
		Assert.assertEquals("000012340000", fa.replaceLowComplexity(4, 3, 0,
				'0').asString());
	}

	@Test
	public void testComplexity5() {
		MutableFSeq fa = create(4, 4);
		String s = "aaaa1234aaaa";
		fa.append(s);
		Assert.assertEquals("000a1234a000", fa.replaceLowComplexity(4, 3, 1,
				'0').asString());
	}

	@Test
	public void testComplexity6() {
		MutableFSeq fa = create(4, 4);
		String s = "aaaaa1234aaaaa";
		fa.append(s);
		Assert.assertEquals("0000a1234a0000", fa.replaceLowComplexity(4, 3, 1,
				'0').asString());
	}

	@Test
	public void testComplexity7() {
		MutableFSeq fa = create(4, 4);
		String s = "aaaaaa1234aaaaaa";
		fa.append(s);
		Assert.assertEquals("00000a1234a00000", fa.replaceLowComplexity(4, 3,
				1, '0').asString());
	}

	@Test
	public void testComplexity8() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678aaaaaaaa12345678";
		fa.append(s);
		Assert.assertEquals("123456780000000012345678", fa
				.replaceLowComplexity(4, 3, 0, '0').asString());
	}

	@Test
	public void testComplexity9() {
		MutableFSeq fa = create(8, 8);
		String s = "12345678aaaaaaaa12345678";
		fa.append(s);
		Assert.assertEquals("12345678a000000a12345678", fa
				.replaceLowComplexity(4, 3, 1, '0').asString());
	}

	@Test
	public void testConst1() {
		create(1, 0);
	}

	@Test
	public void testConst10() {
		MutableFSeq f1 = create(4, 4);
		f1.append("\\5abcde");
		MutableFSeq f2 = create(f1);
		f2.append("\\2XY");
		Assert.assertEquals("\\5abcdeX", f2.asString());
	}

	@Test
	public void testConst11() {
		MutableFSeq f1 = create(4, 4);
		f1.append("a\\5abcde");
		MutableFSeq f2 = create(f1);
		f2.append("\\2XY");
		Assert.assertEquals("a\\5abcdeX", f2.asString());
	}

	@Test
	public void testConst12() {
		MutableFSeq f1 = create(4, 4);
		f1.append("\\5abcde123");
		MutableFSeq f2 = create(f1);
		f2.append("\\2XY");
		Assert.assertEquals("\\5abcde123\\2XY", f2.asString());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testConst2() {
		create(0, 0);
	}

	@Test
	public void testConst3() {
		create(9999, 0);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testConst4() {
		create(-1, 0);
	}

	@Test
	public void testConst5() {
		create(10, 1);
	}

	@Test
	public void testConst6() {
		create(10, 9);
	}

	@Test
	public void testConst7() {
		create(10, 10);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testConst8() {
		create(10, 11);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testConst9() {
		create(9999, 999);
	}

	@Test
	public void testFlatten1() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB\\2CD";
		fa.append(s);
		Assert.assertEquals("AC", fa.flatten().asString());
	}

	@Test
	public void testFlatten2() {
		MutableFSeq fa = create(4, 4);
		String s = "X\\2ABX\\2CDX";
		fa.append(s);
		Assert.assertEquals("XAXCX", fa.flatten().asString());
	}

	@Test
	public void testFlatten3() {
		MutableFSeq fa = create(4, 4);
		String s = "XX";
		fa.append(s);
		Assert.assertEquals("XX", fa.flatten().asString());
	}

	@Test
	public void testFlatten4() {
		MutableFSeq fa = create(4, 4);
		String s = "";
		fa.append(s);
		Assert.assertEquals("", fa.flatten().asString());
	}

	@Test
	public void testFlatten5() {
		MutableFSeq fa = create(4, 4);
		String s = "X";
		fa.append(s);
		Assert.assertEquals("X", fa.flatten().asString());
	}

	@Test
	public void testFlatten6() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2ab";
		fa.append(s);
		Assert.assertEquals("a", fa.flatten().asString());
	}

	@Test
	public void testGetSegmentsFromPositions() {
		MutableFSeq fa = create(4, 4);
		String s = "abcdefghijkl";
		fa.append(s);
		ArrayList<Integer> poslist = new ArrayList<Integer>();
		poslist.add(0);
		poslist.add(4);
		poslist.add(8);
		List<Segment> slist = fa.getSegmentsFromPositions(poslist);
		Assert.assertEquals(poslist.size(), slist.size());
		Assert.assertEquals("abcd", slist.get(0).getData());
		Assert.assertEquals("efgh", slist.get(1).getData());
		Assert.assertEquals("ijkl", slist.get(2).getData());
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar1() {
		MutableFSeq fa = create(1, 0);
		fa.append(' ');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar2() {
		MutableFSeq fa = create(1, 0);
		fa.append('\n');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar3() {
		MutableFSeq fa = create(1, 0);
		fa.append('.');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar4() {
		MutableFSeq fa = create(1, 0);
		fa.append(':');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar5() {
		MutableFSeq fa = create(1, 0);
		fa.append('@');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar6() {
		MutableFSeq fa = create(1, 0);
		fa.append('[');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar7() {
		MutableFSeq fa = create(1, 0);
		fa.append('`');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar8() {
		MutableFSeq fa = create(1, 0);
		fa.append('{');
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testIllegalChar9() {
		MutableFSeq fa = create(1, 0);
		fa.append('\\');
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidComplexity1() {
		MutableFSeq fa = create(4, 4);
		String s = "abab";
		fa.append(s);
		Assert.assertEquals("0000", fa.replaceLowComplexity(4, 4, 0, '0')
				.asString());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidComplexity2() {
		MutableFSeq fa = create(4, 4);
		String s = "abab";
		fa.append(s);
		Assert.assertEquals("0000", fa.replaceLowComplexity(4, -1, 0, '0')
				.asString());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidComplexity3() {
		MutableFSeq fa = create(4, 4);
		String s = "abab";
		fa.append(s);
		Assert.assertEquals("0000", fa.replaceLowComplexity(-1, 2, 0, '0')
				.asString());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidComplexity4() {
		MutableFSeq fa = create(4, 4);
		String s = "abab";
		fa.append(s);
		Assert.assertEquals("0000", fa.replaceLowComplexity(4, 2, -1, '0')
				.asString());
	}

	@Test(expected = ArrayIndexOutOfBoundsException.class)
	public void testInvalidComplexity5() {
		MutableFSeq fa = create(4, 4);
		String s = "abab";
		fa.append(s);
		Assert.assertEquals("0000", fa.replaceLowComplexity(4, 2, 2, '.')
				.asString());
	}

	@Test
	public void testInvalidIntegers() {
		MutableFSeqImpl fa = (MutableFSeqImpl) create(4, 4);
		for (int i = 0; i < 256; i++) {
			try {
				fa.append(i);
			} catch (ArrayIndexOutOfBoundsException e) {
				// System.out.println("Ignored char '" + (i) + "'. Reason: "
				// + e.toString());
			}
		}
		Assert.assertEquals(b64str.length(), fa.getLength());
		Assert
				.assertEquals(
						"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
						fa.asString());
	}

	@Test
	public void testInvalidStrAppend() {
		MutableFSeq fa = create(4, 4);
		ArrayIndexOutOfBoundsException ex = null;
		try {
			fa.append("!");
		} catch (ArrayIndexOutOfBoundsException e) {
			ex = e;
		}
		Assert.assertNotNull(ex);

		ex = null;
		try {
			fa.append(" ");
		} catch (ArrayIndexOutOfBoundsException e) {
			ex = e;
		}
		Assert.assertNotNull(ex);

		ex = null;
		try {
			fa.append("a.a");
		} catch (ArrayIndexOutOfBoundsException e) {
			ex = e;
		}
		Assert.assertNotNull(ex);

		ex = null;
		try {
			fa.append("a a");
		} catch (ArrayIndexOutOfBoundsException e) {
			ex = e;
		}
		Assert.assertNotNull(ex);

		ex = null;
		try {
			fa.append(" aa");
		} catch (ArrayIndexOutOfBoundsException e) {
			ex = e;
		}
		Assert.assertNotNull(ex);

		ex = null;
		try {
			fa.append("aa ");
		} catch (ArrayIndexOutOfBoundsException e) {
			ex = e;
		}
		Assert.assertNotNull(ex);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidString1() {
		MutableFSeq fa = create(4, 4);
		fa.append("\\01bcdefgha");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidString2() {
		MutableFSeq fa = create(4, 4);
		fa.append("\\11bcdefgha");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidString3() {
		MutableFSeq fa = create(4, 4);
		fa.append("\\\\1bcdefgha");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidString4() {
		MutableFSeq fa = create(4, 4);
		fa.append("abcdefgha\\");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidString5() {
		MutableFSeq fa = create(4, 4);
		fa.append("abcdefgha\\2");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidString6() {
		MutableFSeq fa = create(4, 4);
		fa.append("abcdefgha\\3AB");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidString7() {
		MutableFSeq fa = create(4, 4);
		fa.append("abcdefgha\\-ABC");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidSub1() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB\\2CD";
		fa.append(s);
		fa.subseq(-1, fa.getLength());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidSub2() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB\\2CD";
		fa.append(s);
		fa.subseq(1, 0);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testInvalidSub3() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB\\2CD";
		fa.append(s);
		fa.subseq(1, 3);
	}

	@Test
	public void testInvalidSymbols() {
		MutableFSeq fa = create(4, 4);
		for (int i = 0; i < 256; i++) {
			try {
				fa.append((char) i);
			} catch (ArrayIndexOutOfBoundsException e) {
				// System.out.println("Ignored char '" + ((char) i)
				// + "'. Reason: " + e.toString());
			}
		}
		Assert.assertEquals(b64str.length(), fa.getLength());
		Assert
				.assertEquals(
						"+/0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",
						fa.asString());
	}

	@Test
	public void testLowComplexity1() {
		int[] expected;
		int[] actual;
		expected = new int[] { 0, 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("abcabc".toCharArray(),
				3, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 2, 2, 2, 2 };
		actual = MutableFSeqImpl.duplicatesPerSegment("aaaaaa".toCharArray(),
				3, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 1, 2, 2 };
		actual = MutableFSeqImpl.duplicatesPerSegment("abcccc".toCharArray(),
				3, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 2, 1, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("cccabc".toCharArray(),
				3, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 1, 1, 1, 1 };
		actual = MutableFSeqImpl.duplicatesPerSegment("ababab".toCharArray(),
				3, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 2, 2, 2, 2, 2 };
		actual = MutableFSeqImpl.duplicatesPerSegment("abababab".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void testLowComplexity2() {
		int[] expected;
		int[] actual;
		expected = new int[] { 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("abcdab".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 3, 3, 3 };
		actual = MutableFSeqImpl.duplicatesPerSegment("aaaaaa".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 1, 2 };
		actual = MutableFSeqImpl.duplicatesPerSegment("abcddd".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 1, 1, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("dcbabcd".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 0, 1, 1, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment(
				"edcbabcde".toCharArray(), 4, '0');
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void testLowComplexity3() {
		int[] expected;
		int[] actual;
		expected = new int[] { 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("000000".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("0bcdab".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("abcda0".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("a0cd0b".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("ab00ab".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 3, 2, 1 };
		actual = MutableFSeqImpl.duplicatesPerSegment("aaaa00".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 1, 2, 3 };
		actual = MutableFSeqImpl.duplicatesPerSegment("00aaaa".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 1, 1, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment("dcb0bcd".toCharArray(),
				4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 0, 0, 0, 0, 0, 0 };
		actual = MutableFSeqImpl.duplicatesPerSegment(
				"edc000cde".toCharArray(), 4, '0');
		Assert.assertArrayEquals(expected, actual);
		expected = new int[] { 1, 1, 1, 0, 0, 1, 1, 1 };
		actual = MutableFSeqImpl.duplicatesPerSegment("edcc000ccde"
				.toCharArray(), 4, '0');
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void testMutatedChar1() {
		MutableFSeq fa = create(1, 1);
		for (char c : b64str.toCharArray()) {
			fa.append(c);
		}
		Assert.assertEquals(64, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(b64str, fa.asString());
		for (char c : b64str.toCharArray()) {
			fa.append(new char[] { c, c });
		}
		Assert.assertEquals(64 + 64, fa.getLength());
		Assert.assertEquals(64, fa.getMutations());
	}

	@Test
	public void testMutatedChar2() {
		MutableFSeq fa = create(4, 2);
		fa.append('a');
		Assert.assertEquals(1, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(0, fa.getIgnoredMutations());
		Assert.assertEquals("a", fa.asString());
		fa.append(new char[] { 'A', 'B', 'C', 'D' });
		Assert.assertEquals(2, fa.getLength());
		Assert.assertEquals(2, fa.getMutations());
		Assert.assertEquals(1, fa.getIgnoredMutations());
		Assert.assertEquals("a\\3ABC", fa.asString());
		fa.append(new char[] { 'D', 'E', 'F' });
		Assert.assertEquals(3, fa.getLength());
		Assert.assertEquals(2, fa.getMutations());
		Assert.assertEquals(3, fa.getIgnoredMutations());
		Assert.assertEquals("a\\3ABCD", fa.asString());
		fa.append(new char[] { 'E' });
		Assert.assertEquals(4, fa.getLength());
		Assert.assertEquals(2, fa.getMutations());
		Assert.assertEquals(3, fa.getIgnoredMutations());
		Assert.assertEquals("a\\3ABCDE", fa.asString());
		fa.append(new char[] { 'F', 'G' });
		Assert.assertEquals(5, fa.getLength());
		Assert.assertEquals(2, fa.getMutations());
		Assert.assertEquals(4, fa.getIgnoredMutations());
		Assert.assertEquals("a\\3ABCDEF", fa.asString());
		fa.append(new char[] { 'G', 'H' });
		Assert.assertEquals(6, fa.getLength());
		Assert.assertEquals(3, fa.getMutations());
		Assert.assertEquals(4, fa.getIgnoredMutations());
		Assert.assertEquals("a\\3ABCDEF\\2GH", fa.asString());
		fa.append('i');
		Assert.assertEquals(7, fa.getLength());
		Assert.assertEquals(3, fa.getMutations());
		Assert.assertEquals(4, fa.getIgnoredMutations());
		Assert.assertEquals("a\\3ABCDEF\\2GHi", fa.asString());
	}

	@Test
	public void testMutatedChar3() {
		MutableFSeq fa = create(99, 14);
		fa.append(new char[] { '1', '2', '3' });
		Assert.assertEquals("\\3123", fa.asString());
		fa = create(99, 14);
		fa
				.append(new char[] { '1', '2', '3', '4', '5', '6', '7', '8',
						'9', '0' });
		Assert.assertEquals("\\a1234567890", fa.asString());
		fa = create(99, 14);
		fa.append(new char[] { '1', '2', '3', '4', '5', '6', '7', '8', '9',
				'a', 'b', 'c', 'd', 'e', 'f' });
		Assert.assertEquals("\\f123456789abcdef", fa.asString());
		fa = create(99, 14);
		fa.append(new char[] { '1', '2', '3', '4', '5', '6', '7', '8', '9',
				'a', 'b', 'c', 'd', 'e', 'f', '0' });
		Assert.assertEquals("\\f123456789abcdef", fa.asString());
	}

	@Test
	public void testMutatedInt1() {
		MutableFSeqImpl fa = (MutableFSeqImpl) create(1, 1);
		for (int i = 0; i < b64str.length(); i++) {
			fa.append(i);
		}
		Assert.assertEquals(64, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(b64str, fa.asString());
		for (int i = 0; i < b64str.length(); i++) {
			fa.append(new int[] { i, i });
		}
		Assert.assertEquals(64 + 64, fa.getLength());
		Assert.assertEquals(64, fa.getMutations());
	}

	@Test
	public void testMutatedInt2() {
		MutableFSeqImpl fa = (MutableFSeqImpl) create(2, 1);
		for (int i = 0; i < b64str.length(); i++) {
			fa.append(new int[] { i, i });
		}
		Assert.assertEquals(64, fa.getLength());
		Assert.assertEquals(32, fa.getMutations());
	}

	@Test
	public void testMutatedInt3() {
		MutableFSeqImpl fa = (MutableFSeqImpl) create(2, 1);
		for (int i = 0; i < b64str.length(); i++) {
			fa.append(new int[] { i, i, i });
		}
		Assert.assertEquals(64, fa.getLength());
		Assert.assertEquals(32, fa.getMutations());
	}

	@Test
	public void testNonMutatedChar() {
		MutableFSeq fa = create(1, 0);
		for (char c : b64str.toCharArray()) {
			fa.append(c);
		}
		Assert.assertEquals(64, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(b64str, fa.asString());
		for (char c : b64str.toCharArray()) {
			for (char d : b64str.toCharArray()) {
				fa.append(new char[] { c, d });
			}
		}
		Assert.assertEquals(64 + 64 * 64, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
	}

	@Test
	public void testNonMutatedInt() {
		MutableFSeqImpl fa = (MutableFSeqImpl) create(1, 0);
		for (int i = 0; i < b64str.length(); i++) {
			fa.append(i);
		}
		Assert.assertEquals(64, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(b64str, fa.asString());
		for (int i = 0; i < b64str.length(); i++) {
			for (int j = 0; j < b64str.length(); j++) {
				fa.append(new int[] { i, j });
			}
		}
		Assert.assertEquals(64 + 64 * 64, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
	}

	@Test
	public void testString1() {
		MutableFSeq fa = create(4, 4);
		fa.append("a");
		Assert.assertEquals(1, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(0, fa.getIgnoredMutations());
		Assert.assertEquals("a", fa.asString());
		fa.append("bb");
		Assert.assertEquals(3, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(0, fa.getIgnoredMutations());
		Assert.assertEquals("abb", fa.asString());
		fa.append("");
		Assert.assertEquals(3, fa.getLength());
		Assert.assertEquals(0, fa.getMutations());
		Assert.assertEquals(0, fa.getIgnoredMutations());
		Assert.assertEquals("abb", fa.asString());
		fa.append("\\3123");
		Assert.assertEquals(4, fa.getLength());
		Assert.assertEquals(2, fa.getMutations());
		Assert.assertEquals(0, fa.getIgnoredMutations());
		Assert.assertEquals("abb\\3123", fa.asString());
		fa.append("\\3ABC");
		Assert.assertEquals(5, fa.getLength());
		Assert.assertEquals(4, fa.getMutations());
		Assert.assertEquals(0, fa.getIgnoredMutations());
		Assert.assertEquals("abb\\3123\\3ABC", fa.asString());
		fa.append("\\3XYZ");
		Assert.assertEquals(6, fa.getLength());
		Assert.assertEquals(4, fa.getMutations());
		Assert.assertEquals(2, fa.getIgnoredMutations());
		Assert.assertEquals("abb\\3123\\3ABCX", fa.asString());
		fa.append("\\3XYZ\\3abc");
		Assert.assertEquals(8, fa.getLength());
		Assert.assertEquals(6, fa.getMutations());
		Assert.assertEquals(4, fa.getIgnoredMutations());
		Assert.assertEquals("abb\\3123\\3ABCXX\\3abc", fa.asString());
	}

	@Test
	public void testString2() {
		MutableFSeq fa = create(4, 4);
		fa.append("a\\2XY\\2ab1234\\8abcdefghOP\\3LKM");
		Assert.assertEquals(11, fa.getLength());
		Assert.assertEquals(6, fa.getMutations());
		Assert.assertEquals(5, fa.getIgnoredMutations());
		Assert.assertEquals("a\\2XY\\2ab1234\\5abcdeOPL", fa.asString());
	}

	@Test
	public void testString3() {
		MutableFSeq fa = create(4, 4);
		fa.append("a\\8abcdefgh");
		Assert.assertEquals(2, fa.getLength());
		Assert.assertEquals(4, fa.getMutations());
		Assert.assertEquals(3, fa.getIgnoredMutations());
		Assert.assertEquals("a\\5abcde", fa.asString());
	}

	@Test
	public void testString4() {
		MutableFSeq fa = create(4, 4);
		fa.append("\\8abcdefgh");
		Assert.assertEquals(1, fa.getLength());
		Assert.assertEquals(4, fa.getMutations());
		Assert.assertEquals(3, fa.getIgnoredMutations());
		Assert.assertEquals("\\5abcde", fa.asString());
	}

	@Test
	public void testString5() {
		MutableFSeq fa = create(4, 4);
		fa.append("\\8abcdefgha");
		Assert.assertEquals(2, fa.getLength());
		Assert.assertEquals(4, fa.getMutations());
		Assert.assertEquals(3, fa.getIgnoredMutations());
		Assert.assertEquals("\\5abcdea", fa.asString());
	}

	@Test
	public void testSub1() {
		MutableFSeq fa = create(4, 4);
		String s = "abcdefg";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub10() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2ABX\\2CD";
		fa.append(s);
		MutableFSeq sub = fa.subseq(1, fa.getLength() - 1);
		Assert.assertEquals("X", sub.asString());
		Assert.assertEquals(fa.getLength() - 2, sub.getLength());
		Assert.assertEquals(fa.getMutations() - 2, sub.getMutations());
	}

	@Test
	public void testSub11() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB\\2CD";
		fa.append(s);
		MutableFSeq sub = fa.subseq(1, fa.getLength() - 1);
		Assert.assertEquals("", sub.asString());
		Assert.assertEquals(0, sub.getLength());
		Assert.assertEquals(0, sub.getMutations());
	}

	@Test
	public void testSub12() {
		MutableFSeq fa = create(4, 4);
		String s = "XX";
		fa.append(s);
		MutableFSeq sub = fa.subseq(1, fa.getLength() - 1);
		Assert.assertEquals("", sub.asString());
		Assert.assertEquals(0, sub.getLength());
		Assert.assertEquals(0, sub.getMutations());
	}

	@Test
	public void testSub13() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB\\2CD";
		fa.append(s);
		MutableFSeq sub = fa.subseq(1, fa.getLength());
		Assert.assertEquals("\\2CD", sub.asString());
		Assert.assertEquals(1, sub.getLength());
		Assert.assertEquals(1, sub.getMutations());
	}

	@Test
	public void testSub14() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB\\2CD";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength() - 1);
		Assert.assertEquals("\\2AB", sub.asString());
		Assert.assertEquals(1, sub.getLength());
		Assert.assertEquals(1, sub.getMutations());
	}

	@Test
	public void testSub2() {
		MutableFSeq fa = create(4, 4);
		String s = "";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub3() {
		MutableFSeq fa = create(4, 4);
		String s = "aa\\2ABbb";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub4() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2ABbb";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub5() {
		MutableFSeq fa = create(4, 4);
		String s = "aa\\2AB";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub6() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2AB";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub7() {
		MutableFSeq fa = create(4, 4);
		String s = "\\2ABX\\2CD";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub8() {
		MutableFSeq fa = create(4, 4);
		String s = "X\\2ABX\\2CDX";
		fa.append(s);
		MutableFSeq sub = fa.subseq(0, fa.getLength());
		Assert.assertEquals(fa.asString(), sub.asString());
		Assert.assertEquals(fa.getLength(), sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testSub9() {
		MutableFSeq fa = create(4, 4);
		String s = "X\\2ABX\\2CDX";
		fa.append(s);
		MutableFSeq sub = fa.subseq(1, fa.getLength() - 1);
		Assert.assertEquals("\\2ABX\\2CD", sub.asString());
		Assert.assertEquals(fa.getLength() - 2, sub.getLength());
		Assert.assertEquals(fa.getMutations(), sub.getMutations());
	}

	@Test
	public void testValidChars() {
		MutableFSeq fa = create(1, 0);
		for (char c : b64str.toCharArray()) {
			fa.append(c);
		}
	}

	@Test
	public void testValidStrAppend() {
		MutableFSeq fa = create(4, 4);
		fa.append(new String(b64str));
		fa.append("");
		Assert.assertEquals(new String(b64str), fa.asString());
	}
}
