/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.List;

import org.jhears.web.rs.Repeat;
import org.junit.Assert;
import org.junit.Test;

public class RLEncTest {

	@Test
	public void test1() {
		char[] ca = "".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertTrue(list.isEmpty());
	}

	@Test
	public void test2() {
		char[] ca = "aaa".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertTrue(list.isEmpty());
	}

	@Test
	public void test3() {
		char[] ca = "aaaa".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertEquals(1, list.size());
		Assert.assertEquals(new Repeat('a', 0, 4), list.get(0));
	}

	@Test
	public void test4() {
		char[] ca = "aaaaa".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertEquals(1, list.size());
		Assert.assertEquals(new Repeat('a', 0, 5), list.get(0));
	}

	@Test
	public void test5() {
		char[] ca = "Xaaaa".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertEquals(1, list.size());
		Assert.assertEquals(new Repeat('a', 1, 4), list.get(0));
	}

	@Test
	public void test6() {
		char[] ca = "aaaaX".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertEquals(1, list.size());
		Assert.assertEquals(new Repeat('a', 0, 4), list.get(0));
	}

	@Test
	public void test7() {
		char[] ca = "XaaaaaX".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertEquals(1, list.size());
		Assert.assertEquals(new Repeat('a', 1, 5), list.get(0));
	}

	@Test
	public void test8() {
		char[] ca = "aaaaXXXX".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertEquals(2, list.size());
		Assert.assertEquals(new Repeat('a', 0, 4), list.get(0));
		Assert.assertEquals(new Repeat('X', 4, 4), list.get(1));
	}

	@Test
	public void test9() {
		char[] ca = "aaaa0000XXXX".toCharArray();
		RLEnc enc = new RLEnc(4, '0');
		List<Repeat> list = enc.encode(ca);
		Assert.assertEquals(2, list.size());
		Assert.assertEquals(new Repeat('a', 0, 4), list.get(0));
		Assert.assertEquals(new Repeat('X', 8, 4), list.get(1));
	}

}
