/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.configuration.PropertiesConfiguration;
import org.jhears.cnf.Cnf.FingerprintKey;
import org.jhears.web.rs.Segment;
import org.junit.Assert;
import org.junit.Test;

public class SeqMatchTest {

	private static final String b64str = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

	private MutableFSeq create(int segmentLength, int maxMutation) {
		PropertiesConfiguration c = new PropertiesConfiguration();
		c.setProperty(FingerprintKey.SEGMENT_LENGTH, segmentLength);
		c.setProperty(FingerprintKey.SEGMENT_MAX_MUTATIONS, maxMutation);
		c.setProperty(FingerprintKey.SYMBOLS, b64str);
		FSeqFactory factory = new FSeqFactory(c);
		return factory.getNewInstance();
	}

	@Test
	public void test() {
		int position = 960;
		MutableFSeq seq = create(8, 6);
		String fseqStr = "SGG\\2m5OYYBGGBm5YYBG5\\254mmGGG5O5YBm\\24545F5YBBYY5FGGGqq5YGqGGGy5YqBG55qmGqQTTTTiTTTTiQTDTTyyy\\25tt\\263655yl5GJjqqqmm2qYmOtfff2222VljIIffmffffrffff\\25tVXX55oggX65Jo55ottmmmFj\\2mq5mfqmm3m\\25muqFFtt355SFV556ll\\2mqmS\\2SmmOS5Otttm\\25mmWmmfJhIIhzff3m\\26f6ff3m3WVS5g6g\\265\\2F556Wt55ttVJ5\\2tStr55oo\\25F5\\3mq3m\\2m55mmmm5m\\2m3mmzIhqffffxn6PVG\\25m2f2JQQQwwzzQwwQV\\2mOzhm5m5\\3PmOhzzGOWS\\2YGOOWmm5\\25tOl5qfff3m55W556FFFm\\25mS\\2tOtSSmmmf3\\2JqqWq5mffJJJJFttStSYVf\\2fnnJx356m\\2m6\\236ofJJ\\2zqqq\\2mO\\25GqQsjJJ3V\\2q3qq36\\2oV\\2J3Jn3FJmVf\\3JxfQJqqmGq\\2qmqGVJJQQqV\\26V\\2Pmmm\\2qmmJJmmmmBq\\2qG\\2qVmmm5m55mffnJKKm\\2zmmt\\2J66V\\2qJJqqmm5GQjjJJ3VVVq\\2o36V\\2VnJJ\\2jJFGmmVx\\2xQQ\\2qJqqGqm\\2oVJV\\26VJQQQQ\\2VfmmmmgJJXmgggqqBqm\\2qBBq755Bqqmmqqm3\\26JJJjJ\\2qJj\\2qzqq\\2S4SGqnG\\2G5mOmqq6Fyyy\\2jyJqq4SSmqGmmmmVzzfnJJjfOOG45GfqmmmGtfm56ffyDQTJG53555GJ65555P\\35Pt5G6nn\\2nJG556655\\26555t555t5G56\\25FFgS5GG\\25mmq\\33q65m5S4St\\245StFFq5433n3mGff\\23JnyGGVttGYbV5Yqq\\25GqqGGqq5qqFS\\2mqxJTjS4qJJJJm\\25m\\2t4mSm3\\2nxTTnGYPtSm3\\2nfJn\\2Tn\\23GG6tm\\2J6JJ5J6SSmSmm6t\\25tjF5\\2mr\\3tmP\\2mPmmJJJJ\\25Gmfmmffm\\2fm\\2jffmmmSm\\25F5mmJzG35mmmmmmrmmVtummV5F65mmS522f\\2f7TqG\\2WbTDDDDDJq\\2qGGUGVm\\25665\\2f664mmmmmf6\\25OO\\2mO53m4of6Ffnm4SSm66JFy\\2JyFJ\\2Mrt\\2Ot3n\\23f6ffmmfmffnqF35\\2t5Mt5fffqF\\2ny55mmmm\\2f66F6m5OtS\\23m\\2qmmmm6mmf6m\\23mffmmfF\\2JFV\\2tSm5Smt6SSOSSmmmmmq6frrWOOjjF\\2qFFjffW6\\26m66\\2f6f665\\2G5BffnJq\\2mq5\\26Gm66BfJJ\\2zq\\2mqmmGzqjjJJ3o33336\\2yVJJJJFJmVfJxJqqmG\\2q3m6GjJJQQJVmmmmmmJqmmmm\\2qm\\2mqmVmF\\2qmm6\\265SmffnJ\\2qJq\\2mqmmm6mV3Jqqmm5GQjjJJ3qVqq\\2Vo6\\2f6nJJ\\2JjFGmmVJ\\2QxjqqmG3mmGmVJ\\2QJQqq\\36fVtmmmqJJf6\\3gjXgXXqqqJqqq7qqBBB\\2m5Ym\\24SSrtSSmSt\\2mVq\\4Fqm5qmuuSSSttSttStSSS\\2SttF\\2SFttttSt5tFttttttS\\2SGtttFF\\2tOtttSSttttttttSSSttStt\\2SttSSSttttttttSSSttttttutSSSttFFFtutSStFFFtttmVVVVV\\2yj\\2FJqjmVVV6tttttttt\\2StSttttttttt\\2GmmttSttttt\\2OtmmrWOOjOommO\\2OrrtF\\3tSFttmOoV\\2VggOOOOO\\2oOmm\\2OmOO\\3FOtFtStSSttttFOOtuSStttFFFt\\2utrVVVV\\2yOF\\2tm\\2tOt\\2tummVVVVOFOmrVVVVWOOmmmOttttFFFFttttStutttOSt\\2tSt\\26FFtFFFOttGV6F\\2yFFOOttmGqq6FFFmtt\\2tF\\2tSFFttttt\\2tStSStttFFFtttSGFOFFOOOmJJJ6FFOFFmmmVJ6\\2jf\\2qmFOOm\\2Gmm\\2m6m6OO\\2GFWGGmmVmOOF\\2yWWmmmmOOOOFOOGmmmVVOOF\\2WOG\\2GmmmVOOOFOWG\\2mGmmmFOOFW\\2tOmmmV\\3VOmOOOFOmo\\2jOOotttttttOOOoF6OFO\\2tOtF\\2OFoOFttttttttttFtttttttOommOSOttOOoOtFttWtmOOoOFOFFtttttttttFOtSlt\\2SttttttttttmF\\2GFGttuP\\2YSGm6JJJ3qGmmq6VJJJqqqqmmm\\2mq553mmmm\\2Vmmmq\\2qJJ\\2qm5m\\25SSP5JJJ\\2mqmm\\23mmm\\26f5JJJqqqqmm\\25mmm5\\2mqmmmmmm5\\2qm\\2mqGq\\2Gm5555\\43m6f6fJJxz\\3zQqQqo\\2qJ\\2oVVJJ3qVmmmmm6Jmmmmmmmmm5Fq55\\2P54m5FJJJfm\\2qmmm\\2q3qJJJqVq3qm5qq\\3JqFJqmmmmm5qqmmm\\2m4m\\2m55V3m4mW5Srrrmmmmmm\\24SSOttmm4tWmSmmuummSttSStuu\\2mqmm4rrSSrrrmmmmRm\\2SYPtStr4SOPmmSWO\\2tSm4tttS6uSmmmmm\\2f2qq22222mmmOS\\2S4frmmzmmRhmVjX\\2wQQQQz\\2Q0VV0wwwzjmhIzzzUn\\23qxzzxxGqfmmmmmmr\\2m4SmmmmmmmmV5SrmmzzqGffm32222qmmrmS\\2uPmSmm55qmSmmmSmmmmmRR/////////////////////////////////////";
		seq.append(fseqStr);
		List<Segment> fromDatabaseSegments = new ArrayList<Segment>();
		fromDatabaseSegments.add(new Segment("Sm5Smt6S", 0));
		fromDatabaseSegments.add(new Segment("SOSSmmmm", 8));
		fromDatabaseSegments.add(new Segment("mq6frrWO", 16));
		fromDatabaseSegments.add(new Segment("OjjFFFjf", 24));
		fromDatabaseSegments.add(new Segment("6655Bffn", 40));
		fromDatabaseSegments.add(new Segment("Jqq5Gm66", 48));
		fromDatabaseSegments.add(new Segment("BfJJzmmm", 56));
		fromDatabaseSegments.add(new Segment("GzqjjJJ3", 64));
		fromDatabaseSegments.add(new Segment("o33336yJ", 72));
		seq.flatten();
		SeqMatch seqMatch = new SeqMatch(0.8f);
		float confidence = seqMatch.segmentsInSeqConfidence(
				fromDatabaseSegments, position, seq);
		Assert.assertTrue(confidence > 0.9f);
	}
}
