/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import org.junit.Assert;
import org.junit.Test;

public class SequenceSmootherTest {

	@Test
	public void test01() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "abcd".toCharArray();
		char[] expected = "abcd".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test02() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "aaaaXaaaaabcd".toCharArray();
		char[] expected = "aaaaaaaaaabcd".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test03() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "aaaXaaaaaabcd".toCharArray();
		char[] expected = "aaaXaaaaaabcd".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test04() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "aaaaXaaaaaaabcd".toCharArray();
		char[] expected = "aaaaaaaaaaaabcd".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test05() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "aaaaaXaaaaaaabcd".toCharArray();
		char[] expected = "aaaaaaaaaaaaabcd".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test06() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "aaaaaXXaaaaaaabcd".toCharArray();
		char[] expected = "aaaaaaaaaaaaaabcd".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test07() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "aaaaaXXXaaaaaaabcd".toCharArray();
		char[] expected = "aaaaaXXXaaaaaaabcd".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test08() {
		SequenceSmoother o = new SequenceSmoother(4, 2);
		char[] actual = "xzyaaaaXaaaa".toCharArray();
		char[] expected = "xzyaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test09() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test10() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxxaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test11() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxaxaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test12() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxaaxaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test13() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxaaaxaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test14() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxazaxaaaaaa".toCharArray();
		char[] expected = "aaaaaaxazaxaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test15() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxzaxaaaaaa".toCharArray();
		char[] expected = "aaaaaaxzaxaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test16() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxzxaaaaaa".toCharArray();
		char[] expected = "aaaaaaxzxaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test17() {
		SequenceSmoother o = new SequenceSmoother(6, 2);
		char[] actual = "aaaaaaxzaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test18() {
		SequenceSmoother o = new SequenceSmoother(6, 3);
		char[] actual = "aaaaaaxyzaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test19() {
		SequenceSmoother o = new SequenceSmoother(6, 3);
		char[] actual = "aaaaaaxayzaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test20() {
		SequenceSmoother o = new SequenceSmoother(6, 3);
		char[] actual = "aaaaaaxayazaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test21() {
		SequenceSmoother o = new SequenceSmoother(6, 3);
		char[] actual = "aaaaaaxyaazaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test22() {
		SequenceSmoother o = new SequenceSmoother(6, 3);
		char[] actual = "aaaaaaxaayzaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test23() {
		SequenceSmoother o = new SequenceSmoother(6, 3);
		char[] actual = "aaaaaaxaaayzaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void test24() {
		SequenceSmoother o = new SequenceSmoother(6, 3);
		char[] actual = "aaaaaaxaaaayzaaaaaa".toCharArray();
		char[] expected = "aaaaaaaaaaaaaaaaaaa".toCharArray();
		o.smooth(actual);
		Assert.assertArrayEquals(expected, actual);
	}

	@Test
	public void testCountChars1() {
		char[] ca = "X".toCharArray();
		int expected = 1;
		int actual = SequenceSmoother.countChars(ca, 'X', 0, 1);
		Assert.assertEquals(expected, actual);
	}

	@Test
	public void testCountChars2() {
		char[] ca = "".toCharArray();
		int expected = 0;
		int actual = SequenceSmoother.countChars(ca, 'X', 0, 0);
		Assert.assertEquals(expected, actual);
	}

	@Test
	public void testCountChars3() {
		char[] ca = "aXa".toCharArray();
		int expected = 1;
		int actual = SequenceSmoother.countChars(ca, 'X', 0, 3);
		Assert.assertEquals(expected, actual);
	}

	@Test
	public void testCountChars4() {
		char[] ca = "aXa".toCharArray();
		int expected = 1;
		int actual = SequenceSmoother.countChars(ca, 'X', 1, 3);
		Assert.assertEquals(expected, actual);
	}

	@Test
	public void testCountChars5() {
		char[] ca = "aXa".toCharArray();
		int expected = 1;
		int actual = SequenceSmoother.countChars(ca, 'X', 1, 2);
		Assert.assertEquals(expected, actual);
	}

	@Test
	public void testCountChars6() {
		char[] ca = "aXa".toCharArray();
		int expected = 0;
		int actual = SequenceSmoother.countChars(ca, 'X', 2, 2);
		Assert.assertEquals(expected, actual);
	}

	@Test
	public void testCountChars7() {
		char[] ca = "aXa".toCharArray();
		int expected = 0;
		int actual = SequenceSmoother.countChars(ca, 'X', 0, 1);
		Assert.assertEquals(expected, actual);
	}

	@Test
	public void testCountChars8() {
		char[] ca = "XaX".toCharArray();
		int expected = 2;
		int actual = SequenceSmoother.countChars(ca, 'X', 0, 3);
		Assert.assertEquals(expected, actual);
	}

}
