/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.miniserver;

import java.io.File;
import java.io.FilenameFilter;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.webapp.WebAppContext;
import org.eclipse.jetty.xml.XmlConfiguration;
import org.jhears.cnf.Cnf;
import org.jhears.cnf.CnfManager;
import org.jhears.util.SingletonMutablePicoContainer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Container {

	private final Logger logger;

	private MiniServer server;

	private final CnfManager cnfManager;

	public Container(CnfManager cnfManager) {
		this.cnfManager = cnfManager;
		logger = LoggerFactory.getLogger(Container.class);
		SingletonMutablePicoContainer.getInstance().addComponent(cnfManager);
	}

	public void start() throws Exception {
		// pico.start();
		server = new MiniServer(makeContext(), makeServer());
		server.start(); // will block
	}

	public void stop() {
		if (server != null) {
			try {
				server.stop();
				server = null;
			} catch (Exception e) {
				logger.error("Problem while stopping the server", e);
			}
		}
		// pico.stop();
		// pico.dispose();
	}

	public void assemble(String[] args) throws Exception {
		// pico.addComponent(makeServer());
		// pico.addComponent(makeContext());
		// pico.addComponent(MiniServer.class);
	}

	private Server makeServer() throws Exception {
		File jettyConf;
		String jetty = "jetty";
		String jettyProperty = Cnf.PREFIX + Cnf.SEPARATOR + jetty;
		if (System.getProperty(jettyProperty) != null) {
			jettyConf = new File(System.getProperty(jettyProperty));
		} else {
			jettyConf = new File(cnfManager.getConfigDir(), "jetty.xml");
		}
		XmlConfiguration configuration = new XmlConfiguration(jettyConf.toURI()
				.toURL());
		Server server = (Server) configuration.configure();
		return server;
	}

	private WebAppContext makeContext() {
		WebAppContext context = new WebAppContext();
		context.setContextPath("/");

		String devBase = "../jhears-servlet/src/main/webapp";

		if (new File(devBase).exists()) {
			logger.info("Development envirnonment detected: " + devBase);
			context.setDescriptor("/WEB-INF/web.xml");
			context.setResourceBase(devBase);
			context.setParentLoaderPriority(true);
			String[] __plusConfigurationClasses = new String[] {
					org.eclipse.jetty.webapp.WebInfConfiguration.class
							.getCanonicalName(),
					org.eclipse.jetty.plus.webapp.EnvConfiguration.class
							.getCanonicalName(),
					org.eclipse.jetty.plus.webapp.Configuration.class
							.getCanonicalName(),
					org.eclipse.jetty.webapp.JettyWebXmlConfiguration.class
							.getCanonicalName(),
					org.eclipse.jetty.webapp.TagLibConfiguration.class
							.getCanonicalName(),
					org.eclipse.jetty.webapp.WebXmlConfiguration.class
							.getCanonicalName() };

			context.setConfigurationClasses(__plusConfigurationClasses);

		} else {
			String webAppsDir;
			String jhearsWar = "jhears.war";
			String jettyProperty = Cnf.PREFIX + Cnf.SEPARATOR + jhearsWar;
			if (System.getProperty(jettyProperty) != null) {
				webAppsDir = System.getProperty(jettyProperty);
			} else {
				String fs = System.getProperty("file.separator");
				webAppsDir = cnfManager.getHomeDir() + fs + "var" + fs
						+ "webapps" + fs;
				File webapps = new File(webAppsDir);
				if (webapps.isDirectory()) {
					String[] wars = webapps.list(new FilenameFilter() {

						@Override
						public boolean accept(File dir, String name) {
							return name.startsWith("jhears")
									&& name.endsWith("war");
						}
					});
					if (wars.length > 0) {
						jhearsWar = wars[0];
					}
				}
			}
			context.setWar(webAppsDir + jhearsWar);
		}

		return context;
	}

}
