/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.miniserver;

import java.io.File;
import java.io.IOException;
import java.lang.Thread.UncaughtExceptionHandler;

import org.jhears.cnf.CnfManager;
import org.jhears.logging.JulSlf4jBridge;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.util.ContextInitializer;

public class Main {

	private final Logger logger;

	public static void main(String[] args) throws IOException {
		Main main = new Main();
		main.start(args);
	}

	private final Container container;

	public Main() throws IOException {
		CnfManager cnfManager = new CnfManager();
		if (System.getProperty(ContextInitializer.CONFIG_FILE_PROPERTY) == null) {
			File loggerCnf = new File(cnfManager.getConfigDir(), "logback.xml");
			System.setProperty(ContextInitializer.CONFIG_FILE_PROPERTY,
					loggerCnf.getAbsolutePath());
		}
		logger = LoggerFactory.getLogger(Main.class);
		JulSlf4jBridge.install();
		jvmSetup();
		container = new Container(cnfManager);
	}

	private void jvmSetup() {
		UncaughtExceptionHandler eh = new UncaughtExceptionHandler() {

			@Override
			public void uncaughtException(Thread t, Throwable e) {
				logger.error("Uncaught exception. thread=" + t, e);
			}
		};
		Thread.setDefaultUncaughtExceptionHandler(eh);
		Thread hook = new Thread(new Runnable() {

			@Override
			public void run() {
				logger.info("Shutdown hook executing");
				stop();
				logger.info("Shutdown complete");
			}
		});
		Runtime.getRuntime().addShutdownHook(hook);
	}

	private void start(String[] args) {
		try {
			container.assemble(args);
		} catch (Exception e) {
			logger.error("Failed to assemble server", e);
			return;
		}
		try {
			container.start();
		} catch (Exception e) {
			logger.error("Failed to start server", e);
			stop();
		}
	}

	private void stop() {
		container.stop();
	}

}
