/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Version;

import org.hibernate.annotations.Index;

@NamedQueries(value = {

		@NamedQuery(name = HBlob.HQL_CLEAR, query = "DELETE " + HBlob.NAME
				+ " where " + HBlob.pNameMapId + "=:" + HBlob.pNameMapId),

		@NamedQuery(name = HBlob.HQL_GET, query = "from " + HBlob.NAME + " as "
				+ HBlob.alias + " where " + HBlob.pNameMapId + "=:"
				+ HBlob.pNameMapId + " and " + HBlob.pNameHashCode + "=:"
				+ HBlob.pNameHashCode),

		@NamedQuery(name = HBlob.HQL_GET_KEYS, query = "from " + HBlob.NAME
				+ " where " + HBlob.pNameMapId + "=:" + HBlob.pNameMapId),

		@NamedQuery(name = HBlob.HQL_GET_SIZE, query = "SELECT count(*) from "
				+ HBlob.NAME + " where mapId=:mapId")

})
@Entity
public class HBlob implements Serializable {

	private static final long serialVersionUID = -3337602168859332197L;

	public static final String NAME = "HBlob";

	public static final String pNameKeyBytes = "keyBytes";

	public static final String pNameMapId = "mapId";

	public static final String alias = "yb";

	public static final String HQL_CLEAR = "HQL_CLEAR";

	public static final String HQL_GET = "HQL_GET";

	public static final String HQL_GET_KEYS = "HQL_GET_KEYS";

	public static final String HQL_GET_SIZE = "HQL_GET_SIZE";

	public static final String pNameHashCode = "hashCode";

	private Long hashCode;

	private Long id;

	private byte[] keyBytes;

	private Integer mapId;

	private byte[] valueBytes;

	private Integer version;

	@Index(name = "hashIndex")
	@Column(nullable = false)
	public Long getHashCode() {
		return hashCode;
	}

	@Id
	@GeneratedValue
	public Long getId() {
		return id;
	}

	@Lob
	@Column(nullable = false, length = Integer.MAX_VALUE)
	public byte[] getKeyBytes() {
		return keyBytes;
	}

	@Index(name = "mapIdIndex")
	@Column(nullable = false)
	public Integer getMapId() {
		return mapId;
	}

	@Lob
	@Column(nullable = false, length = Integer.MAX_VALUE)
	public byte[] getValueBytes() {
		return valueBytes;
	}

	@Version
	public Integer getVersion() {
		return version;
	}

	public void setHashCode(Long hash) {
		this.hashCode = hash;
	}

	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	public void setKeyBytes(byte[] keyBytes) {
		this.keyBytes = keyBytes;
	}

	public void setMapId(Integer mapId) {
		this.mapId = mapId;
	}

	public void setValueBytes(byte[] valueBytes) {
		this.valueBytes = valueBytes;
	}

	@SuppressWarnings("unused")
	private void setVersion(Integer version) {
		this.version = version;
	}

}
