/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import java.io.File;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.impl.SessionFactoryImpl;

public class HibernateSessionProvider implements SessionProvider {

	private final int batchSize;

	private final SessionFactory sessionFactory;

	public HibernateSessionProvider(File file) {
		sessionFactory = org.jhears.data.HibernateUtil
				.buildSessionFactory(file);
		if (sessionFactory instanceof SessionFactoryImpl) {
			batchSize = ((SessionFactoryImpl) sessionFactory).getSettings()
					.getJdbcBatchSize();
		} else {
			batchSize = -1;
		}
	}

	public HibernateSessionProvider() {
		sessionFactory = org.jhears.data.HibernateUtil.buildSessionFactory();
		if (sessionFactory instanceof SessionFactoryImpl) {
			batchSize = ((SessionFactoryImpl) sessionFactory).getSettings()
					.getJdbcBatchSize();
		} else {
			batchSize = -1;
		}
	}

	@Override
	public int getJdbcBatchSize() {
		return batchSize;
	}

	@Override
	public Session openSession() throws HibernateException {
		return sessionFactory.openSession();
	}

	public void close() throws HibernateException {
		sessionFactory.close();
	}

}
