/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.cfg.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Utility class to construct a Hibernate session factory.
 * 
 * @author Juha Heljoranta
 * 
 */
public final class HibernateUtil {

	private static final String cnfPropName = "hibernate.cfg";

	private HibernateUtil() {
	}

	public final static SessionFactory buildSessionFactory(Configuration c,
			File file) {
		c.configure(file);
		return c.buildSessionFactory();
	}

	public final static SessionFactory buildSessionFactory(Configuration c) {
		Logger logger = LoggerFactory.getLogger(HibernateUtil.class);
		String cnf = System.getProperty(cnfPropName);
		if (cnf != null) {
			File file = new File(cnf);
			if (file.exists()) {
				c.configure(file);
			} else {
				try {
					c.configure(new URL(cnf));
				} catch (MalformedURLException e) {
					logger.info("Failed to access " + cnfPropName
							+ ". Reason: " + e.toString()
							+ ". Loading from resources.");
					c.configure();
				}
			}
		} else {
			logger.info("System property " + cnfPropName
					+ " is not set. Loading from resources.");
			c.configure();
		}
		return c.buildSessionFactory();
	}

	public final static SessionFactory buildSessionFactory() {
		return buildSessionFactory(new AnnotationConfiguration());
	}

	public final static SessionFactory buildSessionFactory(File file) {
		return buildSessionFactory(new AnnotationConfiguration(), file);
	}

}
