/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

public final class LongUtil {

	private LongUtil() {
	}

	/**
	 * Same as {@code ByteBuffer.wrap(ba).asLongBuffer().get()} but faster.
	 * 
	 * @param ba
	 * @return
	 */
	public static final long makeLong(final byte[] ba) {
		return ((((long) ba[0] & 0xff) << 56) | (((long) ba[1] & 0xff) << 48)
				| (((long) ba[2] & 0xff) << 40) | (((long) ba[3] & 0xff) << 32)
				| (((long) ba[4] & 0xff) << 24) | (((long) ba[5] & 0xff) << 16)
				| (((long) ba[6] & 0xff) << 8) | (((long) ba[7] & 0xff) << 0));
	}

	/**
	 * same as {@code ByteBuffer.wrap(retval).asDoubleBuffer().put(d); return
	 * retval;} but faster.
	 * 
	 * @param value
	 * @return
	 */
	public static final byte[] makeBytes(final long value) {
		return new byte[] { (byte) (value >> 56), (byte) (value >> 48),
				(byte) (value >> 40), (byte) (value >> 32),
				(byte) (value >> 24), (byte) (value >> 16),
				(byte) (value >> 8), (byte) (value >> 0) };
	}
}