/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.data;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.jhears.tuple.Tuple1;
import org.jhears.tuple.Tuple2;
import org.jhears.tuple.Tuple3;
import org.jhears.tuple.Tuple4;

public class QB implements IQB {

	private final List<Tuple1<Action>> list;

	public QB() {
		this.list = new ArrayList<Tuple1<Action>>();
	}

	@SuppressWarnings("unchecked")
	@Override
	public Criteria build(Criteria c) {
		for (Tuple1<Action> t : list) {
			switch (t.get1()) {
			case LIKE: {
				Tuple4<Action, String, String, StrMatch> tt = (Tuple4<Action, String, String, StrMatch>) t;
				c.add(Restrictions.like(tt.get2(), tt.get3(), cnv(tt.get4())));
				break;
			}
			case ILIKE: {
				Tuple4<Action, String, String, StrMatch> tt = (Tuple4<Action, String, String, StrMatch>) t;
				c.add(Restrictions.ilike(tt.get2(), tt.get3(), cnv(tt.get4())));
				break;
			}
			case EQ: {
				Tuple3<Action, String, Object> tt = (Tuple3<Action, String, Object>) t;
				c.add(Restrictions.eq(tt.get2(), tt.get3()));
				break;
			}
			case ORDER: {
				Tuple3<Action, String, Boolean> tt = (Tuple3<Action, String, Boolean>) t;
				if (tt.get3().booleanValue()) {
					c.addOrder(Order.asc(tt.get2()));
				} else {
					c.addOrder(Order.desc(tt.get2()));
				}
				break;
			}
			case MAX_RESULTS: {
				Tuple2<Action, Integer> tt = (Tuple2<Action, Integer>) t;
				c.setMaxResults(tt.get2().intValue());
				break;
			}
			case FIRST_RESULT: {
				Tuple2<Action, Integer> tt = (Tuple2<Action, Integer>) t;
				c.setFirstResult(tt.get2().intValue());
				break;
			}
			default:
				throw new IllegalStateException("Unknown value: " + t.get1());
			}
		}
		list.clear();
		return c;
	}

	private MatchMode cnv(StrMatch m) {
		switch (m) {
		case ANYWHERE:
			return MatchMode.ANYWHERE;
		case END:
			return MatchMode.END;
		case EXACT:
			return MatchMode.EXACT;
		case START:
			return MatchMode.START;
		default:
			throw new IllegalStateException("Unknown mode");
		}
	}

	@Override
	public QB eq(String propertyName, Object value) {
		list.add(new Tuple3<Action, String, Object>(Action.EQ, propertyName,
				value));
		return this;
	}

	@Override
	public QB firstResult(int firstResult) {
		list.add(new Tuple2<Action, Integer>(Action.FIRST_RESULT, Integer
				.valueOf(firstResult)));
		return this;
	}

	@Override
	public QB ilike(String propertyName, String value, StrMatch mode) {
		list.add(new Tuple4<Action, String, String, StrMatch>(Action.ILIKE,
				propertyName, value, mode));
		return this;
	}

	@Override
	public QB like(String propertyName, String value, StrMatch mode) {
		list.add(new Tuple4<Action, String, String, StrMatch>(Action.LIKE,
				propertyName, value, mode));
		return this;
	}

	@Override
	public QB maxResults(int count) {
		list.add(new Tuple2<Action, Integer>(Action.MAX_RESULTS, Integer
				.valueOf(count)));
		return this;
	}

	@Override
	public QB order(String propertyName, boolean ascending) {
		list.add(new Tuple3<Action, String, Boolean>(Action.ORDER,
				propertyName, Boolean.valueOf(ascending)));
		return this;
	}

}
