/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import org.hibernate.Session;

/**
 * Operates with {@link Session} with open transaction.
 * 
 * @author Juha Heljoranta
 * 
 * @param <F>
 * @param <V>
 */
public interface Transactional<F, T> {

	/**
	 * {@link DAO} calls this method with a valid {@link Session}. Transaction
	 * is committed automatically when this method returns. If this method
	 * throws an exception then the transaction is rolled back.
	 * <p>
	 * Users of this method <b>must not</b> catch {@link RuntimeException}s
	 * unless they <b>really</b> know what they are doing. Failing to comply
	 * might prevent rollback. See Hibernate manual section:
	 * "Exception handling" for details.
	 * <p>
	 * Users of this method should not try to:
	 * <ul>
	 * <li>Open or close session/transaction.</li>
	 * <li>Perform commit or rollback.</li>
	 * </ul>
	 * 
	 * @param s
	 *            Session with open transaction.
	 * @param arg
	 *            User given value.
	 * @return
	 */
	public T with(Session s, F arg);

}
