/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.jhears.data.IQB.StrMatch;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class QBTest {

	@Test
	public void testNothing() {
		Criteria c = mock(Criteria.class);
		QB qb = new QB();
		qb.build(c);
		verifyZeroInteractions(c);
	}

	@Test
	public void testEq() {
		Criteria c = mock(Criteria.class);
		QB qb = new QB();
		qb.eq("", "");
		qb.build(c);
		verify(c).add(any(Restrictions.eq("", "").getClass()));
		verifyNoMoreInteractions(c);
	}

	@Test
	public void testFirstResult() {
		Criteria c = mock(Criteria.class);
		QB qb = new QB();
		qb.firstResult(0);
		qb.build(c);
		verify(c).setFirstResult(0);
		verifyNoMoreInteractions(c);
	}

	@Test
	public void testMaxResults() {
		Criteria c = mock(Criteria.class);
		QB qb = new QB();
		qb.maxResults(0);
		qb.build(c);
		verify(c).setMaxResults(0);
		verifyNoMoreInteractions(c);
	}

	@Test
	public void testILike() {
		Criteria c = mock(Criteria.class);
		QB qb = new QB();
		qb.ilike("", "", StrMatch.ANYWHERE);
		qb.build(c);
		verify(c).add(
				any(Restrictions.ilike("", "", MatchMode.ANYWHERE).getClass()));
		verifyNoMoreInteractions(c);
	}

	@Test
	public void testLike() {
		Criteria c = mock(Criteria.class);
		QB qb = new QB();
		qb.like("", "", StrMatch.ANYWHERE);
		qb.build(c);
		verify(c).add(
				any(Restrictions.like("", "", MatchMode.ANYWHERE).getClass()));
		verifyNoMoreInteractions(c);
	}

	@Test
	public void testOrder() {
		Criteria c = mock(Criteria.class);
		QB qb = new QB();
		qb.order("", true);
		qb.build(c);
		verify(c).addOrder(any(Order.asc("").getClass()));
		verifyNoMoreInteractions(c);
	}

}
