/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.jhears.data.DatabaseMap;
import org.jhears.util.ArrayIterator;
import org.jhears.util.AbstractIterationObserver;

public class DBMapRepeatCache implements IRepeatGroupCache {

	private final DatabaseMap<IRepeatGroupKey, Set<IRepeatGroupValue>> map;

	public DBMapRepeatCache(
			DatabaseMap<IRepeatGroupKey, Set<IRepeatGroupValue>> map) {
		this.map = map;
	}

	@Override
	public void add(IRepeatGroupKey key, IRepeatGroupValue value) {
		Set<IRepeatGroupValue> set = map.get(key);
		if (set == null) {
			set = new HashSet<IRepeatGroupValue>();
		}
		set.add(value);
		map.put(key, set);
	}

	@Override
	public void clear() {
		map.clear();
	}

	@Override
	public void delete(IRepeatGroupKey key, IRepeatGroupValue value) {
		Set<IRepeatGroupValue> set = map.get(key);
		if (set != null) {
			set.remove(value);
			if (set.isEmpty()) {
				map.remove(key);
			} else {
				map.put(key, set);
			}
		}
	}

	@Override
	public Set<IRepeatGroupValue> get(IRepeatGroupKey key) {
		Set<IRepeatGroupValue> set = map.get(key);
		if (set == null) {
			set = new HashSet<IRepeatGroupValue>();
		}
		return set;
	}

	@Override
	public Map<IRepeatGroupKey, Set<IRepeatGroupValue>> getAll(
			IRepeatGroupKey[] keys) {
		final Map<IRepeatGroupKey, Set<IRepeatGroupValue>> retval = new HashMap<IRepeatGroupKey, Set<IRepeatGroupValue>>(
				keys.length);
		Iterator<IRepeatGroupKey> iter1 = new ArrayIterator<IRepeatGroupKey>(
				keys);
		final Iterator<IRepeatGroupKey> iter2 = new ArrayIterator<IRepeatGroupKey>(
				keys);
		map.getAll(iter1, new AbstractIterationObserver<Set<IRepeatGroupValue>>() {

			@Override
			public void observe(Set<IRepeatGroupValue> elt) {
				retval.put(iter2.next(), elt);

			}
		});
		return retval;
	}

}
