/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.cache;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.jhears.data.DatabaseMap;
import org.jhears.util.ArrayIterator;
import org.jhears.util.AbstractIterationObserver;

public class DBMapSegmentCache implements ISegmentCache {

	private final DatabaseMap<String, Set<ISegmentCacheValue>> map;

	public DBMapSegmentCache(DatabaseMap<String, Set<ISegmentCacheValue>> map) {
		this.map = map;
	}

	@Override
	public void add(String key, ISegmentCacheValue value) {
		Set<ISegmentCacheValue> set = map.get(key);
		if (set == null) {
			set = new HashSet<ISegmentCacheValue>();
		}
		set.add(value);
		map.put(key, set);
	}

	@Override
	public void clear() {
		map.clear();
	}

	@Override
	public void delete(String key, ISegmentCacheValue value) {
		Set<ISegmentCacheValue> set = map.get(key);
		if (set != null) {
			set.remove(value);
			if (set.isEmpty()) {
				map.remove(key);
			} else {
				map.put(key, set);
			}
		}
	}

	@Override
	public Set<ISegmentCacheValue> get(String key) {
		Set<ISegmentCacheValue> set = map.get(key);
		if (set == null) {
			set = new HashSet<ISegmentCacheValue>();
		}
		return set;
	}

	@Override
	public Map<String, Set<ISegmentCacheValue>> getAll(String[] keys) {
		final Map<String, Set<ISegmentCacheValue>> retval = new HashMap<String, Set<ISegmentCacheValue>>(
				keys.length);
		Iterator<String> iter1 = new ArrayIterator<String>(keys);
		final Iterator<String> iter2 = new ArrayIterator<String>(keys);
		map.getAll(iter1, new AbstractIterationObserver<Set<ISegmentCacheValue>>() {

			@Override
			public void observe(Set<ISegmentCacheValue> elt) {
				retval.put(iter2.next(), elt);
			}

		});
		return retval;
	}

}
