/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;

import org.hibernate.annotations.Index;

@Entity
public class LogEntry {

	public static final String OBJECT_ID = "objectId";
	public static final String OBJECT_ENTITY = "objectEntity";
	public static final String USER_ID = "userId";
	public static final String KEY = "key";
	public static final String VALUE = "value";

	private Long id;

	/** What */
	private String key;

	/** We are speaking of apples or oranges */
	private String objectEntity;

	/** Which particular apple */
	private Long objectId;

	/** When */
	private Long timeInMillis;

	/** Who */
	private Long userId;

	/** Details */
	private String value;

	@Id
	@GeneratedValue
	@Column(updatable = false)
	public Long getId() {
		return id;
	}

	@Index(name = "keyIndex")
	@Column(updatable = false, nullable = false)
	public String getKey() {
		return key;
	}

	@Column(updatable = false)
	@Index(name = "objectEntityIndex")
	public String getObjectEntity() {
		return objectEntity;
	}

	@Column(updatable = false)
	@Index(name = "objectIdIndex")
	public Long getObjectId() {
		return objectId;
	}

	@Column(updatable = false, nullable = false)
	@Index(name = "timeInMillisIndex")
	public Long getTimeInMillis() {
		return timeInMillis;
	}

	@Column(updatable = false)
	@Index(name = "userIdIndex")
	public Long getUserId() {
		return userId;
	}

	@Column(updatable = false)
	public String getValue() {
		return value;
	}

	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public void setObjectEntity(String objectEntity) {
		this.objectEntity = objectEntity;
	}

	public void setObjectId(Long objectId) {
		this.objectId = objectId;
	}

	public void setTimeInMillis(Long timeInMillis) {
		this.timeInMillis = timeInMillis;
	}

	public void setUserId(Long userId) {
		this.userId = userId;
	}

	public void setValue(String value) {
		this.value = value;
	}

}
