/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import java.io.Serializable;
import java.security.Principal;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToMany;

import org.jhears.server.IUser;
import org.jhears.util.Identifiable;

@Entity
public class UserDataObj implements IUser, Serializable, Principal,
		Identifiable<Long> {

	private static final long serialVersionUID = -5581678204803382296L;

	public static final String PASSWORD = "password";

	public static final String PASSWORD_SALT = "password-salt";

	public static final String CREATED_TIMESTAMP = "created-timestamp";

	private Long id;

	private String name;

	private Set<RoleDataObj> roles;

	private Map<String, String> properties;

	@ManyToMany(fetch = FetchType.LAZY)
	public Set<RoleDataObj> getRoles() {
		if (roles == null) {
			setRoles(new HashSet<RoleDataObj>());
		}
		return roles;
	}

	public void setRoles(Set<RoleDataObj> roles) {
		this.roles = roles;
	}

	@Id
	@GeneratedValue
	@Override
	public Long getId() {
		return id;
	}

	@Override
	@Column(nullable = false, unique = true)
	public String getName() {
		return name;
	}

	@Override
	@ElementCollection(fetch = FetchType.EAGER)
	public Map<String, String> getProperties() {
		return properties;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setProperties(Map<String, String> properties) {
		this.properties = properties;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		UserDataObj other = (UserDataObj) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

}
