/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.server;

import org.jhears.web.rs.Segment;

/**
 * Encapsulates a query {@link Segment} and a relative position where it
 * matched.
 */
public class PotentialMatch {

	private final Segment querySegment;

	private final int relativePosition;

	/**
	 * 
	 * @param querySegment
	 *            Segment from query string
	 * @param relativePosition
	 *            Relative match position. Calculated as
	 *            <code>querySegment.getPosition() - cacheSegment.getPosition()</code>
	 *            .
	 */
	public PotentialMatch(Segment querySegment, int relativePosition) {
		this.querySegment = querySegment;
		this.relativePosition = relativePosition;
	}

	public Segment getQuerySegment() {
		return querySegment;
	}

	/**
	 * Relative match position. Calculated as
	 * <code>querySegment.getPosition() - cacheSegment.getPosition()</code>
	 * 
	 * @return
	 */
	public int getRelativePosition() {
		return relativePosition;
	}

	@Override
	public String toString() {
		return PotentialMatch.class.getSimpleName() + " [querySegment="
				+ querySegment + ", relativePosition=" + relativePosition + "]";
	}

}
