/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.server;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jhears.cache.IRepeatGroupCache;
import org.jhears.cache.IRepeatGroupValue;
import org.jhears.cache.RepeatGroupKey;
import org.jhears.data.FingerprintDataObj;
import org.jhears.seq.RepeatGroup;
import org.jhears.tuple.Tuple2;
import org.jhears.web.rs.Repeat;

public class RepeatCacheFacade {

	// private final RepeatGroupCache oldcache;

	private final IRepeatGroupCache cache;
	private final RepeatGroupKeyMaker keyMaker;

	public RepeatCacheFacade(IRepeatGroupCache cache, RepeatGroupKeyMaker keyMaker) {
		// this.oldcache = cache;
		this.keyMaker = keyMaker;
		this.cache = cache;
	}

	/**
	 * 
	 * @param fingerprintId
	 *            Value representing {@link FingerprintDataObj#getId()}.
	 * @param repeats
	 * @throws CacheFullException
	 */
	public void add(long fingerprintId, List<Repeat> repeats) {
		List<Tuple2<RepeatGroupKey, IRepeatGroupValue>> list = keyMaker
				.makeInsertList(fingerprintId, repeats);
		for (Tuple2<RepeatGroupKey, IRepeatGroupValue> tuple : list) {
			cache.add(tuple.get1(), tuple.get2());
		}
	}

	public void clear() {
		cache.clear();
	}

	public void delete(long fingerprintId, List<Repeat> repeats) {
		if (repeats.isEmpty()) {
			return;
		}

		List<Tuple2<RepeatGroupKey, IRepeatGroupValue>> insertKeys = keyMaker
				.makeInsertList(fingerprintId, repeats);

		for (Tuple2<RepeatGroupKey, IRepeatGroupValue> tuple : insertKeys) {
			cache.delete(tuple.get1(), tuple.get2());
		}
	}

	/**
	 * 
	 * @param repeats
	 * @return Map which key set represent values of
	 *         {@link FingerprintDataObj#getId()}.
	 */
	public Map<Long, Set<RepeatSearchEntry>> query(List<Repeat> repeats) {
		Map<Long, Set<RepeatSearchEntry>> retval = new HashMap<Long, Set<RepeatSearchEntry>>();
		if (repeats.isEmpty()) {
			return retval;
		}
		List<Tuple2<RepeatGroupKey, int[]>> lookupKeys = keyMaker
				.makeSearchList(repeats);
		for (Tuple2<RepeatGroupKey, int[]> tuple : lookupKeys) {
			RepeatGroupKey key = tuple.get1();
			int[] qpos = tuple.get2();
			Set<IRepeatGroupValue> entries = cache.get(key);
			for (IRepeatGroupValue cacheEntry : entries) {
				RepeatGroup rg = new RepeatGroup(cacheEntry.getPosition(), key
						.getChars(), cacheEntry.getOriginalValues());
				Long fpId = Long.valueOf(cacheEntry.getFingerprintId());
				Set<RepeatSearchEntry> repeatSet = retval.get(fpId);
				if (repeatSet == null) {
					repeatSet = new HashSet<RepeatSearchEntry>();
					retval.put(fpId, repeatSet);
				}
				int i = 0;
				for (Repeat r : keyMaker.getGrouper().groupToRepeat(rg)) {
					int pos = qpos[i++] - r.getPosition();
					RepeatSearchEntry rse = new RepeatSearchEntry(r, pos);
					repeatSet.add(rse);
				}
			}
		}
		return retval;
	}
}
