/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.server;

import java.util.ArrayList;
import java.util.List;

import org.jhears.cache.IRepeatGroupCache;
import org.jhears.cache.IRepeatGroupValue;
import org.jhears.cache.RepeatGroupKey;
import org.jhears.cache.RepeatGroupValue;
import org.jhears.seq.IntQuantizer;
import org.jhears.seq.RepeatGroup;
import org.jhears.seq.RepeatGrouper;
import org.jhears.tuple.Tuple2;
import org.jhears.web.rs.Repeat;

/**
 * Generates keys for inserting and searching {@link IRepeatGroupCache}.
 * objects.
 * <p>
 * This class is thread safe.
 */
public final class RepeatGroupKeyMaker {

	private final RepeatGrouper grouper;

	private final IntQuantizer quantizer;

	public RepeatGroupKeyMaker(RepeatGrouper grouper, IntQuantizer quantizer) {
		this.grouper = grouper;
		this.quantizer = quantizer;
	}

	public RepeatGrouper getGrouper() {
		return grouper;
	}

	public List<Tuple2<RepeatGroupKey, IRepeatGroupValue>> makeInsertList(
			long fingerprintId, List<Repeat> repeatList) {
		List<RepeatGroup> groups = grouper.repeatToGroup(repeatList, false);
		List<Tuple2<RepeatGroupKey, IRepeatGroupValue>> retval = new ArrayList<Tuple2<RepeatGroupKey, IRepeatGroupValue>>();
		for (RepeatGroup group : groups) {
			int[] quantizedIndexes = quantizer
					.nearestIndexes(group.getValues());
			RepeatGroupKey repeatKey = new RepeatGroupKey(group.getChars(),
					quantizedIndexes);
			IRepeatGroupValue ce = new RepeatGroupValue(fingerprintId, group
					.getPosition(), group.getValues());
			retval.add(new Tuple2<RepeatGroupKey, IRepeatGroupValue>(repeatKey,
					ce));
		}
		return retval;
	}

	public List<Tuple2<RepeatGroupKey, int[]>> makeSearchList(
			List<Repeat> repeatList) {
		List<RepeatGroup> groups = grouper.repeatToGroup(repeatList, true);
		List<Tuple2<RepeatGroupKey, int[]>> retval = new ArrayList<Tuple2<RepeatGroupKey, int[]>>(
				repeatList.size() * 2);
		for (RepeatGroup group : groups) {
			int[][] nearbyIndexes = quantizer.nearbyIndexes(group.getValues());
			int[] positions = grouper.positions(group);
			for (int[] quantizedIndexes : nearbyIndexes) {
				RepeatGroupKey repeatKey = new RepeatGroupKey(group.getChars(),
						quantizedIndexes);
				retval.add(new Tuple2<RepeatGroupKey, int[]>(repeatKey,
						positions));
			}
		}
		return retval;
	}

}
