/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.server;

import org.jhears.web.rs.Repeat;

public class RepeatSearchEntry {

	private final int relativePosition;

	private final Repeat repeat;

	/**
	 * 
	 * @param cachedRepeat
	 * @param relativePosition
	 *            Position relative to a query element position. Calculated as
	 *            <code>queryRepeat.getPosition() - cacheRepeat.getPosition()</code>
	 *            .
	 */
	public RepeatSearchEntry(Repeat cachedRepeat, int relativePosition) {
		this.repeat = cachedRepeat;
		this.relativePosition = relativePosition;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RepeatSearchEntry other = (RepeatSearchEntry) obj;
		if (relativePosition != other.relativePosition)
			return false;
		if (repeat == null) {
			if (other.repeat != null)
				return false;
		} else if (!repeat.equals(other.repeat))
			return false;
		return true;
	}

	/**
	 * Position relative to a query element position. Calculated as
	 * <code>queryRepeat.getPosition() - cacheRepeat.getPosition()</code>.
	 * 
	 * @return
	 */
	public int getRelativePosition() {
		return relativePosition;
	}

	public Repeat getRepeat() {
		return repeat;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + relativePosition;
		result = prime * result + ((repeat == null) ? 0 : repeat.hashCode());
		return result;
	}

	@Override
	public String toString() {
		return RepeatSearchEntry.class.getSimpleName() + " [relativePosition="
				+ relativePosition + ", repeat=" + repeat + "]";
	}

}
