/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.server;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jhears.web.rs.Segment;

/**
 * Presents search results related to some fingerprint id.
 */
public class SearchEntry {

	/**
	 * Keys of the map are the {@link Segment}s stored into cache/database.
	 * Values are list of {@link PotentialMatch} objects.
	 */
	private final Map<Segment, List<PotentialMatch>> map;

	private int positionLastSeen;

	/**
	 * @param positionLastSeen
	 *            position in query string where this fingerprint was last seen.
	 */
	public SearchEntry(int lastSeen) {
		this.map = new HashMap<Segment, List<PotentialMatch>>();
		this.positionLastSeen = lastSeen;
	}

	/**
	 * Relative positions within the query string which matched segment of a
	 * fingerprint. Relative position is calculated as
	 * {@link Segment#getPosition()} - {@link Segment#getPosition()}. In other
	 * words the position of the fingerprint segment is subtracted from the
	 * query string segment.
	 * 
	 * @return
	 */
	public Map<Segment, List<PotentialMatch>> getMap() {
		return map;
	}

	/**
	 * position in query string where this fingerprint was last seen.
	 * 
	 * @return
	 */
	public int getPositionLastSeen() {
		return positionLastSeen;
	}

	/**
	 * Refresh this search entry / fingerprint last seen position. Used to
	 * filter out non-matching search entries during the query.
	 * 
	 * @param positionLastSeen
	 */
	public void setPositionLastSeen(int positionLastSeen) {
		this.positionLastSeen = positionLastSeen;
	}

	@Override
	public String toString() {
		return SearchEntry.class.getSimpleName() + " [map=" + map
				+ ", positionLastSeen=" + positionLastSeen + "]";
	}

}
