/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.server;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jhears.data.FingerprintDataObj;
import org.jhears.web.rs.Repeat;
import org.jhears.web.rs.RepeatCluster;
import org.jhears.web.rs.Segment;
import org.jhears.web.rs.SegmentCluster;

/**
 * Provides a simple interface for searching segments and repeats.
 * <p>
 * This class is thread safe.
 */
public class SearchFacade {

	private final RepeatCacheFacade repeatSearch;

	private final RepeatSearchFilter repeatSearchFilter;

	private final SegmentSearchFilter searchFilter;

	private final SegmentCacheSearch segmentSearch;

	public SearchFacade(SegmentCacheSearch segmentSearch,
			SegmentSearchFilter searchFilter,
			RepeatSearchFilter repeatSearchFilter,
			RepeatCacheFacade repeatSearch) {
		this.segmentSearch = segmentSearch;
		this.searchFilter = searchFilter;
		this.repeatSearchFilter = repeatSearchFilter;
		this.repeatSearch = repeatSearch;
	}

	public void addToCache(Long fpId, List<Segment> segments,
			List<Repeat> repeats) {
		segmentSearch.add(fpId, segments);
		repeatSearch.add(fpId, repeats);
	}

	public void deleteFromCache(long fpId, List<Segment> segments,
			List<Repeat> repeats) {
		repeatSearch.delete(fpId, repeats);
		segmentSearch.delete(fpId, segments);
	}

	/**
	 * 
	 * @param repeats
	 * @return Map key set represents values of
	 *         {@link FingerprintDataObj#getId()}.
	 */
	public Map<Long, List<RepeatCluster>> searchRepeats(List<Repeat> repeats) {
		Map<Long, Set<RepeatSearchEntry>> map = repeatSearch.query(repeats);
		return repeatSearchFilter.filter(map);
	}

	/**
	 * 
	 * @param segments
	 * @return Map key set represents values of
	 *         {@link FingerprintDataObj#getId()}.
	 */
	public Map<Long, List<SegmentCluster>> searchSegments(List<Segment> segments) {
		Map<Long, SearchEntry> map = segmentSearch.query(segments);
		if (map.isEmpty()) {
			return new HashMap<Long, List<SegmentCluster>>(0);
		}
		Map<Long, List<SegmentCluster>> fingerprintScores = searchFilter
				.filter(map);
		return fingerprintScores;
	}

}
