/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.catalog;

import java.beans.PropertyVetoException;
import java.io.File;
import java.io.IOException;
import java.util.Random;

import javax.naming.NamingException;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.io.FileUtils;
import org.jhears.cnf.CnfManager;
import org.jhears.server.CatalogEngineBuilder;
import org.jhears.server.FPCatalogEngine;
import org.jhears.web.rs.InsertRequest;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.picocontainer.DefaultPicoContainer;

import com.google.common.io.Files;

public class FPCatalogEngineIntegrationTest2 {

	private static File tmpDir;

	@BeforeClass
	public static void setupClass() throws IOException, PropertyVetoException,
			NamingException {

		tmpDir = Files.createTempDir();
		System.setProperty("database.home", tmpDir.getAbsolutePath());
		System.setProperty(CnfManager.PROP_NAME_HOME, tmpDir.getAbsolutePath());
	}

	@AfterClass
	public static void tearDownClass() throws IOException {
		FileUtils.deleteDirectory(tmpDir);
		// Files.deleteRecursively(tmpDir); // guava bug 359
	}

	@Test
	public void testInsertSearch() throws ConfigurationException, IOException {

		CatalogEngineBuilder builder = new CatalogEngineBuilder(
				new DefaultPicoContainer(), new Random());
		FPCatalogEngine engine = builder.build();

		String audioEntityId = "999378c6-77cf-452e-883e-b40cb261db61";
		String fingerprint = "\\22WW2QQ2WW2\\2DQDQWCWmQmCCC\\2WGmmGCCCmm\\2WmCCCCmmCnnbGmrrCWW\\22Q\\224WWWW2Q2WWW\\2Q2QQWCCWmmrrCCmmmCCCGmmCCCCR\\2rRrebRRWWCCWW\\22Q\\2Qmmmm\\2mQmQQ\\3Q2mmmQQQQ22QQQQ222QQmmmQQQQQQmmmQQmmmQQQQ22QQQ2\\22mmmmmmmmmmQQQmmmQQQ222QQmmmm\\2mQQQQmm\\2WmmmmmmmmQQQmmmmWWWaaQQQ\\32+Q22++\\2+ZZ\\2Z+++++\\22+22\\22+111++++++++++++++++++\\2Z+Z\\2+Z\\2+222QQQ222QQQQQ2\\224zVV1+++++2222\\22QQWWW\\2W2222222QQQQ\\2a2222W\\34W2WWWW\\2WqRRRR\\24R\\24m\\2Q22222D\\2DMD\\2D2222222222\\2Q2QQ\\2QWW\\2WmQQQ22222222222222\\2W2WWWWmmmmQQ\\2Q2WWmmQQWWWW2222aaaWWWWWmm\\2Q22WWmmmmmQQ222222222mmmmmQQQ\\22mmmQQQmmmQQQ\\3m2Q\\22m2\\242mmmmmm\\2mQQQ\\2QmmmmQQmmmQQQQm22QQQ\\2Q2mmmm\\2mQ\\2QmmmmQQQmm\\2m2QQQQ22QQmmmmmmQQQmmmm\\2mQmmmmQQQQ\\22Q22qVV2\\22QQQ\\2QZZ++++\\2ZMZZ\\2M+\\2+M\\2+Z\\2+Z++++XXoooo\\2+1+++++++++++++++Z\\2Z+2222XXX222QQQ2222QQQ\\2Q22+\\2+Z\\2Z+QQQQQQ\\22Q222222222QQQQQQ\\22Q22222\\2424RrrR\\2Gmmm\\2Wmaa\\22a2\\2D2D\\2DMD222\\22QQ\\22Q\\2a2aa\\2WaWmm\\3Wma\\2Waa\\22a222222222a\\22a22222WWmmmm\\2mWWW\\3mWQQmmmmmQ2\\2W111ww2aWWmmmW\\2aWaa2mmmmmmmmQQQQQQQ224vfffl\\2aWWW\\2KWKKa\\2aW222224422WW\\2WCC\\2WCWGGWWPccCCW\\2Wa\\2424444\\24W44444\\34RGGGGG\\2GW\\2mWmWWWWWWWWWWCCCCWCCC\\2CWWaaWWW\\2W44\\2424444444444444444aaaaaKPPPP4\\24222\\2D2\\22D2222222444a\\2a2a22WWPPPPW\\22W22\\22D\\2D222DDDD2222WWWWWR\\2tvPP\\2qPW\\2W22222222222W\\2vW\\24v\\2W4WWWWW\\2aWWWWWWjj\\2jK\\2Wj\\2Wj\\2WKKaaaaaa44444WWammaaaaQQaCCCa\\2amaaCC\\2GammGCbGGmmCCCCmmCCCCGmGCCW\\2W4444W22QQQ\\2m4\\2m42\\2QDDDWCCWQQRC\\2CeGmmmCCCGmmGCCCmmrbb\\2bGGWGCCWWRRRWWmmmmmm\\3mGWW\\22W2\\2WaWaa2\\2W2222QQQQQ\\2QmQQQQ\\2Q22\\2a22QQQQ\\3m2Q\\22QQQQQ\\2m2224RRRRWmmmmmmWa\\2a2aaaam222222\\221222222QQ2\\2aWW\\22QQQmmmQQQ\\2Q222222\\2a2aaaaa\\2aWWWW\\2Waa2\\242WWWWWWW\\2aWa4\\24U\\3zU2\\3az2aaaaaWWWWW\\22a2WWWWW4\\24P\\2vPvPPPWWCWWWWWW\\2aWaaaaaWWWW\\2a2\\2244444\\24aa\\2WaWWWWWWWWWWW22\\2W22222W\\2PWPPUUU2444RGGGGGGGR\\2vRv4444OOO444hhhhhhhh444444\\2422QQQQQmmmmmmmmmmmm\\3am4aaa444\\2W4WWaazzzzzzz\\32Uz2222\\3Uz2a\\2zazzz22\\22zzzzz";

		// if lazy loading is used this test will cause hibernate to deadlock
		InsertRequest insertReq = new InsertRequest(audioEntityId, fingerprint);
		engine.insertFingerprint(insertReq.getAudioEntityId(), insertReq
				.getFingerprint(), null);
		engine.insertFingerprint(insertReq.getAudioEntityId(), insertReq
				.getFingerprint(), null);

	}
}
