/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.catalog;

import java.util.ArrayList;
import java.util.List;

import org.jhears.server.RepeatInsertFilter;
import org.jhears.web.rs.Repeat;
import org.junit.Assert;
import org.junit.Test;

public class RepeatInsertFilterTest {

	private List<Repeat> makeRepeats(int size) {
		ArrayList<Repeat> retval = new ArrayList<Repeat>(size);
		for (int i = 0; i < size; i++) {
			retval.add(new Repeat('i', i + 10, i + 20));
		}
		return retval;
	}

	@Test
	public void testEmpty() {
		RepeatInsertFilter filter = new RepeatInsertFilter(16);
		List<Repeat> existingRepeats = new ArrayList<Repeat>(0);
		List<Repeat> seqRepeats = makeRepeats(0);
		List<Repeat> actual = filter.extractInsertableRepeats(seqRepeats,
				existingRepeats);
		Assert.assertEquals(0, actual.size());
	}

	@Test
	public void testExactSize() {
		RepeatInsertFilter filter = new RepeatInsertFilter(16);
		List<Repeat> existingRepeats = new ArrayList<Repeat>(0);
		for (int i = 0; i < 10; i++) {
			List<Repeat> seqRepeats = makeRepeats(16);
			List<Repeat> actual = filter.extractInsertableRepeats(seqRepeats,
					existingRepeats);
			Assert.assertEquals(16, actual.size());
		}
	}

	@Test
	public void testLackingOne() {
		RepeatInsertFilter filter = new RepeatInsertFilter(16);
		List<Repeat> existingRepeats = new ArrayList<Repeat>(0);
		List<Repeat> seqRepeats = makeRepeats(15);
		List<Repeat> actual = filter.extractInsertableRepeats(seqRepeats,
				existingRepeats);
		Assert.assertEquals(0, actual.size());
	}

	@Test
	public void testOne() {
		RepeatInsertFilter filter = new RepeatInsertFilter(16);
		List<Repeat> existingRepeats = new ArrayList<Repeat>(0);
		List<Repeat> seqRepeats = makeRepeats(1);
		List<Repeat> actual = filter.extractInsertableRepeats(seqRepeats,
				existingRepeats);
		Assert.assertEquals(0, actual.size());
	}

	@Test
	public void testPlusOne() {
		RepeatInsertFilter filter = new RepeatInsertFilter(16);
		List<Repeat> existingRepeats = new ArrayList<Repeat>(0);
		for (int i = 0; i < 10; i++) {
			List<Repeat> seqRepeats = makeRepeats(17);
			List<Repeat> actual = filter.extractInsertableRepeats(seqRepeats,
					existingRepeats);
			Assert.assertEquals(17, actual.size());
		}
	}

	@Test
	public void testPlusTwo() {
		RepeatInsertFilter filter = new RepeatInsertFilter(16);
		List<Repeat> existingRepeats = new ArrayList<Repeat>(0);
		for (int i = 0; i < 10; i++) {
			List<Repeat> seqRepeats = makeRepeats(18);
			List<Repeat> actual = filter.extractInsertableRepeats(seqRepeats,
					existingRepeats);
			Assert.assertEquals(18, actual.size());
		}
	}

}
