/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.catalog;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.jhears.server.tool.SeqUtil;
import org.jhears.web.rs.Segment;
import org.junit.Test;

public class SeqUtilTest {

	@Test
	public void test1Orphan() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		list.add(new Segment("", 0));
		int segLen = 8;
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(list.isEmpty());
	}

	@Test
	public void test1Valid() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen));
		list.add(new Segment("", segLen * 2));
		ArrayList<Segment> copy = new ArrayList<Segment>(list);
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(copy.equals(list));
	}

	@Test
	public void test1Valid2Orphans() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 7));
		list.add(new Segment("", segLen * 9));
		list.add(new Segment("", segLen * 11));
		ArrayList<Segment> copy = new ArrayList<Segment>(list);
		copy.remove(3);
		copy.remove(3);
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(copy.equals(list));
	}

	@Test
	public void test1Valid2Orphans2() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 7));
		list.add(new Segment("", segLen * 9));
		list.add(new Segment("", segLen * 11));
		ArrayList<Segment> copy = new ArrayList<Segment>(list);
		copy.remove(3);
		copy.remove(3);
		int requredAdjacent = 3;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(copy.equals(list));
	}

	@Test
	public void test2Orphan() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		list.add(new Segment("", 0));
		list.add(new Segment("", 16));
		int segLen = 8;
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(list.isEmpty());
	}

	@Test
	public void test2Valid() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen));
		list.add(new Segment("", segLen * 2));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		ArrayList<Segment> copy = new ArrayList<Segment>(list);
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(copy.equals(list));
	}

	@Test
	public void test2Valid1OrphanBegin() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 2));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 10));
		list.add(new Segment("", segLen * 11));
		ArrayList<Segment> copy = new ArrayList<Segment>(list);
		copy.remove(0);
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(copy.equals(list));
	}

	@Test
	public void test2Valid1OrphanEnd() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 10));
		list.add(new Segment("", segLen * 11));
		list.add(new Segment("", segLen * 13));
		ArrayList<Segment> copy = new ArrayList<Segment>(list);
		copy.remove(4);
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(copy.equals(list));
	}

	@Test
	public void test2Valid1OrphanMiddle() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 8));
		list.add(new Segment("", segLen * 10));
		list.add(new Segment("", segLen * 11));
		ArrayList<Segment> copy = new ArrayList<Segment>(list);
		copy.remove(2);
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(copy.equals(list));
	}

	@Test
	public void test3Orphan() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen));
		list.add(new Segment("", segLen * 3));
		list.add(new Segment("", segLen * 5));
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(list.isEmpty());
	}

	@Test
	public void test3Orphans() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 2));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 7));
		list.add(new Segment("", segLen * 9));
		list.add(new Segment("", segLen * 11));
		int requredAdjacent = 3;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(list.isEmpty());
	}

	@Test
	public void testEmpty() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		int requredAdjacent = 2;
		SeqUtil.removeOrphans(list, segLen, requredAdjacent);
		Assert.assertTrue(list.isEmpty());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testIllegalArg() {
		int segLen = 8;
		int requredAdjacent = 1;
		SeqUtil
				.removeOrphans(new ArrayList<Segment>(), segLen,
						requredAdjacent);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testIllegalArg2() {
		int segLen = 0;
		int requredAdjacent = 2;
		SeqUtil
				.removeOrphans(new ArrayList<Segment>(), segLen,
						requredAdjacent);
	}

	@Test
	public void testSelect0Valid() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 1));
		list.add(new Segment("", segLen * 3));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 8));
		list.add(new Segment("", segLen * 9));
		int requredAdjacent = 2;
		int selectCnt = 3;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.clear();
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 2));
		list.add(new Segment("", segLen * 3));
		int requredAdjacent = 2;
		int selectCnt = 2;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid1Orphan1() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 1));
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 5));
		int requredAdjacent = 2;
		int selectCnt = 2;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.remove(0);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid1Orphan2() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 7));
		int requredAdjacent = 2;
		int selectCnt = 2;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.remove(2);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid2() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		int requredAdjacent = 2;
		int selectCnt = 2;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.remove(2);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid3() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 7));
		list.add(new Segment("", segLen * 8));
		int requredAdjacent = 2;
		int selectCnt = 2;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.remove(2);
		list.remove(2);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid4() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 8));
		list.add(new Segment("", segLen * 9));
		int requredAdjacent = 2;
		int selectCnt = 2;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.remove(2);
		list.remove(2);
		list.remove(2);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid5() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 1));
		list.add(new Segment("", segLen * 3));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 8));
		list.add(new Segment("", segLen * 9));
		int requredAdjacent = 2;
		int selectCnt = 2;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.remove(0);
		list.remove(0);
		list.remove(0);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid6() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 1));
		list.add(new Segment("", segLen * 3));
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 5));
		list.add(new Segment("", segLen * 9));
		int requredAdjacent = 2;
		int selectCnt = 3;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		list.remove(0);
		list.remove(3);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelect1Valid7() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 3));
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 7));
		int requredAdjacent = 2;
		int selectCnt = 3;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		Assert.assertTrue(list.equals(selection));
	}

	@Test
	public void testSelectValid8() {
		ArrayList<Segment> list = new ArrayList<Segment>();
		int segLen = 8;
		list.add(new Segment("", segLen * 2));
		list.add(new Segment("", segLen * 3));
		list.add(new Segment("", segLen * 4));
		list.add(new Segment("", segLen * 6));
		list.add(new Segment("", segLen * 7));
		list.add(new Segment("", segLen * 8));
		int requredAdjacent = 3;
		int selectCnt = 4;
		List<Segment> selection = SeqUtil.select(list, segLen, requredAdjacent,
				selectCnt);
		Assert.assertTrue(list.equals(selection));
	}
}
