/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.jhears.data.DatabaseMapTest.IdGn;
import org.jhears.filter.bloom.AbstractBloomFilterTest;
import org.jhears.filter.bloom.BloomFilter;
import org.jhears.filter.bloom.HVBloomFilter;
import org.jhears.filter.bloom.Hasher;
import org.jhears.filter.bloom.IBloomFilter;
import org.jhears.filter.bloom.IHasher;
import org.jhears.filter.bloom.util.BloomMath;
import org.jhears.util.DefaultHashFunction;
import org.jhears.util.HashFunction;
import org.jhears.util.LCG;
import org.jhears.util.Serialization;
import org.jhears.util.Serializer;
import org.junit.AfterClass;
import org.junit.BeforeClass;

import com.google.common.io.Files;
import com.sleepycat.je.Durability;
import com.sleepycat.je.EnvironmentConfig;

public class DatabaseBloomFilterTest extends AbstractBloomFilterTest {

	private static HashFunction<?> hashFunction;
	private static BerkeleyMapManager manager;

	private static Serialization serialization;
	private static File envHome;

	@AfterClass
	public static void afterClass() throws IOException {
		FileUtils.deleteDirectory(envHome);
		// Files.deleteRecursively(envHome); // guava bug 359
	}

	@BeforeClass
	public static void before() {
		envHome = Files.createTempDir();
		EnvironmentConfig cnf = BerkeleyMapManager
				.getDefaultEnvironmentConfig();
		cnf.setDurability(Durability.COMMIT_NO_SYNC);
		cnf.setLocking(false);
		manager = new BerkeleyMapManager(envHome);
		serialization = new Serializer();
		hashFunction = new DefaultHashFunction<Object>();
	}

	@Override
	protected IBloomFilter<Long> createFilter(int size, double p0) {
		int m0 = BloomMath.optimalSliceSize(size);
		int k0 = BloomMath.optimalSliceCount(p0);
		DatabaseBFState state = new DatabaseBFState(m0, k0, manager,
				DatabaseCountingBloomFilterTest.class.getSimpleName() + "-"
						+ IdGn.get(), serialization, hashFunction);
		state.clear();
		HVBloomFilter alg = new HVBloomFilter(state);
		HashFunction<Object> genericHash = new DefaultHashFunction<Object>();
		IHasher hasher = new Hasher(genericHash, new LCG());
		return new BloomFilter<Long>(alg, hasher);
	}

	@Override
	public void testP0OneBasispoint() {
	}

	@Override
	public void testP0OnePermil() {
	}

	@Override
	public void testP0OnePercet() {
	}

}
