/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.data;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.jhears.bitarray.ICountingBitArray;
import org.jhears.bitarray.ICountingBitArrayTest;
import org.jhears.util.DefaultHashFunction;
import org.jhears.util.HashFunction;
import org.jhears.util.Serialization;
import org.jhears.util.Serializer;
import org.junit.AfterClass;
import org.junit.BeforeClass;

import com.google.common.io.Files;
import com.sleepycat.je.DatabaseConfig;
import com.sleepycat.je.Durability;
import com.sleepycat.je.EnvironmentConfig;

public class DatabaseCountingBitArrayTest extends ICountingBitArrayTest {

	private static String databaseName;

	private static HashFunction<Long> hashFunction;

	private static BerkeleyMapManager manager;

	private static Serialization serialization;

	private static File envHome;

	@AfterClass
	public static void afterClass() throws IOException {
		try {
			manager.shutDown();
		} finally {
			FileUtils.deleteDirectory(envHome);
			// Files.deleteRecursively(envHome); // guava bug 359
		}
	}

	@BeforeClass
	public static void beforeClass() {
		envHome = Files.createTempDir();
		EnvironmentConfig cnf = BerkeleyMapManager
				.getDefaultEnvironmentConfig();
		cnf.setDurability(Durability.COMMIT_NO_SYNC);
		cnf.setLocking(false);
		manager = new BerkeleyMapManager(envHome);
		databaseName = "DatabaseBitArray";
		serialization = new Serializer();
		hashFunction = new DefaultHashFunction<Long>();
	}

	@Override
	protected ICountingBitArray getBitArray(int capacity) {
		DatabaseConfig databaseConfig = BerkeleyMapManager
				.getDefaultDatabaseConfig();
		DatabaseMap<Long, Byte> map = new BerkeleyMap<Long, Byte>(manager,
				databaseName, databaseConfig, serialization, hashFunction);
		DatabaseCountingBitArray arr = new DatabaseCountingBitArray(map,
				capacity);
		arr.clear();
		return arr;
	}

	@Override
	protected int[] getCapacities() {
		return new int[] { 7, 8, 9, 30, 31, 32, 33, 34, 62, 63, 64, 65, 66,
				126, 127, 128, 129, 130 };
	}

}
