/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.seq;

import java.util.List;

import org.jhears.cache.IRepeatGroupValue;
import org.jhears.cache.RepeatGroupKey;
import org.jhears.server.RepeatGroupKeyMaker;
import org.jhears.tuple.Tuple2;
import org.jhears.web.rs.Repeat;
import org.junit.Assert;
import org.junit.Test;

public class RepeatGroupCEKeyMakerTest {

	@Test
	public void testInsertKeys() {
		IntQuantizer keyGen = new IntQuantizer(new int[] { 2, 4, 8, 16 }, 0,
				1.0, 1.0);
		RepeatGrouper lshSeed = new RepeatGrouper(3);
		RepeatGroupKeyMaker keyMaker = new RepeatGroupKeyMaker(lshSeed,
				keyGen);
		RLEnc enc = new RLEnc(3, '/');
		char[] ca = "aaaabbbccccdddeeeeeefffgggghhh".toCharArray();
		List<Repeat> list = enc.encode(ca);
		List<Tuple2<RepeatGroupKey, IRepeatGroupValue>> insertKeys = keyMaker
				.makeInsertList(123L, list);
		Assert.assertEquals(2, insertKeys.size());
		// length and positions map to indexes of the values set
		int[] exp = new int[] { 1, 1, 1, 1, 2 };

		Assert.assertArrayEquals(exp, insertKeys.get(0).get1().getValues());
		// length and positions map to indexes of the values set
		exp = new int[] { 1, 2, 1, 1, 2 };
		Assert.assertArrayEquals(exp, insertKeys.get(1).get1().getValues());
		Assert.assertArrayEquals(new char[] { 'a', 'b', 'c' }, insertKeys
				.get(0).get1().getChars());
		Assert.assertArrayEquals(new char[] { 'd', 'e', 'f' }, insertKeys
				.get(1).get1().getChars());
		Assert.assertEquals(0, insertKeys.get(0).get2().getPosition());
		Assert.assertEquals(11, insertKeys.get(1).get2().getPosition());
	}

	@Test
	public void testSearchKeys() {
		IntQuantizer keyGen = new IntQuantizer(new int[] { 2, 4, 8, 16 }, 0,
				1.0, 1.0);
		RepeatGrouper lshSeed = new RepeatGrouper(3);
		RepeatGroupKeyMaker keyMaker = new RepeatGroupKeyMaker(lshSeed,
				keyGen);
		RLEnc enc = new RLEnc(3, '/');
		char[] ca = "aaaabbbccccdddeeeeeefffgggghhh".toCharArray();
		List<Repeat> list = enc.encode(ca);
		List<Tuple2<RepeatGroupKey, int[]>> searchKeys = keyMaker
				.makeSearchList(list);
		Assert.assertEquals(6, searchKeys.size());
		char[] exp = new char[] { 'a', 'b', 'c', };
		Assert.assertArrayEquals(exp, searchKeys.get(0).get1().getChars());
		exp = new char[] { 'b', 'c', 'd' };
		Assert.assertArrayEquals(exp, searchKeys.get(1).get1().getChars());
		exp = new char[] { 'f', 'g', 'h' };
		Assert.assertArrayEquals(exp, searchKeys.get(5).get1().getChars());
	}

}
