/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.picocontainer;

import java.io.IOException;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.commons.configuration.ConfigurationException;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.map.SerializationConfig;
import org.jhears.auth.UserInformationCentralImpl;
import org.jhears.cnf.Cnf;
import org.jhears.cnf.CnfManager;
import org.jhears.cnf.Cnf.ServerKey;
import org.jhears.data.DAO;
import org.jhears.server.CatalogEngineBuilder;
import org.jhears.server.FPCatalogEngine;
import org.jhears.user.UserInformationCentral;
import org.jhears.util.SingletonMutablePicoContainer;
import org.jhears.web.rs.AudioEntityResource;
import org.jhears.web.rs.ConfigurationResource;
import org.jhears.web.rs.FingerprintResource;
import org.jhears.web.rs.InsertResource;
import org.jhears.web.rs.SearchResource;
import org.jhears.web.rs.VersionResource;
import org.jhears.web.rs.BaseResource.BaseResourceParam;
import org.picocontainer.MutablePicoContainer;
import org.picocontainer.Parameter;
import org.picocontainer.parameters.ComponentParameter;
import org.picocontainer.parameters.ConstantParameter;

import com.google.common.base.Throwables;
import com.sun.jersey.spi.pico.container.servlet.BasicPicoContainerProvider;

public class JHearsProviderListener implements ServletContextListener {

	private static final String CONTEXT_KEY = "jhears.provider";

	private FPCatalogEngine makeEngine(MutablePicoContainer child)
			throws ConfigurationException, IOException {
		// setup the catalog

		// String serverVersion = ManifestUtil.getImplementationVersion(
		// new Attributes.Name("Maven-Group-Artifact"),
		// "org.jhears:jhears-server");
		// if (serverVersion == null) {
		// serverVersion = "development";
		// }
		// getServletContext().setAttribute("serverVersion", serverVersion);

		CatalogEngineBuilder cp = new CatalogEngineBuilder(child, new Random());
		FPCatalogEngine engine = cp.build();
		return engine;
	}

	@Override
	public void contextInitialized(ServletContextEvent event) {
		MutablePicoContainer parent = SingletonMutablePicoContainer
				.getInstance();

		boolean security;
		try {
			makeEngine(parent);
			security = parent.getComponent(CnfManager.class).getConfiguration(
					Cnf.server).getBoolean(ServerKey.SECURITY);
		} catch (ConfigurationException e) {
			throw Throwables.propagate(e);
		} catch (IOException e) {
			throw Throwables.propagate(e);
		}

		Set<Class<?>> resources = new HashSet<Class<?>>();
		resources.add(AudioEntityResource.class);
		resources.add(ConfigurationResource.class);
		resources.add(FingerprintResource.class);
		resources.add(InsertResource.class);
		resources.add(SearchResource.class);
		resources.add(VersionResource.class);
		for (Class<?> clazz : resources) {
			parent.addComponent(clazz);
		}

		Parameter[] params = new Parameter[4];
		params[0] = new ComponentParameter();
		params[1] = new ComponentParameter();
		params[2] = new ConstantParameter(Boolean.valueOf(security));
		params[3] = new ComponentParameter();
		parent.addComponent(BaseResourceParam.class, BaseResourceParam.class,
				params);

		ObjectMapper mapper = new ObjectMapper();
		mapper.configure(SerializationConfig.Feature.INDENT_OUTPUT, true);
		parent.addComponent(mapper);

		// for login servlet
		parent.addComponent(UserInformationCentral.class,
				new UserInformationCentralImpl(parent.getComponent(DAO.class)));

		ServletContext context = event.getServletContext();
		context.setAttribute(CONTEXT_KEY, new BasicPicoContainerProvider(
				parent, resources));
	}

	@Override
	public void contextDestroyed(ServletContextEvent event) {
	}

}
