/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.servlet;

import java.io.IOException;
import java.util.regex.Pattern;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jhears.user.UserInformation;
import org.jhears.user.UserInformationCentral;
import org.jhears.util.SingletonMutablePicoContainer;

public class LoginServlet extends HttpServlet {

	private static final String CONTENT_TYPE = "text/plain; charset=UTF-8";

	private static final Pattern pCntrl = Pattern.compile("\\p{Cntrl}");

	private static final long serialVersionUID = 2312124539936680803L;

	public LoginServlet() {
		super();
	}

	private boolean auth(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException {
		String username = req.getParameter("username");
		String password = req.getParameter("password");
		if (req.getParameter("logout") != null) {
			invalidateSession(req.getSession(false));
			return true;
		} else if (username != null && password != null) {
			prepareUserInformationCentral();
			UserInformation info = ctrl.getUserInformation(username);
			if (info != null && ctrl.authenticate(info, password)) {
				invalidateSession(req.getSession(false));
				req.getSession(true).setAttribute("jhears-user-information",
						info);
				return true;
			}
		}
		return false;
	}

	private transient volatile UserInformationCentral ctrl;

	private void prepareUserInformationCentral() {
		if (ctrl == null) {
			synchronized (LoginServlet.class) {
				if (ctrl == null) {
					ctrl = SingletonMutablePicoContainer.getInstance()
							.getComponent(UserInformationCentral.class);
				}
			}
		}
	}

	private void doAuth(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		if (auth(req, resp)) {
			String redirectOnSuccess = req.getParameter("redirectOnSuccess");
			if (redirectOnSuccess != null) {
				redir(resp, redirectOnSuccess);
			} else {
				resp.setContentType(CONTENT_TYPE);
				resp.getOutputStream().println(Boolean.TRUE.toString());
			}
		} else {
			String redirectOnFailure = req.getParameter("redirectOnFailure");
			if (redirectOnFailure != null) {
				redir(resp, redirectOnFailure);
			} else {
				resp.setContentType(CONTENT_TYPE);
				resp.getOutputStream().println(Boolean.FALSE.toString());
			}
		}
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		doAuth(req, resp);
	}

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		doAuth(req, resp);
	}

	@Override
	public void init() throws ServletException {
		super.init();
	}

	@Override
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
	}

	private void invalidateSession(HttpSession session) {
		if (session != null) {
			session.removeAttribute("jhears-user-information");
			session.invalidate();
		}
	}

	private boolean isValidRedirect(String str) {
		return !pCntrl.matcher(str).find();
	}

	private void redir(HttpServletResponse resp, String url) throws IOException {
		if (isValidRedirect(url)) {
			resp.sendRedirect(resp.encodeRedirectURL(url));
		} else {
			resp.sendError(HttpServletResponse.SC_BAD_REQUEST,
					"Invalid redirect url: " + url);
		}
	}

}
