/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.servlet;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.velocity.tools.view.VelocityViewServlet;
import org.jhears.cnf.Cnf;
import org.jhears.cnf.CnfManager;
import org.picocontainer.PicoContainer;

import com.google.common.collect.Lists;
import com.google.common.io.Files;
import com.sun.jersey.spi.pico.container.servlet.BasicPicoContainerProvider;

public class VelocityServlet extends VelocityViewServlet {

	private static final long serialVersionUID = 6509296911422743708L;

	private File srcDir;
	private Map<String, String> versionMap;
	private String identifier;

	@Override
	public void init() throws ServletException {
		BasicPicoContainerProvider provider = (BasicPicoContainerProvider) getServletContext()
				.getAttribute("jhears.provider");
		PicoContainer container = provider.getContainer();
		CnfManager man = container.getComponent(CnfManager.class);
		srcDir = new File(man.getHomeDir(), "src");
		if (!srcDir.exists()) {
			if (!srcDir.mkdirs()) {
				throw new ServletException(new IOException(
						"Failed to create directory: "
								+ srcDir.getAbsolutePath()));
			}
		}
		versionMap = new TreeMap<String, String>();
		for (String artifactId : new String[] { "jhears-core", "jhears-server",
				"jhears-api", "jhears-servlet-api" }) {
			versionMap.put(artifactId, CnfManager.getVersion(artifactId));
		}
		try {
			identifier = man.getConfiguration(Cnf.server).getString(
					Cnf.ServerKey.IDENTIFIER);
		} catch (ConfigurationException e) {
			throw new ServletException("Configuration error", e);
		}
		super.init();
	}

	@Override
	protected void doRequest(HttpServletRequest req, HttpServletResponse resp)
			throws IOException {
		req.setAttribute("versionMap", versionMap);
		req.setAttribute("identifier", identifier);
		if ("/download.vm".equals(req.getServletPath())) {
			ArrayList<File> files = Lists.newArrayList(srcDir
					.listFiles(new FileFilter() {
						@Override
						public boolean accept(File pathname) {
							return pathname.isFile() && pathname.canRead();
						}
					}));
			String fileName = req.getParameter("file");
			if (fileName != null) {
				File from = new File(srcDir, fileName);
				if (from.isFile() && files.contains(from)) {
					resp.setContentType("application/x-download");
					resp.setHeader("Content-Disposition",
							"attachment; filename=" + from.getName());
					resp.setContentLength((int) from.length());
					Files.copy(from, resp.getOutputStream());
					return;
				}
			}
			req.setAttribute("path", req.getServletPath() + "?file=");
			req.setAttribute("files", files);
		}
		super.doRequest(req, resp);
	}
}
