/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.web.rs;

import javax.ws.rs.Consumes;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.jhears.auth.JHearsRoles;
import org.jhears.server.FPCatalogEngine;

@Path(ApiPath.INSERT)
public class InsertResource extends BaseResource {

	public InsertResource(BaseResourceParam param) {
		super(param);
	}

	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(value = { MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN })
	public Response process(InsertRequest insert,
			@DefaultValue("json") @QueryParam("format") Format format) {
		if (!isAuthorized(JHearsRoles.USER)) {
			return unauthorized();
		}
		FPCatalogEngine catalog = getCatalog();
		Fingerprint fingerprint = catalog.insertFingerprint(
				insert.getAudioEntityId(), insert.getFingerprint(), getCtx());
		if (fingerprint == null) {
			return Response.status(Status.FORBIDDEN)
					.entity("Insert request was not accepted").build();
		}
		if (Format.plain.equals(format)) {
			return Response.ok(toText(fingerprint), MediaType.TEXT_PLAIN)
					.build();
		} else {
			return Response.ok(fingerprint, MediaType.APPLICATION_JSON).build();
		}
	}

}
