/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.web.rs;

import javax.ws.rs.Consumes;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.jhears.server.FPCatalogEngine;

import com.google.common.base.Throwables;

@Path(ApiPath.SEARCH)
public class SearchResource extends BaseResource {

	public SearchResource(BaseResourceParam param) {
		super(param);
	}

	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(value = { MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN })
	public Response json(String fingerprint,
			@DefaultValue("json") @QueryParam("format") Format format) {
		// if (!isAuthorized(Const.USER)) {
		// return unauthorized();
		// }

		/*
		 * The string gets here as a raw json encoded data which means that it
		 * has a quotation marks around it (as json encoded strings should). I
		 * suspect that this is either a bug in jersey or jackson. The string
		 * should have gone through json parser before arriving here...
		 */
		String fp;
		try {
			fp = readValue(fingerprint, String.class);
		} catch (Exception e) {
			throw Throwables.propagate(e);
		}
		FPCatalogEngine engine = getCatalog();
		SearchResult[] res = engine.search(fp, getCtx());
		if (Format.plain.equals(format)) {
			return Response.ok(toText(res), MediaType.TEXT_PLAIN).build();
		} else {
			return Response.ok(res, MediaType.APPLICATION_JSON).build();
		}
	}

	@GET
	@Produces(value = { MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN })
	public Response plain(@QueryParam("fingerprint") String fingerprint,
			@DefaultValue("json") @QueryParam("format") Format format) {
		// if (!isAuthorized(Const.USER)) {
		// return unauthorized();
		// }
		FPCatalogEngine engine = getCatalog();
		SearchResult[] res = engine.search(fingerprint, getCtx());
		if (Format.plain.equals(format)) {
			return Response.ok(toText(res), MediaType.TEXT_PLAIN).build();
		} else {
			return Response.ok(res, MediaType.APPLICATION_JSON).build();
		}
	}

}
