/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.concurrent;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import com.google.common.base.Preconditions;

public class Executors {

	private static Executor exec;

	/**
	 * The same as the
	 * {@link java.util.concurrent.Executors#newFixedThreadPool(int)} except
	 * that the work queue length is limited to {@code (nThreads + 1)
	 * (nThreads + 1)} and that the rejected handler is CallerRunsPolicy.
	 * 
	 * @param nThreads
	 * @return
	 */
	public static ExecutorService newFixedBoundedThreadPool(int nThreads) {
		return newFixedBoundedThreadPool(nThreads, (nThreads + 1)
				* (nThreads + 1));
	}

	/**
	 * The same as {@link #newFixedBoundedThreadPool(int)} except the queue
	 * length can be defined.
	 * 
	 * @param nThreads
	 * @param queueLength
	 * @return
	 */
	public static ExecutorService newFixedBoundedThreadPool(int nThreads,
			int queueLength) {
		Preconditions.checkArgument(nThreads > 0);
		Preconditions.checkArgument(queueLength > 0);
		return new ThreadPoolExecutor(nThreads, nThreads, 0L,
				TimeUnit.MILLISECONDS, new ArrayBlockingQueue<Runnable>(
						queueLength), new ThreadPoolExecutor.CallerRunsPolicy());
	}

	public static Executor getExecutor() {
		if (exec == null) {
			exec = new Executor() {
				@Override
				public void execute(Runnable command) {
					new Thread(command).start();
				}
			};
		}
		return exec;
	}

}
