/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.util;

import java.io.Serializable;
import java.util.Arrays;

/**
 * Wrapper for java primitive type int[] implementing {@link #equals(Object)}
 * and {@link #hashCode()} properly.
 */
public class IntArr implements Serializable {

	private static final long serialVersionUID = 4490770588053302112L;

	private final int[] intArr;

	public IntArr(final int[] intArr) {
		this.intArr = intArr;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		IntArr other = (IntArr) obj;
		if (!Arrays.equals(intArr, other.intArr))
			return false;
		return true;
	}

	public int[] getIntArr() {
		return intArr;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(intArr);
		return result;
	}

	@Override
	public String toString() {
		return IntArr.class.getSimpleName() + " [intArr="
				+ Arrays.toString(intArr) + "]";
	}

}
