/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.util;

/**
 * Observer pattern interface suitable for using with iterators.
 * 
 * @author Juha Heljoranta
 * 
 * @param <E>
 */
public interface IterationObserver<E> extends Observer<E> {

	/**
	 * True if {@link #observe(Object)} should be called. Can be used to break
	 * out from iterator loop.
	 * 
	 * @return true if iteration should be continued, false otherwise.
	 */
	public boolean observeNext();

	/**
	 * Notify that the iteration is completed for now. The implementation can
	 * use this method, for example, to release some resources.
	 */
	public void completed();

}
