/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.util;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterator which notifies the observer for every element returned by the
 * iterator.
 * <p>
 * This class is not thread safe.
 * 
 * @author Juha Heljoranta
 * 
 * @param <E>
 */
public class ObservableIterator<E> implements Iterator<E> {

	private final Iterator<E> itr;

	private final IterationObserver<E> observer;

	public ObservableIterator(Iterator<E> itr, IterationObserver<E> observer)
			throws IllegalArgumentException {
		if (itr == null) {
			throw new IllegalArgumentException("Iterator is null");
		}
		if (observer == null) {
			throw new IllegalArgumentException("Observer is null");
		}
		this.itr = itr;
		this.observer = observer;
	}

	@Override
	public boolean hasNext() {
		return itr.hasNext();
	}

	@Override
	public E next() {
		if (!hasNext()) {
			throw new NoSuchElementException();
		}
		E e = itr.next();
		observer.observe(e);
		return e;
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}

}
