/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.junit.Assert;
import org.junit.Test;

public class ParallelExecutorTest {

	private static class TestTask implements Callable<String> {

		private final long delay;

		public TestTask(long delay) {
			this.delay = delay;
		}

		@Override
		public String call() throws Exception {
			Thread.sleep(delay);
			return Long.toString(delay);
		}

	}

	@Test(timeout = 10000)
	public void test() {
		Executor executor = Executors.newFixedThreadPool(10);
		ParallelExecutor<String> pex = new ParallelExecutor<String>(executor,
				3000, TimeUnit.MILLISECONDS, true);
		List<Callable<String>> tasks = new ArrayList<Callable<String>>();
		int max = 100;
		Random r = new Random(0);
		tasks.add(new TestTask(r.nextInt(max)));
		tasks.add(new TestTask(r.nextInt(max)));
		tasks.add(new TestTask(r.nextInt(max)));
		tasks.add(new TestTask(r.nextInt(max)));
		List<String> results = pex.execute(tasks);
		Assert.assertEquals(4, results.size());
		r = new Random(0);
		for (String s : results) {
			Assert.assertEquals(Long.toString(r.nextInt(max)), s);
		}
	}

}
