/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.apache.wicket.contrib.recaptcha;

public abstract class DefaultReCaptchaConfig implements ReCaptchaConfig {

	private static final long serialVersionUID = -1942706846531777066L;

	private final String noscriptFragment;

	private final String scriptFragment;

	private final String verifyUrl;

	private final boolean secure;

	public DefaultReCaptchaConfig(boolean secure) {
		this.secure = secure;
		if (secure) {
			this.noscriptFragment = "https://api-secure.recaptcha.net/noscript?k=";
			this.scriptFragment = "https://api-secure.recaptcha.net/challenge?k=";
			this.verifyUrl = "https://api-verify.recaptcha.net/verify";
		} else {
			this.noscriptFragment = "http://api.recaptcha.net/noscript?k=";
			this.scriptFragment = "http://api.recaptcha.net/challenge?k=";
			this.verifyUrl = "http://api-verify.recaptcha.net/verify";
		}
	}

	@Override
	public boolean isSecure() {
		return secure;
	}

	/** "http://api.recaptcha.net/noscript?k=" */
	@Override
	public String getNoscriptUrlFragment() {
		return noscriptFragment;
	}

	/** "http://api.recaptcha.net/challenge?k=" */
	@Override
	public String getScriptUrlFragment() {
		return scriptFragment;
	}

	/** "http://api-verify.recaptcha.net/verify" */
	@Override
	public String getVerifyUrl() {
		return verifyUrl;
	}

}
