/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.apache.wicket.contrib.recaptcha;

import org.apache.wicket.IClusterable;

public interface ReCaptchaConfig extends IClusterable {

	/** e.g. "https://api-secure.recaptcha.net/noscript?k=" */
	String getNoscriptUrlFragment();

	String getPrivateKey();

	String getPublicKey();

	/** e.g. "https://api-secure.recaptcha.net/challenge?k=" */
	String getScriptUrlFragment();

	/**
	 * Determines the timeout in milliseconds until a connection is established.
	 * A timeout value of zero is interpreted as an infinite timeout.
	 */
	int getVerifyConnectionTimeoutInMillis();

	/**
	 * Defines the socket timeout (SO_TIMEOUT) in milliseconds, which is the
	 * timeout for waiting for data. A timeout value of zero is interpreted as
	 * an infinite timeout.
	 */
	int getVerifySocketTimeoutInMillis();

	/** e.g. "https://api-verify.recaptcha.net/verify" */
	String getVerifyUrl();

	/**
	 * True if all traffic should be secured with HTTPS. 
	 * @return
	 */
	boolean isSecure();
}