/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.apache.wicket.contrib.recaptcha;

public class ReCaptchaConfigBuilder {

	private int connectionTimeout;
	private String privateKey;
	private String publicKey;
	private boolean secure;
	private int socketTimeout;

	public ReCaptchaConfigBuilder() {
		init();
	}

	public ReCaptchaConfig build() {
		final String pubKey = publicKey;
		final String privKey = privateKey;
		final int connTimeout = connectionTimeout;
		final int sTimeout = socketTimeout;
		init();
		return new DefaultReCaptchaConfig(secure) {

			private static final long serialVersionUID = -4104774441866713609L;

			@Override
			public String getPrivateKey() {
				return privKey;
			}

			@Override
			public String getPublicKey() {
				return pubKey;
			}

			@Override
			public int getVerifyConnectionTimeoutInMillis() {
				return connTimeout;
			}

			@Override
			public int getVerifySocketTimeoutInMillis() {
				return sTimeout;
			}
		};

	}

	private void init() {
		publicKey = null;
		privateKey = null;
		connectionTimeout = 8000;
		socketTimeout = 8000;
	}

	public ReCaptchaConfigBuilder setPrivateKey(String privateKey) {
		this.privateKey = privateKey;
		return this;
	}

	public ReCaptchaConfigBuilder setPublicKey(String publicKey) {
		this.publicKey = publicKey;
		return this;
	}

	public ReCaptchaConfigBuilder setSecure(boolean secure) {
		this.secure = secure;
		return this;
	}

	public ReCaptchaConfigBuilder setVerifyConnectionTimeoutInMillis(
			int connectionTimeout) {
		this.connectionTimeout = connectionTimeout;
		return this;
	}

	public ReCaptchaConfigBuilder setVerifySocketTimeoutInMillis(
			int socketTimeout) {
		this.socketTimeout = socketTimeout;
		return this;
	}
}
