/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.webgui;

import org.apache.wicket.Request;
import org.apache.wicket.authentication.AuthenticatedWebSession;
import org.apache.wicket.authorization.strategies.role.Roles;
import org.apache.wicket.injection.web.InjectorHolder;
import org.jhears.user.UserInformation;
import org.jhears.user.UserInformationCentral;

import com.google.inject.Inject;

public class WebguiAuthenticatedWebSession extends AuthenticatedWebSession {

	private static final long serialVersionUID = 1L;

	public WebguiAuthenticatedWebSession(Request request) {
		super(request);
		username = null;
		// logger =
		// LoggerFactory.getLogger(WebguiAuthenticatedWebSession.class);
	}

	public boolean hasAnyRole(Roles roles) {
		return this.roles != null && roles != null
				&& this.roles.hasAllRoles(roles);
	}

	public void signIn(String username, Roles roles) {
		this.username = username;
		this.roles = roles;
		super.signIn(true);
	}

	@Inject
	private transient UserInformationCentral central;

	private Roles roles;

	@Override
	public boolean authenticate(String username, String password) {
		InjectorHolder.getInjector().inject(this);
		UserInformation info = central.getUserInformation(username);
		if (info != null) {
			if (central.authenticate(info, password)) {
				this.username = username;
				this.roles = new Roles(info.getRoles().toArray(new String[0]));
				return true;
			}
		}
		return false;
	}

	public String getUsername() {
		return username;
	}

	private String username;

	@Override
	public Roles getRoles() {
		if (isSignedIn()) {
			return roles;
		}
		return null;
	}

	public void authenticate(String name) {
		signIn(true);
		this.username = name;
	}

}
