/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.webgui;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.wicket.contrib.recaptcha.ReCaptchaConfig;
import org.apache.wicket.contrib.recaptcha.ReCaptchaConfigBuilder;
import org.apache.wicket.protocol.http.WebApplication;
import org.jhears.auth.UserInformationCentralImpl;
import org.jhears.cnf.Cnf;
import org.jhears.cnf.CnfManager;
import org.jhears.cnf.Cnf.ServerKey;
import org.jhears.data.DAO;
import org.jhears.data.GeneralDao;
import org.jhears.data.IGenericDao;
import org.picocontainer.MutablePicoContainer;

import com.google.common.base.Throwables;
import com.google.inject.AbstractModule;

public class WebguiModule extends AbstractModule {
	
	private final MutablePicoContainer container;

	public WebguiModule(MutablePicoContainer container) {
		super();
		this.container = container;
	}
	
	@Override
	protected void configure() {
		bind(WebApplication.class).to(WicketApplication.class);
		GeneralDao generalDao = new GeneralDao(container.getComponent(DAO.class));
		bind(IGenericDao.class).toInstance(generalDao);
		AccountManager accountManager = new AccountManager(generalDao);
		CnfManager mgr = container.getComponent(CnfManager.class);
		Configuration configuration;
		try {
			configuration = mgr.getConfiguration(Cnf.server);
		} catch (ConfigurationException e) {
			throw Throwables.propagate(e);
		}
		boolean secureRecaptcha = configuration.getBoolean(ServerKey.RECAPTCHA_SECURE, false);
		String pubKey = configuration.getString(ServerKey.RECAPTCHA_PUBLIC_KEY, null);
		if (pubKey != null && pubKey.length() < 2) {
			pubKey = null;
		}
		String privKey = configuration.getString(ServerKey.RECAPTCHA_PRIVATE_KEY, null);
		int connectionTimeout = configuration.getInt(ServerKey.RECAPTCHA_CONNECTION_TIMEOUT, -1);
		int socketTimeout = configuration.getInt(ServerKey.RECAPTCHA_SOCKET_TIMEOUT, -1);
		ReCaptchaConfigBuilder b = new ReCaptchaConfigBuilder();
		b.setSecure(secureRecaptcha);
		b.setPrivateKey(privKey);
		b.setPublicKey(pubKey);
		b.setVerifyConnectionTimeoutInMillis(connectionTimeout);
		b.setVerifySocketTimeoutInMillis(socketTimeout);
		ReCaptchaConfig	reCaptchaConfig = b.build();
		bind(ReCaptchaConfig.class).toInstance(reCaptchaConfig);
		bind(IAccountManager.class).toInstance(accountManager);
	}

}
