/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.webgui;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.wicket.Session;
import org.apache.wicket.authentication.AuthenticatedWebApplication;
import org.apache.wicket.authentication.AuthenticatedWebSession;
import org.apache.wicket.authorization.strategies.role.RoleAuthorizationStrategy;
import org.apache.wicket.authorization.strategies.role.Roles;
import org.apache.wicket.guice.GuiceComponentInjector;
import org.apache.wicket.injection.web.InjectorHolder;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.protocol.http.WebRequest;
import org.jhears.auth.JHearsRoles;
import org.jhears.data.IGenericDao;
import org.jhears.data.RoleDataObj;
import org.jhears.data.UserDataObj;
import org.jhears.user.UserInformation;
import org.jhears.util.SingletonMutablePicoContainer;
import org.jhears.webgui.pages.HomePage;
import org.jhears.webgui.pages.login.LoginPage;
import org.jhears.webgui.pages.login.LogoutPage;
import org.jhears.webgui.pages.register.RegisterPage;
import org.jhears.webgui.util.UserRolesAuthorizer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.inject.Inject;

/**
 * Application object for your web application. If you want to run this
 * application without deploying, run the Start class.
 * 
 * @see org.oma.wicket.Start#main(String[])
 */
public class WicketApplication extends AuthenticatedWebApplication {

	public static final String PASSWORD_SALT = "Kth6D27KQoRYzTfT7fl8HcNDZJFVslfX";

	@Inject
	private IAccountManager accountManager;

	private final Logger logger = LoggerFactory
			.getLogger(WicketApplication.class);

	private String loginRedirectFailure;

	private String loginRedirectSuccess;

	private String loginServlet;

	private String logoutRedirect;

	public WicketApplication() {
		super();
	}

	/**
	 * @see org.apache.wicket.Application#getHomePage()
	 */
	@Override
	public Class<HomePage> getHomePage() {
		return HomePage.class;
	}

	public String getLoginRedirectFailure() {
		return loginRedirectFailure;
	}

	public String getLoginRedirectSuccess() {
		return loginRedirectSuccess;
	}

	public String getLoginServlet() {
		return loginServlet;
	}

	public String getLogoutRedirect() {
		return logoutRedirect;
	}

	// @Override
	// protected WebRequest newWebRequest(final HttpServletRequest
	// servletRequest) {
	// final WebRequest webRequest = super.newWebRequest(servletRequest);
	// final Session session = getSessionStore().lookup(webRequest);
	// if (session != null) {
	// WebguiAuthenticatedWebSession ses = (WebguiAuthenticatedWebSession)
	// session;
	// if (!ses.isSignedIn() && servletRequest.getUserPrincipal() != null) {
	// String username = servletRequest.getUserPrincipal().getName();
	// ses.authenticate(username);
	// logger.info("user " + username + " logged in");
	// }
	// }
	// return webRequest;
	// }

	@Override
	protected Class<? extends WebPage> getSignInPageClass() {
		return LoginPage.class;
	}

	@Override
	protected Class<? extends AuthenticatedWebSession> getWebSessionClass() {
		return WebguiAuthenticatedWebSession.class;
	}

	@Override
	protected void init() {
		super.init();
		loginRedirectSuccess = getInitParameter("loginRedirectOnSuccess");
		loginRedirectFailure = getInitParameter("loginRedirectOnFailure");
		logoutRedirect = getInitParameter("logoutRedirect");
		loginServlet = getInitParameter("loginServlet");
		getDebugSettings().setDevelopmentUtilitiesEnabled(true);
		addComponentInstantiationListener(new GuiceComponentInjector(this,
				new WebguiModule(SingletonMutablePicoContainer.getInstance())));
		getApplicationSettings().setAccessDeniedPage(LoginPage.class);
		mountBookmarkablePage("/login", LoginPage.class);
		mountBookmarkablePage("/register", RegisterPage.class);
		mountBookmarkablePage("/logout", LogoutPage.class);
		InjectorHolder.getInjector().inject(this);
		IGenericDao dao = accountManager.getGenericDao();
		if (dao.count(UserDataObj.class) == 0) {
			String name = "root";
			String password = "root";
			logger.info("Creating a default admin account. name=" + name
					+ ", password=" + password);

			RoleDataObj userRole = dao.get(RoleDataObj.class, "name",
					JHearsRoles.USER);
			if (userRole == null) {
				userRole = new RoleDataObj();
				userRole.setName(JHearsRoles.USER);
				dao.save(userRole);
			}
			RoleDataObj adminRole = dao.get(RoleDataObj.class, "name",
					JHearsRoles.ADMIN);
			if (adminRole == null) {
				adminRole = new RoleDataObj();
				adminRole.setName(JHearsRoles.ADMIN);
				dao.save(adminRole);
			}

			String[] roles = new String[] { JHearsRoles.USER, JHearsRoles.ADMIN };
			accountManager.createAccount(name, password, roles);
		}
		getSecuritySettings().setAuthorizationStrategy(
				new RoleAuthorizationStrategy(new UserRolesAuthorizer()));
	}

	@Override
	protected WebRequest newWebRequest(final HttpServletRequest servletRequest) {
		final WebRequest webRequest = super.newWebRequest(servletRequest);
		final Session session = getSessionStore().lookup(webRequest);
		if (session != null) {
			WebguiAuthenticatedWebSession ses = (WebguiAuthenticatedWebSession) session;
			HttpSession httpSession = servletRequest.getSession(false);
			if (!ses.isSignedIn()
					&& httpSession != null
					&& httpSession.getAttribute("jhears-user-information") != null) {
				UserInformation info = (UserInformation) httpSession
						.getAttribute("jhears-user-information");
				Roles roles = new Roles(info.getRoles().toArray(
						new String[info.getRoles().size()]));
				ses.signIn(info.getUsername(), roles);
				// logger.info("user " + info.getUsername() + " logged in");
			}
		}
		return webRequest;
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

}
