/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.webgui.pages.admin;

import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.PropertyModel;
import org.jhears.data.IGenericDao;
import org.jhears.data.UserDataObj;
import org.jhears.webgui.util.EntityModel;

import com.google.inject.Inject;

public class EditPersonPage extends AdminTemplatePage {

	private final PasswordTextField confirmPassword;

	private final PasswordTextField password;

	public EditPersonPage(UserDataObj person, final WebPage previous) {
		super();
		confirmPassword = new PasswordTextField("confirmPassword",
				new PropertyModel<String>(this, "confirmPassword"));
		confirmPassword.setRequired(false);
		password = new PasswordTextField("password", new PropertyModel<String>(
				this, "password"));
		password.setRequired(false);
		setPageTitle("Edit person");
		EntityModel<UserDataObj> modelnew = new EntityModel<UserDataObj>(person);
		final CompoundPropertyModel<UserDataObj> m = new CompoundPropertyModel<UserDataObj>(
				modelnew);
		Form<UserDataObj> form = new Form<UserDataObj>("editForm", m);
		form
				.add(new CheckBox("locked", new PropertyModel<Boolean>(m,
						"locked")));
		form.add(new Label("name", new PropertyModel<String>(m, "name")));
		String created = person.getProperties().get(UserDataObj.CREATED_TIMESTAMP);
		form.add(new Label("created", created));
		form.add(password);
		form.add(confirmPassword);

		Button deleteButton = new Button("save") {

			private static final long serialVersionUID = 1L;

			@Inject
			private IGenericDao dao;

			@Override
			public void onSubmit() {
				// InjectorHolder.getInjector().inject(this);
				dao.update(m.getObject());
				setResponsePage(previous);
			}
		};
		form.add(deleteButton);

		Button cancelButton = new Button("cancel") {

			private static final long serialVersionUID = 1L;

			@Override
			public void onSubmit() {
				setResponsePage(previous);
			}
		};
		form.add(cancelButton);
		add(form);
	}
}