/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.webgui.pages.admin;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.Component;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.jhears.data.IGenericDao;
import org.jhears.data.UserDataObj;
import org.jhears.webgui.util.GenericDataProvider;
import org.jhears.webgui.util.SortableGenericDataProvider;

import com.google.inject.Inject;

public class ListPersonsPage extends AdminTemplatePage {

	static abstract class ActionPanel<T> extends Panel {

		private static final long serialVersionUID = 1L;

		public ActionPanel(String id, IModel<T> model, Component component) {
			super(id, model);
			add(component);
		}
	}

	static class DeleteActionPanel extends ActionPanel<UserDataObj> {

		private static final long serialVersionUID = 1L;

		public DeleteActionPanel(String id, IModel<UserDataObj> model,
				final WebPage page) {
			super(id, model, new Link<UserDataObj>("deletePerson", model) {

				private static final long serialVersionUID = 1L;

				@Override
				public void onClick() {
					UserDataObj p = getModelObject();
					setResponsePage(new DeletePersonPage(p, page));
				}
			});
		}

	}

	static class EditActionPanel extends ActionPanel<UserDataObj> {

		private static final long serialVersionUID = 1L;

		public EditActionPanel(String id, IModel<UserDataObj> model,
				final WebPage page) {
			super(id, model, new Link<UserDataObj>("editPerson", model) {

				private static final long serialVersionUID = 1L;

				@Override
				public void onClick() {
					UserDataObj p = getModelObject();
					setResponsePage(new EditPersonPage(p, page));
				}
			});
		}

	}

	private String searchExpression;

	private static final long serialVersionUID = 1L;

	public ListPersonsPage() {
		super();

		searchExpression = "";

		setPageTitle("List persons page");

		Form<UserDataObj> form = new Form<UserDataObj>("searchForm");
		form.add(new Link<AddPersonPage>("addPerson") {

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick() {
				setResponsePage(new AddPersonPage(ListPersonsPage.this));
			}

		});

		final TextField<String> searchExpressionTextField = new TextField<String>(
				"searchExpression", new Model<String>(searchExpression));
		form.add(searchExpressionTextField);
		Button search = new Button("search") {

			private static final long serialVersionUID = 1L;

			@Inject
			private IGenericDao dao;

			@Override
			public void onSubmit() {
				searchExpression = searchExpressionTextField.getModelObject();
			}
		};
		form.add(search);

		SortableGenericDataProvider<UserDataObj> provider = new SortableGenericDataProvider<UserDataObj>(
				new GenericDataProvider<UserDataObj>(UserDataObj.class));

		List<IColumn<?>> columns = new ArrayList<IColumn<?>>(2);

		columns.add(new PropertyColumn<UserDataObj>(new Model<String>("Name"),
				"name"));

		columns.add(new AbstractColumn<UserDataObj>(new Model<String>("edit")) {

			private static final long serialVersionUID = 1L;

			public void populateItem(Item<ICellPopulator<UserDataObj>> cellItem,
					String componentId, IModel<UserDataObj> model) {
				cellItem.add(new EditActionPanel(componentId, model,
						ListPersonsPage.this));
			}
		});

		columns.add(new AbstractColumn<UserDataObj>(new Model<String>(null)) {

			private static final long serialVersionUID = 1L;

			public void populateItem(Item<ICellPopulator<UserDataObj>> cellItem,
					String componentId, IModel<UserDataObj> model) {
				cellItem.add(new DeleteActionPanel(componentId, model,
						ListPersonsPage.this));
			}
		});
		DefaultDataTable tbl = new DefaultDataTable("personTable", columns,
				provider, 8);

		add(form);
		add(tbl);
	}
}
