/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.webgui.pages.login;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.commons.collections.iterators.IteratorEnumeration;

/**
 * Allows to override/modify HTTP request parameters.
 * 
 * @author Juha Heljoranta
 * 
 */
public class HttpServletRequestParamble extends HttpServletRequestWrapper {

	private Map<String, String[]> params;

	@SuppressWarnings("unchecked")
	public HttpServletRequestParamble(HttpServletRequest request) {
		super(request);
		params = new HashMap<String, String[]>(request.getParameterMap());
	}

	public void addParam(String name, String value) {
		String[] values = params.get(name);
		if (values == null) {
			params.put(name, new String[] { value });
		} else {
			values = Arrays.copyOf(values, values.length + 1);
			values[values.length - 1] = value;
			params.put(name, values);
		}
	}

	@Override
	public String getParameter(String name) {
		String[] arr = params.get(name);
		if (arr != null && arr.length > 0) {
			return arr[0];
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	@Override
	public Map getParameterMap() {
		return params;
	}

	@SuppressWarnings("unchecked")
	@Override
	public Enumeration getParameterNames() {
		return new IteratorEnumeration(params.keySet().iterator());
	}

	@Override
	public String[] getParameterValues(String name) {
		return params.get(name);
	}
}