/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.webgui.pages.login;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.apache.wicket.Application;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.PropertyModel;
import org.jhears.webgui.WebguiAuthenticatedWebSession;
import org.jhears.webgui.WicketApplication;
import org.jhears.webgui.pages.HomePage;
import org.jhears.webgui.pages.TemplatePage;

import com.google.common.base.Throwables;

public final class LoginPage extends TemplatePage {

	private PasswordTextField password;
	private TextField<String> username;
	// private final Logger logger;
	private String passwordValue;
	private String usernameValue;

	public LoginPage() {
		super();

		setPageTitle("Login");
		// this.logger = LoggerFactory.getLogger(LoginPage.class);

		if (((WebguiAuthenticatedWebSession) getSession()).isSignedIn()) {
			setRedirect(true);
			setResponsePage(HomePage.class);
		} else {
			// ((WebguiAuthenticatedWebSession) getSession()).clear();
			// ((WebguiAuthenticatedWebSession) getSession()).replaceSession();
		}

		password = new PasswordTextField("password", new PropertyModel<String>(
				this, "passwordValue"));
		password.setRequired(false);

		username = new TextField<String>("username", new PropertyModel<String>(
				this, "usernameValue"));
		username.setRequired(false);

		Form<String> form = new Form<String>("loginForm");
		form.add(password);
		form.add(username);

		Button loginButton = new Button("login") {

			private static final long serialVersionUID = 1L;

			@Override
			public void onSubmit() {
				HttpServletRequest req = getWebRequestCycle().getWebRequest()
						.getHttpServletRequest();
				RequestDispatcher dispatcher = req
						.getRequestDispatcher(((WicketApplication) Application
								.get()).getLoginServlet());

				HttpServletRequestParamble request = new HttpServletRequestParamble(
						req);
				request.addParam("username", usernameValue);
				request.addParam("password", passwordValue);

				request.addParam("redirectOnSuccess",
						((WicketApplication) Application.get())
								.getLoginRedirectSuccess());
				request.addParam("redirectOnFailure",
						((WicketApplication) Application.get())
								.getLoginRedirectFailure());
				try {
					dispatcher.forward(request, getWebRequestCycle()
							.getWebResponse().getHttpServletResponse());
				} catch (ServletException e) {
					throw Throwables.propagate(e);
				} catch (IOException e) {
					throw Throwables.propagate(e);
				}
				// stop wicket page processing
				getWebRequestCycle().setRequestTarget(null);
			}

		};
		form.add(loginButton);

		add(form);
	}
}
