/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jhears.webgui.pages.register;

import java.io.IOException;
import java.util.regex.Pattern;

import org.apache.wicket.contrib.recaptcha.ReCaptchaConfig;
import org.apache.wicket.contrib.recaptcha.ReCaptchaException;
import org.apache.wicket.contrib.recaptcha.ReCaptchaPanel;
import org.apache.wicket.injection.web.InjectorHolder;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.Model;
import org.jhears.auth.JHearsRoles;
import org.jhears.data.UserDataObj;
import org.jhears.webgui.IAccountManager;
import org.jhears.webgui.WebguiAuthenticatedWebSession;
import org.jhears.webgui.pages.TemplatePage;
import org.jhears.webgui.pages.user.PersonalAccountPage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.inject.Inject;

public class RegisterPage extends TemplatePage {

	private static final Pattern ILLEGAL_USERNAME_PATTERN = Pattern
			.compile(".*[\\p{Cntrl}\\p{Space}]+.*");

	private static final long serialVersionUID = 1L;

	private final Label captchaErrorLabel;

	private final PasswordTextField confirmPassword;

	@Inject
	private transient IAccountManager accountManager;

	private final Form<Object> form;

	private final TextField<String> name;

	private final Label nameErrorLabel;

	private final PasswordTextField password;

	private final Label passwordErrorLabel;

	private final ReCaptchaPanel recapPan;

	@Inject
	private ReCaptchaConfig captchaConfig;

	public RegisterPage() {
		super();
		setPageTitle("Registration");
		nameErrorLabel = new Label("nameError", "Name error");
		passwordErrorLabel = new Label("passwordError", "Password error");
		captchaErrorLabel = new Label("captchaError", "Capatcha error");
		name = new TextField<String>("name", new Model<String>(""));
		password = new PasswordTextField("password", new Model<String>(""));
		confirmPassword = new PasswordTextField("confirmPassword",
				new Model<String>(""));
		password.setRequired(false);
		confirmPassword.setRequired(false);

		form = new Form<Object>("registerForm") {

			private static final long serialVersionUID = 1L;

			@Inject
			private IAccountManager accountManager;

			@Override
			protected void onSubmit() {
				boolean nameErro = isNameError();
				boolean passwordErro = isPasswordError();
				boolean captchaErro = false;
				if (captchaConfig.getPublicKey() != null) {
					captchaErro = !recapPan.isReCaptchaFieldsFilled();
				}
				nameErrorLabel.setVisible(nameErro);
				passwordErrorLabel.setVisible(passwordErro);
				captchaErrorLabel.setVisible(captchaErro);
				boolean allOk = !nameErro && !passwordErro && !captchaErro;
				if (allOk) {
					if (captchaConfig.getPublicKey() != null) {
						try {
							captchaErro = !recapPan.verifyCaptcha();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
							captchaErro = true;
						} catch (ReCaptchaException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
							captchaErro = true;
						}
						allOk = !captchaErro;
						captchaErrorLabel.setVisible(captchaErro);
					}
				}
				if (allOk) {
					InjectorHolder.getInjector().inject(this);
					try {
						accountManager.createAccount(name.getModelObject(),
								password.getModelObject(),
								new String[] { JHearsRoles.USER });
						WebguiAuthenticatedWebSession session = (WebguiAuthenticatedWebSession) getSession();
						session.signIn(name.getModelObject(), JHearsRoles.USER);
						Logger logger = LoggerFactory
								.getLogger(RegisterPage.class);
						logger.info("User registration completed: "
								+ name.getModelObject());
						setRedirect(true);
						setResponsePage(PersonalAccountPage.class);
					} catch (RuntimeException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}

		};

		nameErrorLabel.setVisible(false);
		passwordErrorLabel.setVisible(false);
		captchaErrorLabel.setVisible(false);
		recapPan = new ReCaptchaPanel("reCaptcha", captchaConfig);
		form.add(recapPan);
		recapPan.setVisibilityAllowed(captchaConfig.getPublicKey() != null);
		form.add(password);
		form.add(confirmPassword);
		form.add(name);
		form.add(nameErrorLabel);
		form.add(passwordErrorLabel);
		form.add(captchaErrorLabel);
		add(form);
	}

	private boolean isNameError() {
		if (name.getModelObject() == null) {
			return true;
		}
		String trimmed = name.getModelObject().trim();
		if (!trimmed.equals(name.getModelObject())) {
			return true;
		}
		if (ILLEGAL_USERNAME_PATTERN.matcher(trimmed).matches()) {
			return true;
		}
		if (trimmed.length() < 4) {
			return true;
		}
		return accountManager.getGenericDao().count(UserDataObj.class, "name",
				trimmed) > 0L;
	}

	private boolean isPasswordError() {
		String p = password.getModelObject();
		if (p == null) {
			return true;
		}
		return p.length() < 6 || !p.equals(confirmPassword.getModelObject());
	};

}
