/*
 * jHears, acoustic fingerprinting framework.
 * Copyright (C) 2009-2010  Juha Heljoranta.
 *
 * This file is part of jHears.
 *
 * jHears is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * jHears is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with jHears.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 
 */
package org.jhears.webgui.util;

import java.util.Iterator;
import java.util.List;

import org.apache.wicket.injection.web.InjectorHolder;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.IModel;
import org.jhears.data.IGenericDao;
import org.jhears.util.Identifiable;

import com.google.inject.Inject;

public class GenericDataProvider<T extends Identifiable<?>> implements
		IDataProvider<T> {

	@Inject
	private IGenericDao dao;

	private static final long serialVersionUID = 1L;

	private final Class<T> entityClass;

	public GenericDataProvider(Class<T> entityClass) {
		this.entityClass = entityClass;
		InjectorHolder.getInjector().inject(this);
	}

	@Override
	public Iterator<? extends T> iterator(int first, int count) {
		List<T> list = dao.list(entityClass, first, count);
		return list.iterator();
	}

	public Iterator<? extends T> iterator(int first, int count,
			String property, boolean ascending) {
		List<T> list = dao.list(entityClass, first, count, property, ascending);
		return list.iterator();
	}

	@Override
	public IModel<T> model(T object) {
		return new EntityModel<T>(object);
	}

	@Override
	public int size() {
		return (int) dao.count(entityClass);
	}

	@Override
	public void detach() {
		// nop
	}

}